@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1998--2005  Guido Masarotto and Brian Ripley
- *  Copyright (C) 2004--2007  The R Foundation
+ *  Copyright (C) 2004--2009  The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,16 +34,18 @@
 #include <stdio.h>
 #undef DEBUG /* needed for mingw-runtime 2.0 */
 /* the user menu code looks at the internal structure */
+#define GA_EXTERN
 #include "graphapp/internal.h"
 #include "graphapp/ga.h"
 #ifdef USE_MDI
 # include "graphapp/stdimg.h"
 #endif
+
 #include "console.h"
 #include "rui.h"
 #include "preferences.h"
 #include <Rversion.h>
-#include "getline/getline.h"  /* for gl_load/savehistory */
+#include "getline/wc_history.h"  /* for wgl_load/savehistory */
 #include <Startup.h>          /* for SA_DEFAULT */
 
 #define TRACERUI(a)
@@ -54,17 +56,17 @@ console RConsole = NULL;
 #ifdef USE_MDI
 int   RguiMDI = RW_MDI | RW_TOOLBAR | RW_STATUSBAR;
 int   MDIset = 0;
-int   R_LoadRconsole = 1;
-window RFrame;
+window RFrame = NULL; /* some compilers want initialized for export */
 rect MDIsize;
 #endif
 extern int ConsoleAcceptCmd, R_is_running;
 extern Rboolean DebugMenuitem;
+Rboolean R_LoadRconsole = TRUE; /* used in commandLineArgs */
 
 static menubar RMenuBar;
 static popup RConsolePopup;
 static menuitem msource, mdisplay, mload, msave, mloadhistory,
-    msavehistory, mpaste, mpastecmds, mcopy, mcopypaste, mlazy, mcomplete, 
+    msavehistory, mpaste, mpastecmds, mcopy, mcopypaste, mlazy, mcomplete,
     mfncomplete, mconfig, mls, mrm, msearch, mde, mtools, mstatus;
 static int lmanintro, lmanref, lmandata, lmanlang, lmanext, lmanint, lmanadmin;
 static menu m;
@@ -86,7 +88,6 @@ static void double_backslashes(char *s, char *out)
 {
     char *p = s;
 
-#ifdef SUPPORT_MBCS
     int i;
     if(mbcslocale) {
 	mbstate_t mb_st; int used;
@@ -96,12 +97,8 @@ static void double_backslashes(char *s, char *out)
 	    for(i = 0; i < used; i++) *out++ = *p++;
 	}
     } else
-#endif
-    for (; *p; p++)
-	if (*p == '\\') {
-	    *out++ = *p;
-	    *out++ = *p;
-	} else *out++ = *p;
+	for (; *p; p++)
+	    if (*p == '\\') {*out++ = *p; *out++ = *p;} else *out++ = *p;
     *out = '\0';
 }
 
@@ -116,16 +113,38 @@ void closeconsole(control m)  /* can also be called from editor menus */
     R_CleanUp(SA_DEFAULT, 0, 1);
 }
 
+static void quote_fn(wchar_t *fn, char *s)
+{
+    char *p = s;
+    wchar_t *w;
+    int used;
+    for (w = fn; *w; w++) {
+	if(*w  == L'\\') {
+	    *p++ = '\\';
+	    *p++ = '\\';
+	} else {
+	    used = wctomb(p, *w);
+	    if(used > 0) p += used;
+	    else {
+		sprintf(p, "\\u%04x", (unsigned int) *w);
+		p += 6;
+	    }
+	}
+    }
+    *p = '\0';
+}
+
+
 static void menusource(control m)
 {
-    char *fn, local[MAX_PATH];
+    wchar_t *fn;
+    char local[MAX_PATH];
 
     if (!ConsoleAcceptCmd) return;
-    setuserfilter("R files (*.R)\0*.R\0S files (*.q, *.ssc, *.S)\0*.q;*.ssc;*.S\0All files (*.*)\0*.*\0\0");
-    fn = askfilename(G_("Select file to source"), "");
-/*    show(RConsole); */
+    setuserfilterW(L"R files (*.R)\0*.R\0S files (*.q, *.ssc, *.S)\0*.q;*.ssc;*.S\0All files (*.*)\0*.*\0\0");
+    fn = askfilenameW(G_("Select file to source"), "");
     if (fn) {
-	double_backslashes(fn, local);
+	quote_fn(fn, local);
 	snprintf(cmd, 1024, "source(\"%s\")", local);
 	consolecmd(RConsole, cmd);
     }
@@ -139,14 +158,14 @@ static void menudisplay(control m)
 
 static void menuloadimage(control m)
 {
-    char *fn, s[2*MAX_PATH];
+    wchar_t *fn;
+    char s[MAX_PATH];
 
     if (!ConsoleAcceptCmd) return;
-    setuserfilter("R images (*.RData)\0*.RData\0R images - old extension (*.rda)\0*.rda\0All files (*.*)\0*.*\0\0");
-    fn = askfilename(G_("Select image to load"), "");
-/*    show(RConsole); */
+    setuserfilterW(L"R images (*.RData)\0*.RData\0R images - old extension (*.rda)\0*.rda\0All files (*.*)\0*.*\0\0");
+    fn = askfilenameW(G_("Select image to load"), "");
     if (fn) {
-	double_backslashes(fn, s);
+	quote_fn(fn, s);
 	snprintf(cmd, 1024, "load(\"%s\")", s);
 	consolecmd(RConsole, cmd);
     }
@@ -154,14 +173,14 @@ static void menuloadimage(control m)
 
 static void menusaveimage(control m)
 {
-    char *fn, s[2*MAX_PATH];
+    wchar_t *fn;
+    char s[MAX_PATH];
 
     if (!ConsoleAcceptCmd) return;
-    setuserfilter("R images (*.RData)\0*.RData\0All files (*.*)\0*.*\0\0");
-    fn = askfilesave(G_("Save image in"), ".RData");
-/*    show(RConsole); */
+    setuserfilterW(L"R images (*.RData)\0*.RData\0All files (*.*)\0*.*\0\0");
+    fn = askfilesaveW(G_("Save image in"), ".RData");
     if (fn) {
-	double_backslashes(fn, s);
+	quote_fn(fn, s);
 	if (!strcmp(&s[strlen(s) - 2], ".*")) s[strlen(s) - 2] = '\0';
 	snprintf(cmd, 1024, "save.image(\"%s\")", s);
 	consolecmd(RConsole, cmd);
@@ -174,7 +193,7 @@ static void menuloadhistory(control m)
 
     setuserfilter("All files (*.*)\0*.*\0\0");
     fn = askfilename(G_("Load history from"), R_HistoryFile);
-    if (fn) gl_loadhistory(fn);
+    if (fn) wgl_loadhistory(fn);
 }
 
 static void menusavehistory(control m)
@@ -185,26 +204,23 @@ static void menusavehistory(control m)
     s = askfilesave(G_("Save history in"), R_HistoryFile);
     if (s) {
 	R_setupHistory(); /* re-read the history size */
-	gl_savehistory(s, R_HistorySize);
+	wgl_savehistory(s, R_HistorySize);
     }
 }
 
 static void menuchangedir(control m)
 {
     askchangedir();
-/*    show(RConsole); */
 }
 
 static void menuprint(control m)
 {
     consoleprint(RConsole);
-/*    show(RConsole); */
 }
 
 static void menusavefile(control m)
 {
     consolesavefile(RConsole, 0);
-/*    show(RConsole); */
 }
 
 static void menuexit(control m)
@@ -310,7 +326,7 @@ void menuconfig(control m)
 /*    show(RConsole); */
 }
 
-static void menutools(control m) 
+static void menutools(control m)
 {
     if(ischecked(mtools)) {
 	toolbar_hide();
@@ -348,24 +364,24 @@ static void menulazy(control m)
 /*    show(RConsole); */
 }
 
-extern void set_rcompgen_available(int x);
+extern void set_completion_available(int x);
 
 static int filename_completion_on = 1;
 
 static int check_file_completion(void)
 {
-    /* ought really to ask rcompgen, but that means loading it */
+    /* ought really to ask utils */
     return filename_completion_on;
 }
 
 static void menucomplete(control m)
 {
     if(ischecked(mcomplete)) {
-	set_rcompgen_available(0);
+	set_completion_available(0);
 	uncheck(mcomplete);
 	uncheck(mfncomplete);
     } else {
-	set_rcompgen_available(-1);
+	set_completion_available(-1);
 	check(mcomplete);
 	if(check_file_completion()) check(mfncomplete);
 	else uncheck(mfncomplete);
@@ -380,13 +396,13 @@ static void menufncomplete(control m)
 	uncheck(mfncomplete);
 	filename_completion_on = 0;
     } else {
-	c0 = "TRUE";	
+	c0 = "TRUE";
 	check(mfncomplete);
 	filename_completion_on = 1;
     }
-    sprintf(cmd, "rcompgen::rc.settings(files=%s)", c0);
+    sprintf(cmd, "utils::rc.settings(files=%s)", c0);
     consolecmd(RConsole, cmd);
-    
+
 }
 
 static void menuconsolestayontop(control m)
@@ -405,18 +421,18 @@ static void menukillall(control m)
     Rf_jump_to_toplevel();
 }
 
-static Rboolean isdebuggerpresent()
+static Rboolean isdebuggerpresent(void)
 {
-    typedef BOOL (*R_CheckDebugger)();
+    typedef BOOL (*R_CheckDebugger)(void);
     R_CheckDebugger entry;
-    entry = 
+    entry =
 	(R_CheckDebugger) GetProcAddress((HMODULE)GetModuleHandle("KERNEL32"),
 					 "IsDebuggerPresent");
     if (entry == NULL) return(FALSE);
     else return (Rboolean) entry();
 }
 
-void breaktodebugger()
+void breaktodebugger(void)
 {
     asm("int $3");
 }
@@ -473,7 +489,7 @@ static void menupkgupdatebioc(control m)
 }
 
 
-static void menupkginstallbioc(control m) 
+static void menupkginstallbioc(control m)
 {
     if (!ConsoleAcceptCmd) return;
     consolecmd(RConsole, "utils:::menuInstallBioc()");
@@ -504,6 +520,37 @@ static void menupkginstalllocal(control m)
     if (!ConsoleAcceptCmd) return;
     consolecmd(RConsole, "utils:::menuInstallLocal()");
 }
+ 
+ 
+static void menucascade(control m)
+{
+    if (!ConsoleAcceptCmd) return;
+    consolecmd(RConsole, "utils::arrangeWindows(action='cascade')");
+}
+
+static void menutilehoriz(control m)
+{
+    if (!ConsoleAcceptCmd) return;
+    consolecmd(RConsole, "utils::arrangeWindows(action='horizontal')");
+}
+
+static void menutilevert(control m)
+{
+    if (!ConsoleAcceptCmd) return;
+    consolecmd(RConsole, "utils::arrangeWindows(action='vertical')");
+}
+
+static void menuminimizegroup(control m)
+{
+    if (!ConsoleAcceptCmd) return;
+    consolecmd(RConsole, "utils::arrangeWindows(action='minimize')");
+}
+
+static void menurestoregroup(control m)
+{
+    if (!ConsoleAcceptCmd) return;
+    consolecmd(RConsole, "utils::arrangeWindows(action='restore')");
+}
 
 static void menuconsolehelp(control m)
 {
@@ -610,10 +657,10 @@ static void menuapropos(control m)
 
 static void menuhelpstart(control m)
 {
-/*    if (!ConsoleAcceptCmd) return;
+    if (!ConsoleAcceptCmd) return;
     consolecmd(RConsole, "help.start()");
-    show(RConsole);*/
-    internal_shellexec("doc\\html\\index.html");
+/*    show(RConsole); 
+    internal_shellexec("doc\\html\\index.html"); */
 }
 
 static void menuFAQ(control m)
@@ -630,7 +677,7 @@ static void menuabout(control m)
 {
     char  s[256], s2[256];
 
-    
+
     PrintVersionString(s2);
     sprintf(s, "%s\n%s %s %s",
 	    s2,
@@ -657,16 +704,18 @@ void helpmenuact(HelpMenuItems hmenu)
 {
     if (ConsoleAcceptCmd) {
 	enable(hmenu->mhelp);
+	enable(hmenu->mhelpstart);
 	enable(hmenu->mhelpsearch);
 	enable(hmenu->msearchRsite);
-	enable(hmenu->mapropos);    
+	enable(hmenu->mapropos);
 	enable(hmenu->mCRAN);
     } else {
 	disable(hmenu->mhelp);
+	disable(hmenu->mhelpstart);
 	disable(hmenu->mhelpsearch);
 	disable(hmenu->msearchRsite);
-	disable(hmenu->mapropos);    
-        disable(hmenu->mCRAN);
+	disable(hmenu->mapropos);
+	disable(hmenu->mCRAN);
     }
 }
 
@@ -729,7 +778,7 @@ static void menuact(control m)
 	disable(mpaste);
 	disable(mpastecmds);
     }
-    
+
     helpmenuact(hmenu);
     pkgmenuact(pmenu);
 
@@ -743,38 +792,11 @@ void readconsolecfg()
     char  fn[128];
     int   sty = Plain;
     char  optf[PATH_MAX];
-    
+
     struct structGUI gui;
 
-    gui.crows = 25;
-    gui.ccols = 80;
-    gui.cx = gui.cy = 0;
-    gui.grx = Rwin_graphicsx;
-    gui.gry = Rwin_graphicsy;
-    gui.bg = White;
-    gui.fg = DarkBlue;
-    gui.user = gaRed;
-    gui.hlt = DarkRed;
-    gui.prows = 25;
-    gui.pcols = 80;
-    gui.pagerMultiple = 0;
-    gui.cbb = 65000;
-    gui.cbl = 8000;
-    gui.setWidthOnResize = 1;
-    strcpy(gui.font, "Courier New");
-    strcpy(gui.style, "normal");
-    gui.tt_font = 1;
-    gui.pointsize = 10;
-    strcpy(gui.language, "");
-    gui.buffered = 1;
-    
-#ifdef USE_MDI
-    gui.toolbar = ((RguiMDI & RW_TOOLBAR) != 0);
-    gui.statusbar = ((RguiMDI & RW_STATUSBAR) != 0);
-    gui.MDI = ((RguiMDI & RW_MDI) != 0);
-    
-    gui.MDIsize = rect(0, 0, 0, 0);
-#endif
+    getDefaults(&gui);
+
     if (R_LoadRconsole) {
 	sprintf(optf, "%s/Rconsole", getenv("R_USER"));
 	if (!loadRconsole(&gui, optf)) {
@@ -786,28 +808,28 @@ void readconsolecfg()
 	    }
 	}
     }
-    if (gui.tt_font) { 
-    	strcpy(fn, "TT ");
-    	strcpy(fn+3, gui.font);
+    if (gui.tt_font) {
+	strcpy(fn, "TT ");
+	strcpy(fn+3, gui.font);
     } else strcpy(fn, gui.font);
-    
+
     MDIsize = gui.MDIsize;
-    
+
     if (gui.MDI)  RguiMDI |= RW_MDI;
     else          RguiMDI &= ~RW_MDI;
 
     if (MDIset == 1)  RguiMDI |= RW_MDI;
-    if (MDIset == -1) RguiMDI &= ~RW_MDI;    
-        
+    if (MDIset == -1) RguiMDI &= ~RW_MDI;
+
     if (gui.toolbar) RguiMDI |= RW_TOOLBAR;
     else	     RguiMDI &= ~RW_TOOLBAR;
     if (gui.statusbar) RguiMDI |= RW_STATUSBAR;
     else	       RguiMDI &= ~RW_STATUSBAR;
-    
+
     if (!strcmp(gui.style, "normal")) sty = Plain;
     if (!strcmp(gui.style, "bold")) sty = Bold;
     if (!strcmp(gui.style, "italic")) sty = Italic;
-    
+
     Rwin_graphicsx = gui.grx;
     Rwin_graphicsy = gui.gry;
 
@@ -818,7 +840,7 @@ void readconsolecfg()
     }
     setconsoleoptions(fn, sty, gui.pointsize, gui.crows, gui.ccols,
 		      gui.cx, gui.cy,
-		      gui.fg, gui.user, gui.bg, gui.hlt,
+		      gui.guiColors,
 		      gui.prows, gui.pcols, gui.pagerMultiple, gui.setWidthOnResize,
 		      gui.cbb, gui.cbl, gui.buffered);
 }
@@ -886,7 +908,7 @@ static void popupact(control m)
 	disable(ConsolePopup[2].m);
     }
     if (ismdi())
-    	disable(ConsolePopup[10].m);
+	disable(ConsolePopup[10].m);
     else {
 	if (isTopmost(RConsole))
 	    check(ConsolePopup[10].m);
@@ -921,7 +943,7 @@ int RguiPackageMenu(PkgMenuItems pmenu)
     return 0;
 }
 
-static void CheckForManuals()
+static void CheckForManuals(void)
 {
     lmanintro = check_doc_file("doc\\manual\\R-intro.pdf");
     lmanref = check_doc_file("doc\\manual\\refman.pdf");
@@ -932,7 +954,7 @@ static void CheckForManuals()
     lmanadmin = check_doc_file("doc\\manual\\R-admin.pdf");
 }
 
-/* Help functions common to all R windows. 
+/* Help functions common to all R windows.
    These should be appended to each context-specific help menu */
 
 int RguiCommonHelp(menu m, HelpMenuItems hmenu)
@@ -945,35 +967,35 @@ int RguiCommonHelp(menu m, HelpMenuItems hmenu)
     if (!check_doc_file("doc\\html\\rw-FAQ.html")) disable(hmenu->mrwFAQ);
 
 
-    if (!lmanintro && !lmanref && !lmandata && !lmanlang && !lmanext 
+    if (!lmanintro && !lmanref && !lmandata && !lmanlang && !lmanext
        && !lmanint && !lmanadmin) {
 	MCHECK(hmenu->mman0 = newmenuitem(G_("Manuals (in PDF)"), 0, NULL));
 	disable(hmenu->mman0);
     } else {
 	MCHECK(hmenu->mman = newsubmenu(m, G_("Manuals (in PDF)")));
-	MCHECK(hmenu->mmanintro = newmenuitem("An &Introduction to R", 0, 
+	MCHECK(hmenu->mmanintro = newmenuitem("An &Introduction to R", 0,
 				       menumainman));
 	if (!lmanintro) disable(hmenu->mmanintro);
-	MCHECK(hmenu->mmanref = newmenuitem("R &Reference Manual", 0, 
+	MCHECK(hmenu->mmanref = newmenuitem("R &Reference Manual", 0,
 				     menumainref));
 	if (!lmanref) disable(hmenu->mmanref);
-	MCHECK(hmenu->mmandata = newmenuitem("R Data Import/Export", 0, 
+	MCHECK(hmenu->mmandata = newmenuitem("R Data Import/Export", 0,
 				      menumaindata));
 	if (!lmandata) disable(hmenu->mmandata);
-	MCHECK(hmenu->mmanlang = newmenuitem("R Language Definition", 0, 
+	MCHECK(hmenu->mmanlang = newmenuitem("R Language Definition", 0,
 				      menumainlang));
 	if (!lmanlang) disable(hmenu->mmanlang);
-	MCHECK(hmenu->mmanext = newmenuitem("Writing R Extensions", 0, 
+	MCHECK(hmenu->mmanext = newmenuitem("Writing R Extensions", 0,
 				     menumainext));
 	if (!lmanext) disable(hmenu->mmanext);
-	MCHECK(hmenu->mmanint = newmenuitem("R Internals", 0, 
+	MCHECK(hmenu->mmanint = newmenuitem("R Internals", 0,
 				     menumainint));
 	if (!lmanint) disable(hmenu->mmanint);
-	MCHECK(hmenu->mmanadmin = newmenuitem("R Installation and Administration", 0, 
-				       menumainadmin));	
+	MCHECK(hmenu->mmanadmin = newmenuitem("R Installation and Administration", 0,
+				       menumainadmin));
 	if (!lmanadmin) disable(hmenu->mmanadmin);
     }
-    
+
 
     addto(m);
     MCHECK(newmenuitem("-", 0, NULL));
@@ -981,9 +1003,9 @@ int RguiCommonHelp(menu m, HelpMenuItems hmenu)
 				      menuhelp));
     MCHECK(hmenu->mhelpstart = newmenuitem(G_("Html help"), 0, menuhelpstart));
     if (!check_doc_file("doc\\html\\index.html")) disable(hmenu->mhelpstart);
-    MCHECK(hmenu->mhelpsearch = newmenuitem(G_("Search help..."), 0, 
+    MCHECK(hmenu->mhelpsearch = newmenuitem(G_("Search help..."), 0,
 					    menuhelpsearch));
-    MCHECK(hmenu->msearchRsite = newmenuitem("search.r-project.org ...", 0, 
+    MCHECK(hmenu->msearchRsite = newmenuitem("search.r-project.org ...", 0,
 					     menusearchRsite));
     MCHECK(newmenuitem("-", 0, NULL));
     MCHECK(hmenu->mapropos = newmenuitem(G_("Apropos..."), 0, menuapropos));
@@ -995,16 +1017,34 @@ int RguiCommonHelp(menu m, HelpMenuItems hmenu)
     return 0;
 }
 
+static int RguiWindowMenu()
+{
+#ifdef USE_MDI
+    if (ismdi())
+	newmdimenu();
+    else 
+#endif
+    {
+	MCHECK(newmenu(G_("Windows")));
+	MCHECK(newmenuitem(G_("Cascade"), 0, menucascade));
+	MCHECK(newmenuitem(G_("Tile &Horizontally"), 0, menutilehoriz));
+	MCHECK(newmenuitem(G_("Tile &Vertically"), 0, menutilevert));
+	MCHECK(newmenuitem(G_("Minimize group"), 0, menuminimizegroup));
+	MCHECK(newmenuitem(G_("Restore group"), 0, menurestoregroup));
+    }
+    return 0;
+}
+
 #include <locale.h>
 
-int setupui()
+int setupui(void)
 {
     char *p, *ctype, Rlocale[1000] = ""; /* Windows' locales can be very long */
 
     initapp(0, 0);
 
     /* set locale before doing anything with menus */
-    setlocale(LC_CTYPE, ""); /* necessary in case next fails to set 
+    setlocale(LC_CTYPE, ""); /* necessary in case next fails to set
 				a valid locale */
     if((p = getenv("LC_ALL"))) strcpy(Rlocale, p);
     if((p = getenv("LC_CTYPE"))) strcpy(Rlocale, p);
@@ -1027,49 +1067,49 @@ int setupui()
     }
 #endif
     TRACERUI("Console");
-    if (!(RConsole = newconsole("R Console",
-				StandardWindow | Document | Menubar)))
-	return 0;
+    int flags = StandardWindow | Document | Menubar;
+    if(mbcslocale) flags |= UseUnicode;
+    if (!(RConsole = newconsole("R Console", flags ))) return 0;
     TRACERUI("Console done");
 #ifdef USE_MDI
     if (ismdi()) {
-          int btsize = 24;
-          rect r = rect(2, 2, btsize, btsize);
-          control tb, bt;
+	  int btsize = 24;
+	  rect r = rect(2, 2, btsize, btsize);
+	  control tb, bt;
 
-          MCHECK(tb = newtoolbar(btsize + 4));
-          addto(tb);
+	  MCHECK(tb = newtoolbar(btsize + 4));
+	  addto(tb);
 
-          MCHECK(bt = newtoolbutton(open_image, r, menueditoropen));
-          MCHECK(addtooltip(bt, G_("Open script")));
-          r.x += (btsize + 1) ;
+	  MCHECK(bt = newtoolbutton(open_image, r, menueditoropen));
+	  MCHECK(addtooltip(bt, G_("Open script")));
+	  r.x += (btsize + 1) ;
 
-          MCHECK(bt = newtoolbutton(open1_image, r, menuloadimage));
-          MCHECK(addtooltip(bt, G_("Load image")));
-          r.x += (btsize + 1) ;
+	  MCHECK(bt = newtoolbutton(open1_image, r, menuloadimage));
+	  MCHECK(addtooltip(bt, G_("Load workspace")));
+	  r.x += (btsize + 1) ;
 
-          MCHECK(bt = newtoolbutton(save_image, r, menusaveimage));
-          MCHECK(addtooltip(bt, G_("Save image")));
-          r.x += (btsize + 6);
+	  MCHECK(bt = newtoolbutton(save_image, r, menusaveimage));
+	  MCHECK(addtooltip(bt, G_("Save workspace")));
+	  r.x += (btsize + 6);
 
-          MCHECK(bt = newtoolbutton(copy_image, r, buttoncopy));
-          MCHECK(addtooltip(bt, G_("Copy")));
-          r.x += (btsize + 1);
+	  MCHECK(bt = newtoolbutton(copy_image, r, buttoncopy));
+	  MCHECK(addtooltip(bt, G_("Copy")));
+	  r.x += (btsize + 1);
 
-          MCHECK(bt = newtoolbutton(paste_image, r, buttonpaste));
-          MCHECK(addtooltip(bt, G_("Paste")));
-          r.x += (btsize + 1);
+	  MCHECK(bt = newtoolbutton(paste_image, r, buttonpaste));
+	  MCHECK(addtooltip(bt, G_("Paste")));
+	  r.x += (btsize + 1);
 
-          MCHECK(bt = newtoolbutton(copypaste_image, r, buttoncopypaste));
-          MCHECK(addtooltip(bt, G_("Copy and paste")));
-          r.x += (btsize + 6);
+	  MCHECK(bt = newtoolbutton(copypaste_image, r, buttoncopypaste));
+	  MCHECK(addtooltip(bt, G_("Copy and paste")));
+	  r.x += (btsize + 6);
 
-          MCHECK(bt = newtoolbutton(stop_image, r, buttonkill));
-          MCHECK(addtooltip(bt, G_("Stop current computation")));
-          r.x += (btsize + 6) ;
+	  MCHECK(bt = newtoolbutton(stop_image, r, buttonkill));
+	  MCHECK(addtooltip(bt, G_("Stop current computation")));
+	  r.x += (btsize + 6) ;
 
-          MCHECK(bt = newtoolbutton(print_image, r, menuprint));
-          MCHECK(addtooltip(bt, G_("Print")));
+	  MCHECK(bt = newtoolbutton(print_image, r, menuprint));
+	  MCHECK(addtooltip(bt, G_("Print")));
     }
     if (ismdi() && (RguiMDI & RW_STATUSBAR)) {
 	TRACERUI("status bar");
@@ -1112,7 +1152,7 @@ int setupui()
     MCHECK(newmenu(G_("Edit")));
     MCHECK(mcopy = newmenuitem(G_("Copy"), 'C', menucopy));
     MCHECK(mpaste = newmenuitem(G_("Paste"), 'V', menupaste));
-    MCHECK(mpastecmds = newmenuitem(G_("Paste commands only"), 0, 
+    MCHECK(mpastecmds = newmenuitem(G_("Paste commands only"), 0,
 				    menupastecmds));
     MCHECK(mcopypaste = newmenuitem(G_("Copy and Paste"), 'X', menucopypaste));
     MCHECK(newmenuitem(G_("Select all"), 0, menuselectall));
@@ -1131,7 +1171,7 @@ int setupui()
     }
 #endif
     MCHECK(newmenu(G_("Misc")));
-    MCHECK(newmenuitem(G_("Stop current computation           \tESC"), 0, 
+    MCHECK(newmenuitem(G_("Stop current computation           \tESC"), 0,
 		       menukill));
     MCHECK(newmenuitem(G_("Stop all computations"), 0, menukillall));
     if (DebugMenuitem || isdebuggerpresent())
@@ -1140,22 +1180,20 @@ int setupui()
     MCHECK(mlazy = newmenuitem(G_("Buffered output"), 'W', menulazy));
     MCHECK(mcomplete = newmenuitem(G_("Word completion"), 0, menucomplete));
     check(mcomplete);
-    MCHECK(mfncomplete = newmenuitem(G_("Filename completion"), 0, 
+    MCHECK(mfncomplete = newmenuitem(G_("Filename completion"), 0,
 				     menufncomplete));
     if(check_file_completion())
 	check(mfncomplete);
-    else 
+    else
 	uncheck(mfncomplete);
     MCHECK(newmenuitem("-", 0, NULL));
     MCHECK(mls = newmenuitem(G_("List objects"), 0, menuls));
     MCHECK(mrm = newmenuitem(G_("Remove all objects"), 0, menurm));
-    MCHECK(msearch = newmenuitem(G_("List &search path"), 0, menusearch));
+    MCHECK(msearch = newmenuitem(G_("List search &path"), 0, menusearch));
 
     pmenu = (PkgMenuItems) malloc(sizeof(struct structPkgMenuItems));
     RguiPackageMenu(pmenu);
-#ifdef USE_MDI
-    newmdimenu();
-#endif
+    RguiWindowMenu();
     MCHECK(m = newmenu(G_("Help")));
     MCHECK(newmenuitem(G_("Console"), 0, menuconsolehelp));
     MCHECK(newmenuitem("-", 0, NULL));
@@ -1163,8 +1201,8 @@ int setupui()
     hmenu = (HelpMenuItems) malloc(sizeof(struct structHelpMenuItems));
     RguiCommonHelp(m, hmenu);
     consolesetbrk(RConsole, menukill, ESC, 0);
-    gl_hist_init(R_HistorySize, 0);
-    if (R_RestoreHistory) gl_loadhistory(R_HistoryFile);
+    wgl_hist_init(R_HistorySize, 0);
+    if (R_RestoreHistory) wgl_loadhistory(R_HistoryFile);
     if (ismdi() && !(RguiMDI & RW_TOOLBAR)) toolbar_hide();
     show(RConsole);
     return 1;
@@ -1172,23 +1210,24 @@ int setupui()
 
 #ifdef USE_MDI
 static RECT RframeRect; /* for use by pagercreate */
-RECT *RgetMDIsize()
+RECT *RgetMDIsize(void)
 {
     GetClientRect(hwndClient, &RframeRect);
     return &RframeRect;
 }
 
-int RgetMDIwidth()
+int RgetMDIwidth(void)
 {
     return RgetMDIsize()->right;
 }
 
-int RgetMDIheight()
+int RgetMDIheight(void)
 {
     return RgetMDIsize()->bottom;
 }
 #endif
 
+#if 0
 extern int  CharacterMode;
 int DialogSelectFile(char *buf, int len)
 {
@@ -1197,13 +1236,14 @@ int DialogSelectFile(char *buf, int len)
     setuserfilter("All files (*.*)\0*.*\0\0");
     fn = askfilename(G_("Select file"), "");
 /*    if (!CharacterMode)
-  	show(RConsole); */
+	show(RConsole); */
     if (fn)
 	strncpy(buf, fn, len);
     else
 	strcpy(buf, "");
     return (strlen(buf));
 }
+#endif
 
 static menu *usermenus;
 static char **usermenunames;
@@ -1229,7 +1269,7 @@ char *getusermenuname(int pos) {
     return(usermenunames[pos]);
 }
 
-menuItems *wingetmenuitems(char *mname, char *errmsg) {
+menuItems *wingetmenuitems(const char *mname, char *errmsg) {
     menuItems *items;
     char mitem[1002], *p, *q, *r;
     int i,j=0;
@@ -1249,7 +1289,7 @@ menuItems *wingetmenuitems(char *mname, char *errmsg) {
     strcpy(mitem, mname); strcat(mitem, "/");
 
     for (i = 0; i < nitems; i++) {
-	p = (char *)strstr(umitems[i]->name, mitem);
+	p = strstr(umitems[i]->name, mitem);
 
 	if (p == NULL)
 	    continue;
@@ -1297,9 +1337,7 @@ void freemenuitems(menuItems *items) {
     free(items);
 }
 
-extern menu getGraphMenu(char *); /* from devga.c */
-
-static menu getMenu(char * name)
+static menu getMenu(const char * name)
 {
     int i;
     for (i = 0; i < nmenus; i++)
@@ -1313,13 +1351,14 @@ static menu getMenu(char * name)
     else return(NULL);
 }
 
-int winaddmenu(char * name, char *errmsg)
+int winaddmenu(const char *name, char *errmsg)
 {
-    char *p, *submenu = name, start[501];
+    const char *submenu = name;
+    char *p, start[501];
     menu parent;
 
     if (getMenu(name))
-    	return 0;	/* Don't add repeats */
+	return 0;	/* Don't add repeats */
 
     if (nmenus >= alloc_menus) {
 	if(alloc_menus <= 0) {
@@ -1330,7 +1369,7 @@ int winaddmenu(char * name, char *errmsg)
 	    alloc_menus += 10;
 	    usermenus = (menu *) realloc(usermenus, sizeof(menu) * alloc_menus);
 	    usermenunames = (char **) realloc(usermenunames,
-					      sizeof(char *) * alloc_menus);   
+					      sizeof(char *) * alloc_menus);
 	}
     }
     if (strlen(name) > 500) {
@@ -1354,7 +1393,7 @@ int winaddmenu(char * name, char *errmsg)
     }
     if (m) {
 	usermenus[nmenus] = m;
-	usermenunames[nmenus]= strdup(name);
+	usermenunames[nmenus] = strdup(name);
 	nmenus++;
 	show(RConsole);
 	return 0;
@@ -1364,7 +1403,8 @@ int winaddmenu(char * name, char *errmsg)
     }
 }
 
-int winaddmenuitem(char * item, char * menu, char * action, char *errmsg)
+int winaddmenuitem(const char * item, const char * menu,
+		   const char * action, char *errmsg)
 {
     int i, im;
     menuitem m;
@@ -1416,7 +1456,7 @@ int winaddmenuitem(char * item, char * menu, char * action, char *errmsg)
 		    umitems = (Uitem *) malloc(sizeof(Uitem) * alloc_items);
 		} else {
 		    alloc_items += 100;
-		    umitems = (Uitem *) realloc(umitems, 
+		    umitems = (Uitem *) realloc(umitems,
 						sizeof(Uitem) * alloc_items);
 		}
 	    }
@@ -1445,14 +1485,14 @@ int winaddmenuitem(char * item, char * menu, char * action, char *errmsg)
     return 0;
 }
 
-int windelmenu(char * menu, char *errmsg)
+int windelmenu(const char * menu, char *errmsg)
 {
     int i, j, count = 0, len = strlen(menu);
 
     j = 0;
     for (i = 0; i < nmenus; i++) {
 	if (strcmp(menu, usermenunames[i]) == 0
-	  || (strncmp(menu, usermenunames[i], len) == 0 && 
+	  || (strncmp(menu, usermenunames[i], len) == 0 &&
 	      usermenunames[i][len] == '/')) {
 	    remove_menu_item(usermenus[i]);
 	    count++;
@@ -1462,7 +1502,7 @@ int windelmenu(char * menu, char *errmsg)
 		usermenus[j] = usermenus[i];
 	    }
 	    j++;
-        }
+	}
     }
     nmenus -= count;
     if (!count) {
@@ -1473,7 +1513,7 @@ int windelmenu(char * menu, char *errmsg)
     /* Delete any menu items in this menu */
 
     for (j = nitems - 1; j >= 0; j--) {
-	if (strncmp(menu, umitems[j]->name, len) == 0 && 
+	if (strncmp(menu, umitems[j]->name, len) == 0 &&
 	    umitems[j]->name[len] == '/')
 	    windelmenuitem(umitems[j]->name + len + 1, menu, errmsg);
     }
@@ -1483,7 +1523,7 @@ int windelmenu(char * menu, char *errmsg)
     return 0;
 }
 
-void windelmenus(char * prefix)
+void windelmenus(const char * prefix)
 {
     int i, len = strlen(prefix);
 
@@ -1493,7 +1533,7 @@ void windelmenus(char * prefix)
     }
 }
 
-int windelmenuitem(char * item, char * menu, char *errmsg)
+int windelmenuitem(const char * item, const char * menu, char *errmsg)
 {
     int i;
     char mitem[1002];
@@ -1516,4 +1556,3 @@ int windelmenuitem(char * item, char * menu, char *errmsg)
     show(RConsole);
     return 0;
 }
-
