@@ -1,3 +1,19 @@
+#  File src/library/base/R/attach.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 attach <- function(what, pos = 2, name = deparse(substitute(what)),
                    warn.conflicts = TRUE)
 {
@@ -14,18 +30,19 @@ attach <- function(what, pos = 2, name = deparse(substitute(what)),
                 break
             }
         }
-        ob <- objects(db.pos, all = TRUE)
+        ob <- objects(db.pos, all.names = TRUE)
         if(.isMethodsDispatchOn()) {
-            these <- objects(db.pos, all = TRUE)
-            these <- these[substr(these, 1, 6) == ".__M__"]
+            ## <FIXME>: this is wrong-headed: see library().
+            these <- objects(db.pos, all.names = TRUE)
+            these <- these[substr(these, 1L, 6L) == ".__M__"]
             gen <- gsub(".__M__(.*):([^:]+)", "\\1", these)
             from <- gsub(".__M__(.*):([^:]+)", "\\2", these)
             gen <- gen[from != ".GlobalEnv"]
             ob <- ob[!(ob %in% gen)]
         }
-        ipos <- seq_along(sp)[-c(db.pos, match(c("Autoloads", "CheckExEnv"), sp, 0))]
+        ipos <- seq_along(sp)[-c(db.pos, match(c("Autoloads", "CheckExEnv"), sp, 0L))]
         for (i in ipos) {
-            obj.same <- match(objects(i, all = TRUE), ob, nomatch = 0)
+            obj.same <- match(objects(i, all.names = TRUE), ob, nomatch = 0L)
             if (any(obj.same > 0)) {
                 same <- ob[obj.same]
                 same <- same[!(same %in% dont.mind)]
@@ -43,7 +60,7 @@ attach <- function(what, pos = 2, name = deparse(substitute(what)),
                 if(length(same)) {
                     cat("\n\tThe following object(s) are masked",
                         if (i < db.pos) "_by_" else "from", sp[i],
-                        if (sum(sp == sp[i]) > 1) paste("( position", i, ")"),
+                        if (sum(sp == sp[i]) > 1L) paste("( position", i, ")"),
                         ":\n\n\t", same, "\n\n")
                 }
             }
@@ -55,7 +72,7 @@ attach <- function(what, pos = 2, name = deparse(substitute(what)),
                 "*** Note that 'pos=1' will give an error in the future")
         pos <- 2
     }
-    if (is.character(what) && (length(what)==1)){
+    if (is.character(what) && (length(what) == 1L)){
         if (!file.exists(what))
             stop(gettextf("file '%s' not found", what), domain = NA)
         name <- paste("file:", what, sep="")
@@ -68,24 +85,23 @@ attach <- function(what, pos = 2, name = deparse(substitute(what)),
        !exists(".conflicts.OK", envir = value, inherits = FALSE)) {
         checkConflicts(value)
     }
-    if((length(objects(envir = value, all=TRUE)) > 0)
+    if( length(objects(envir = value, all.names = TRUE) )
        && .isMethodsDispatchOn())
       methods:::cacheMetaData(value, TRUE)
     invisible(value)
 }
 
-detach <- function(name, pos=2, version)
+detach <- function(name, pos=2, unload=FALSE, character.only = FALSE)
 {
     if(!missing(name)) {
-        name <- substitute(name)# when a name..
+	if(!character.only)
+	    name <- substitute(name)# when a name..
 	pos <-
 	    if(is.numeric(name))
                 name
 	    else {
                 if (!is.character(name))
                     name <- deparse(name)
-                if (!missing(version))
-                    name <- manglePackageName(name, version)
                 match(name, search())
             }
 	if(is.na(pos))
@@ -93,34 +109,50 @@ detach <- function(name, pos=2, version)
     }
     env <- as.environment(pos)
     packageName <- search()[[pos]]
-    libpath <- attr(env, "path")
-    if(length(grep("^package:", packageName))) {
-        pkgname <- sub("^package:", "", packageName)
-        hook <- getHook(packageEvent(pkgname, "detach")) # might be list()
-        for(fun in rev(hook)) try(fun(pkgname, libpath))
+
+    ## we need treat packages differently from other objects.
+    isPkg <- grepl("^package:", packageName)
+    if (!isPkg) {
+        .Internal(detach(pos))
+        return(invisible())
     }
+
+    pkgname <- sub("^package:", "", packageName)
+    libpath <- attr(env, "path")
+    hook <- getHook(packageEvent(pkgname, "detach")) # might be list()
+    for(fun in rev(hook)) try(fun(pkgname, libpath))
     if(exists(".Last.lib", mode = "function", where = pos, inherits=FALSE)) {
         .Last.lib <- get(".Last.lib",  mode = "function", pos = pos,
                          inherits=FALSE)
         if(!is.null(libpath)) try(.Last.lib(libpath))
     }
     .Internal(detach(pos))
-    ## note: here the code internally assumes the separator is "/" even
-    ## on Windows.
-    if(length(grep("^package:", packageName)))
-        .Call("R_lazyLoadDBflush",
-              paste(libpath, "/R/", pkgname, ".rdb", sep=""), PACKAGE="base")
-    ## Check for detaching a  package required by another package (not
+
+    ## Check for detaching a package require()d by another package (not
     ## by .GlobalEnv because detach() can't currently fix up the
     ## .required there)
-    for(pkgs in search()[-1]) {
-        if(!isNamespace(as.environment(pkgs)) &&
-           exists(".required", pkgs, inherits = FALSE) &&
-           packageName %in% paste("package:", get(".required", pkgs, inherits = FALSE),sep=""))
-            warning(packageName, " is required by ", pkgs, " (still attached)")
+    for(pkg in search()[-1L]) {
+        ## How can a namespace environment get on the search list?
+        ## (base fails isNamespace).
+        if(!isNamespace(as.environment(pkg)) &&
+           exists(".required", pkg, inherits = FALSE) &&
+           pkgname %in% get(".required", pkg, inherits = FALSE))
+            warning(packageName, " is required by ", pkg, " (still attached)")
     }
-    if(.isMethodsDispatchOn())
-            methods:::cacheMetaData(env, FALSE)
+
+    if(pkgname %in% loadedNamespaces()) {
+        ## the lazyload DB is flushed when the name space is unloaded
+        if(unload) {
+            tryCatch(unloadNamespace(pkgname),
+                     error = function(e)
+                     warning(pkgname, " namespace cannot be unloaded\n",
+                             conditionMessage(e), call. = FALSE))
+        }
+    } else
+        .Call("R_lazyLoadDBflush",
+              paste(libpath, "/R/", pkgname, ".rdb", sep=""),
+              PACKAGE="base")
+    invisible()
 }
 
 ls <- objects <-
@@ -140,13 +172,13 @@ ls <- objects <-
     }
     all.names <- .Internal(ls(envir, all.names))
     if (!missing(pattern)) {
-        if ((ll <- length(grep("[", pattern, fixed=TRUE))) > 0 &&
+        if ((ll <- length(grep("[", pattern, fixed=TRUE))) &&
             ll != length(grep("]", pattern, fixed=TRUE))) {
             if (pattern == "[") {
                 pattern <- "\\["
                 warning("replaced regular expression pattern '[' by  '\\\\['")
             }
-            else if (length(grep("[^\\\\]\\[<-", pattern) > 0)) {
+            else if (length(grep("[^\\\\]\\[<-", pattern))) {
                 pattern <- sub("\\[<-", "\\\\\\[<-", pattern)
                 warning("replaced '[<-' by '\\\\[<-' in regular expression pattern")
             }
