@@ -20,7 +20,7 @@
 ## The difftime class already covers time differences in days.
 
 ## Need to take timezone into account here
-Sys.Date <- function() .Internal(POSIXlt2Date(as.POSIXlt(Sys.time())))
+Sys.Date <- function() as.Date(as.POSIXlt(Sys.time()))
 
 as.Date <- function(x, ...) UseMethod("as.Date")
 
@@ -37,21 +37,27 @@ as.Date.factor <- function(x, ...) as.Date(as.character(x), ...)
 
 as.Date.character <- function(x, format="", ...)
 {
-    fromchar <- function(x) {
-	xx <- x[1]
+    charToDate <- function(x) {
+	xx <- x[1L]
         if(is.na(xx)) {
-            j <- 1
-            while(is.na(xx) && (j <- j+1) <= length(x)) xx <- x[j]
+            j <- 1L
+            while(is.na(xx) && (j <- j+1L) <= length(x)) xx <- x[j]
             if(is.na(xx)) f <- "%Y-%m-%d" # all NAs
         }
 	if(is.na(xx) ||
-	   !is.na(strptime(xx, f <- "%Y-%m-%d")) ||
-	   !is.na(strptime(xx, f <- "%Y/%m/%d"))
+	   !is.na(strptime(xx, f <- "%Y-%m-%d", tz="GMT")) ||
+	   !is.na(strptime(xx, f <- "%Y/%m/%d", tz="GMT"))
            ) return(strptime(x, f))
 	stop("character string is not in a standard unambiguous format")
     }
-    res <- if(missing(format)) fromchar(x) else strptime(x, format)
-    .Internal(POSIXlt2Date(res))
+    res <- if(missing(format)) charToDate(x) else strptime(x, format, tz="GMT")
+    as.Date(res)
+}
+
+as.Date.numeric <- function(x, origin, ...)
+{
+    if(missing(origin)) stop("'origin' must be supplied")
+    as.Date(origin, ...) + x
 }
 
 as.Date.default <- function(x, ...)
@@ -79,8 +85,8 @@ as.Date.dates <- function(x, ...)
     if(inherits(x, "dates")) {
         z <- attr(x, "origin")
         x <- trunc(as.numeric(x))
-        if(length(z) == 3 && is.numeric(z))
-            x  <- x + as.numeric(as.Date(paste(z[3], z[1], z[2], sep="/")))
+        if(length(z) == 3L && is.numeric(z))
+            x  <- x + as.numeric(as.Date(paste(z[3L], z[1L], z[2L], sep="/")))
         return(structure(x, class = "Date"))
     } else stop(gettextf("'%s' is not a \"dates\" object",
                          deparse(substitute(x)) ))
@@ -101,7 +107,7 @@ print.Date <- function(x, ...)
 
 summary.Date <- function(object, digits = 12, ...)
 {
-    x <- summary.default(unclass(object), digits = digits, ...)[1:6]# not NA's
+    x <- summary.default(unclass(object), digits = digits, ...)[1L:6L]# not NA's
     class(x) <- oldClass(object)
     x
 }
@@ -109,11 +115,9 @@ summary.Date <- function(object, digits = 12, ...)
 "+.Date" <- function(e1, e2)
 {
     coerceTimeUnit <- function(x)
-    {
-        round(switch(attr(x,"units"),
-               secs = x/86400, mins = x/1440, hours = x/24,
-               days = x, weeks = 7*x))
-    }
+        as.vector(round(switch(attr(x,"units"),
+                               secs = x/86400, mins = x/1440, hours = x/24,
+                               days = x, weeks = 7*x)))
 
     if (nargs() == 1) return(e1)
     # only valid if one of e1 and e2 is a scalar.
@@ -127,16 +131,14 @@ summary.Date <- function(object, digits = 12, ...)
 "-.Date" <- function(e1, e2)
 {
     coerceTimeUnit <- function(x)
-    {
-        round(switch(attr(x, "units"),
-               secs = x/86400, mins = x/1440, hours = x/24,
-               days = x, weeks = 7*x))
-    }
+        as.vector(round(switch(attr(x,"units"),
+                               secs = x/86400, mins = x/1440, hours = x/24,
+                               days = x, weeks = 7*x)))
     if(!inherits(e1, "Date"))
         stop("Can only subtract from Date objects")
     if (nargs() == 1) stop("unary - is not defined for Date objects")
     if(inherits(e2, "Date")) return(difftime(e1, e2, units="days"))
-    if (inherits(e2, "difftime")) e2 <- unclass(coerceTimeUnit(e2))
+    if (inherits(e2, "difftime")) e2 <- coerceTimeUnit(e2)
     if(!is.null(attr(e2, "class")))
         stop("can only subtract numbers from Date objects")
     structure(unclass(as.Date(e1)) - e2, class = "Date")
@@ -164,7 +166,7 @@ Summary.Date <- function (..., na.rm)
     ok <- switch(.Generic, max = , min = , range = TRUE, FALSE)
     if (!ok) stop(.Generic, " not defined for Date objects")
     val <- NextMethod(.Generic)
-    class(val) <- oldClass(list(...)[[1]])
+    class(val) <- oldClass(list(...)[[1L]])
     val
 }
 
@@ -211,15 +213,15 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
 {
     if (missing(from)) stop("'from' must be specified")
     if (!inherits(from, "Date")) stop("'from' must be a Date object")
-        if(length(as.Date(from)) != 1) stop("'from' must be of length 1")
+        if(length(as.Date(from)) != 1L) stop("'from' must be of length 1")
     if (!missing(to)) {
         if (!inherits(to, "Date")) stop("'to' must be a Date object")
-        if (length(as.Date(to)) != 1) stop("'to' must be of length 1")
+        if (length(as.Date(to)) != 1L) stop("'to' must be of length 1")
     }
     if (!missing(along.with)) {
         length.out <- length(along.with)
     }  else if (!is.null(length.out)) {
-        if (length(length.out) != 1) stop("'length.out' must be of length 1")
+        if (length(length.out) != 1L) stop("'length.out' must be of length 1")
         length.out <- ceiling(length.out)
     }
     status <- c(!missing(to), !missing(by), !is.null(length.out))
@@ -232,27 +234,27 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
         return(structure(res, class = "Date"))
     }
 
-    if (length(by) != 1) stop("'by' must be of length 1")
-    valid <- 0
+    if (length(by) != 1L) stop("'by' must be of length 1")
+    valid <- 0L
     if (inherits(by, "difftime")) {
         by <- switch(attr(by,"units"), secs = 1/86400, mins = 1/1440,
                      hours = 1/24, days = 1, weeks = 7) * unclass(by)
     } else if(is.character(by)) {
-        by2 <- strsplit(by, " ", fixed=TRUE)[[1]]
-        if(length(by2) > 2 || length(by2) < 1)
+        by2 <- strsplit(by, " ", fixed=TRUE)[[1L]]
+        if(length(by2) > 2L || length(by2) < 1L)
             stop("invalid 'by' string")
         valid <- pmatch(by2[length(by2)],
                         c("days", "weeks", "months", "years"))
         if(is.na(valid)) stop("invalid string for 'by'")
         if(valid <= 2) {
             by <- c(1, 7)[valid]
-            if (length(by2) == 2) by <- by * as.integer(by2[1])
+            if (length(by2) == 2L) by <- by * as.integer(by2[1L])
         } else
-            by <- if(length(by2) == 2) as.integer(by2[1]) else 1
+            by <- if(length(by2) == 2L) as.integer(by2[1L]) else 1
     } else if(!is.numeric(by)) stop("invalid mode for 'by'")
     if(is.na(by)) stop("'by' is NA")
 
-    if(valid <= 2) {
+    if(valid <= 2L) {
         from <- unclass(as.Date(from))
         if(!is.null(length.out))
             res <- seq.int(from, by=by, length.out=length.out)
@@ -264,7 +266,7 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
         return(structure(res, class="Date"))
     } else {  # months or years or DSTdays
         r1 <- as.POSIXlt(from)
-        if(valid == 4) {
+        if(valid == 4L) {
             if(missing(to)) { # years
                 yr <- seq.int(r1$year, by = by, length.out = length.out)
             } else {
@@ -272,8 +274,8 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
                 yr <- seq.int(r1$year, to$year, by)
             }
             r1$year <- yr
-            res <- .Internal(POSIXlt2Date(r1))
-        } else if(valid == 3) { # months
+            res <- as.Date(r1)
+        } else if(valid == 3L) { # months
             if(missing(to)) {
                 mon <- seq.int(r1$mon, by = by, length.out = length.out)
             } else {
@@ -281,7 +283,7 @@ seq.Date <- function(from, to, by, length.out=NULL, along.with=NULL, ...)
                 mon <- seq.int(r1$mon, 12*(to$year - r1$year) + to$mon, by)
             }
             r1$mon <- mon
-            res <- .Internal(POSIXlt2Date(r1))
+            res <- as.Date(r1)
         }
         return(res)
     }
@@ -296,43 +298,55 @@ cut.Date <-
 
     if (inherits(breaks, "Date")) {
 	breaks <- as.Date(breaks)
-    } else if(is.numeric(breaks) && length(breaks) == 1) {
+    } else if(is.numeric(breaks) && length(breaks) == 1L) {
 	## specified number of breaks
-    } else if(is.character(breaks) && length(breaks) == 1) {
-        by2 <- strsplit(breaks, " ", fixed=TRUE)[[1]]
-        if(length(by2) > 2 || length(by2) < 1)
+    } else if(is.character(breaks) && length(breaks) == 1L) {
+        by2 <- strsplit(breaks, " ", fixed=TRUE)[[1L]]
+        if(length(by2) > 2L || length(by2) < 1L)
             stop("invalid specification of 'breaks'")
 	valid <-
-	    pmatch(by2[length(by2)], c("days", "weeks", "months", "years"))
+	    pmatch(by2[length(by2)], c("days", "weeks", "months", "years", "quarters"))
 	if(is.na(valid)) stop("invalid specification of 'breaks'")
 	start <- as.POSIXlt(min(x, na.rm=TRUE))
-	if(valid == 1) incr <- 1
-	if(valid == 2) {
+	if(valid == 1L) incr <- 1L
+	if(valid == 2L) {
 	    start$mday <- start$mday - start$wday
 	    if(start.on.monday)
-		start$mday <- start$mday + ifelse(start$wday > 0, 1, -6)
-	    incr <- 7
+		start$mday <- start$mday + ifelse(start$wday > 0L, 1L, -6L)
+	    incr <- 7L
 	}
-	if(valid == 3) {
-        start$mday <- 1
+    if(valid == 3L) {
+        start$mday <- 1L
         end <- as.POSIXlt(max(x, na.rm = TRUE))
-        end <- as.POSIXlt(end + (31 * 86400))
-        end$mday <- 1
-        breaks <- as.Date(seq(start, end, "months"))
-    } else if(valid == 4) {
-        start$mon <- 0
-        start$mday <- 1
+        step <- ifelse(length(by2) == 2L, as.integer(by2[1L]), 1L)
+        end <- as.POSIXlt(end + (31 * step * 86400))
+        end$mday <- 1L
+        breaks <- as.Date(seq(start, end, breaks))
+    } else if(valid == 4L) {
+        start$mon <- 0L
+        start$mday <- 1L
         end <- as.POSIXlt(max(x, na.rm = TRUE))
-        end <- as.POSIXlt(end + (366 * 86400))
-        end$mon <- 0
-        end$mday <- 1
-        breaks <- as.Date(seq(start, end, "years"))
+        step <- ifelse(length(by2) == 2L, as.integer(by2[1L]), 1L)
+        end <- as.POSIXlt(end + (366 * step * 86400))
+        end$mon <- 0L
+        end$mday <- 1L
+        breaks <- as.Date(seq(start, end, breaks))
+    } else if(valid == 5L) {
+        qtr <- rep(c(0L, 3L, 6L, 9L), each = 3L)
+        start$mon <- qtr[start$mon + 1L]
+        start$mday <- 1L
+        end <- as.POSIXlt(max(x, na.rm = TRUE))
+        step <- ifelse(length(by2) == 2L, as.integer(by2[1L]), 1L)
+        end <- as.POSIXlt(end + (93 * step * 86400))
+        end$mon <- qtr[end$mon + 1L]
+        end$mday <- 1L
+        breaks <- as.Date(seq(start, end, paste(step * 3L, "months")))
     } else {
-        start <- .Internal(POSIXlt2Date(start))
-        if (length(by2) == 2) incr <- incr * as.integer(by2[1])
-	    maxx <- max(x, na.rm = TRUE)
+        start <- as.Date(start)
+        if (length(by2) == 2L) incr <- incr * as.integer(by2[1L])
+        maxx <- max(x, na.rm = TRUE)
         breaks <- seq.int(start, maxx + incr, breaks)
-        breaks <- breaks[1:(1+max(which(breaks < maxx)))]
+        breaks <- breaks[seq_len(1L+max(which(breaks <= maxx)))]
       }
     } else stop("invalid specification of 'breaks'")
     res <- cut(unclass(x), unclass(breaks), labels = labels,
@@ -343,7 +357,7 @@ cut.Date <-
 
 julian.Date <- function(x, origin = as.Date("1970-01-01"), ...)
 {
-    if(length(origin) != 1) stop("'origin' must be of length one")
+    if(length(origin) != 1L) stop("'origin' must be of length one")
     structure(unclass(x) - unclass(origin), "origin" = origin)
 }
 
@@ -355,8 +369,8 @@ months.Date <- function(x, abbreviate = FALSE)
 
 quarters.Date <- function(x, ...)
 {
-    x <- (as.POSIXlt(x)$mon) %/% 3
-    paste("Q", x+1, sep = "")
+    x <- (as.POSIXlt(x)$mon) %/% 3L
+    paste("Q", x+1L, sep = "")
 }
 
 ## These only make sense for negative digits, but still ...
@@ -369,7 +383,7 @@ round.Date <- function(x, ...)
     val
 }
 
-## must avoid truncating dates prior to 1970-01-01 forwards.
+## must avoid truncating forards dates prior to 1970-01-01.
 trunc.Date <- function(x, ...) round(x - 0.4999999)
 
 rep.Date <- function(x, ...)
@@ -378,23 +392,36 @@ rep.Date <- function(x, ...)
     structure(y, class="Date")
 }
 
-diff.Date <- function (x, lag = 1, differences = 1, ...)
+diff.Date <- function (x, lag = 1L, differences = 1L, ...)
 {
     ismat <- is.matrix(x)
-    xlen <- if (ismat) dim(x)[1] else length(x)
-    if (length(lag) > 1 || length(differences) > 1 || lag < 1 || differences < 1)
+    xlen <- if (ismat) dim(x)[1L] else length(x)
+    if (length(lag) > 1L || length(differences) > 1L || lag < 1L || differences < 1L)
         stop("'lag' and 'differences' must be integers >= 1")
     if (lag * differences >= xlen)
-        return(structure(numeric(0), class="difftime", units="days"))
+        return(structure(numeric(0L), class="difftime", units="days"))
     r <- x
-    i1 <- -1:-lag
-    if (ismat) for (i in 1:differences) r <- r[i1, , drop = FALSE] -
-            r[-nrow(r):-(nrow(r) - lag + 1), , drop = FALSE]
-    else for (i in 1:differences)
-        r <- r[i1] - r[-length(r):-(length(r) - lag + 1)]
+    i1 <- -seq_len(lag)
+    if (ismat)
+        for (i in seq_len(differences)) r <- r[i1, , drop = FALSE] -
+            r[-nrow(r):-(nrow(r) - lag + 1L), , drop = FALSE]
+    else for (i in seq_len(differences))
+        r <- r[i1] - r[-length(r):-(length(r) - lag + 1L)]
     r
 }
 
-# ---- additions in 2.6.0 -----
+## ---- additions in 2.6.0 -----
 
 is.numeric.Date <- function(x) FALSE
+
+## ---- additions in 2.8.0 -----
+
+split.Date <-
+function(x, f, drop = FALSE, ...)
+{
+    y <- split.default(as.integer(x), f, drop = drop)
+    for(i in seq_along(y)) class(y[[i]]) <- "Date"
+    y
+}
+
+xtfrm.Date <- function(x) as.numeric(x)
