@@ -25,10 +25,7 @@
 
 #include <Defn.h>
 
-#ifdef SUPPORT_MBCS
-# include <R_ext/rlocale.h>
-# include <wchar.h>
-#endif
+# include <R_ext/rlocale.h> /* includes wchar.h */
 
 #define R_USE_PROTOTYPES 1
 #include <R_ext/GraphicsEngine.h>
@@ -487,7 +484,6 @@ static double PicTeX_StrWidth(const char *str,
     size = gc->cex * gc->ps + 0.5;
     SetFont(gc->fontface, size, ptd);
     sum = 0;
-#if defined(SUPPORT_MBCS)
     if(mbcslocale && ptd->fontface != 5) {
 	/* This version at least uses the state of the MBCS */
 	int i, status, ucslen = mbcsToUcs2(str, NULL, 0, CE_NATIVE);
@@ -504,9 +500,9 @@ static double PicTeX_StrWidth(const char *str,
 	} else
 	    warning(_("invalid string in '%s'"), "PicTeX_StrWidth");
     } else
-#endif
 	for(p = str; *p; p++)
 	    sum += charwidth[ptd->fontface-1][(int)*p];
+
     return sum * ptd->fontsize;
 }
 
@@ -761,7 +757,7 @@ SEXP PicTeX(SEXP args)
     R_CheckDeviceAvailable();
     BEGIN_SUSPEND_INTERRUPTS {
 	pDevDesc dev;
-	if (!(dev = (pDevDesc) calloc(1, sizeof(NewDevDesc))))
+	if (!(dev = (pDevDesc) calloc(1, sizeof(DevDesc))))
 	    return 0;
 	if(!PicTeXDeviceDriver(dev, file, bg, fg, width, height, debug)) {
 	    free(dev);
