@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2004-5   The R Development Core Team.
+ *  Copyright (C) 2004-8   The R Development Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,12 +13,13 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #include <Rinternals.h>
 #include <R_ext/Boolean.h>
+#include <R_ext/GraphicsEngine.h> /* for DevDesc */
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -41,18 +42,22 @@ SEXP Type1FontInUse(SEXP, SEXP);
 SEXP CIDFontInUse(SEXP, SEXP);
 
 SEXP Quartz(SEXP);
+SEXP makeQuartzDefault();
 
 SEXP R_GD_nullDevice();
 
-Rboolean 
-PSDeviceDriver(NewDevDesc*, char*, char*, char*, char**,
-	       char*, char*, char*, double, double, Rboolean, double, 
-	       Rboolean, Rboolean, Rboolean, char*, char*, SEXP, char*);
+Rboolean
+PSDeviceDriver(pDevDesc, const char*, const char*, const char*,
+	       const char **, const char*, const char*, const char*,
+	       double, double, Rboolean, double, Rboolean, Rboolean,
+	       Rboolean, const char*, const char*, SEXP, const char*, int,
+	       Rboolean);
 
 Rboolean
-PDFDeviceDriver(NewDevDesc*, char *, char *, char *, char**, char *, 
-		char *, char *, double, double, double,
-		int, int, char*, SEXP, int, int);
+PDFDeviceDriver(pDevDesc, const char *, const char *, const char *,
+		const char **, const char *, const char *, const char *,
+		double, double, double, int, int, const char*, SEXP, 
+		int, int, const char *, int, int, Rboolean);
 
 #ifdef WIN32
 SEXP devga(SEXP);
