@@ -26,14 +26,14 @@ function(formula, data = list(),
          breaks = NULL, tol.ylab = 0.05, off = NULL, ylevels = NULL,
          col = NULL, main = "", xlab = NULL, ylab = NULL,
          xaxlabels = NULL, yaxlabels = NULL,
-         xlim = NULL, ylim = c(0, 1), ...,
+         xlim = NULL, ylim = c(0, 1), axes = TRUE, ...,
          subset = NULL)
 {
     ## extract x, y from formula
     m <- match.call(expand.dots = FALSE)
-    m <- m[c(1, match(c("formula", "data", "subset"), names(m), 0))]
+    m <- m[c(1L, match(c("formula", "data", "subset"), names(m), 0L))]
     require(stats, quietly=TRUE)
-    m[[1]] <- as.name("model.frame")
+    m[[1L]] <- as.name("model.frame")
     mf <- eval.parent(m)
     if(NCOL(mf) != 2)
         stop("'formula' should specify exactly two variables")
@@ -45,14 +45,14 @@ function(formula, data = list(),
     x <- mf[,2]
 
     ## graphical parameters
-    if(is.null(xlab)) xlab <- names(mf)[2]
-    if(is.null(ylab)) ylab <- names(mf)[1]
+    if(is.null(xlab)) xlab <- names(mf)[2L]
+    if(is.null(ylab)) ylab <- names(mf)[1L]
 
     ## call default interface
     spineplot(x, y, breaks = breaks, tol.ylab = tol.ylab, off = off, ylevels = NULL,
               col = col, main = main, xlab = xlab, ylab = ylab,
               xaxlabels = xaxlabels, yaxlabels = yaxlabels,
-              xlim = xlim, ylim = ylim, ...)
+              xlim = xlim, ylim = ylim, axes = axes, ...)
 }
 
 spineplot.default <-
@@ -60,7 +60,7 @@ function(x, y = NULL,
          breaks = NULL, tol.ylab = 0.05, off = NULL, ylevels = NULL,
          col = NULL, main = "", xlab = NULL, ylab = NULL,
          xaxlabels = NULL, yaxlabels = NULL,
-         xlim = NULL, ylim = c(0, 1), ...)
+         xlim = NULL, ylim = c(0, 1), axes = TRUE, ...)
 {
     ## either supply a 2-way table (i.e., both y and x are categorical)
     ## or two variables (y has to be categorical - x can be categorical
@@ -70,10 +70,10 @@ function(x, y = NULL,
             stop("a 2-way table has to be specified")
         tab <- x
         x.categorical <- TRUE
-        if(is.null(xlab)) xlab <- names(dimnames(tab))[1]
-        if(is.null(ylab)) ylab <- names(dimnames(tab))[2]
-        xnam <- dimnames(tab)[[1]]
-        ynam <- dimnames(tab)[[2]]
+        if(is.null(xlab)) xlab <- names(dimnames(tab))[1L]
+        if(is.null(ylab)) ylab <- names(dimnames(tab))[2L]
+        xnam <- dimnames(tab)[[1L]]
+        ynam <- dimnames(tab)[[2L]]
         ny <- NCOL(tab)
         nx <- NROW(tab)
     } else {
@@ -121,7 +121,7 @@ function(x, y = NULL,
     }
 
     ## compute rectangle positions on y axis
-    yat <- rbind(0, apply(prop.table(tab, 1), 1, cumsum))
+    yat <- rbind(0, apply(prop.table(tab, 1), 1L, cumsum))
 
     if(is.null(xlim)) xlim <- c(0, 1 + off * (nx-1))
     else if(any(xlim < 0) || any(xlim > 1)) {
@@ -142,7 +142,7 @@ function(x, y = NULL,
     ## compute coordinates
     ybottom <- as.vector(yat[-(ny+1),])
     ytop <- as.vector(yat[-1,])
-    xleft <- rep(xat[1:nx], rep(ny, nx))
+    xleft <- rep(xat[1L:nx], rep(ny, nx))
     xright <- rep(xat[2:(nx+1)] - off, rep(ny, nx))
     col <- rep(col, nx)
 
@@ -150,22 +150,26 @@ function(x, y = NULL,
     rect(xleft, ybottom, xright, ytop, col = col, ...)
 
     ## axes
-    ## 1: either numeric or level names
-    if(x.categorical)
-        axis(1, at = (xat[1:nx] + xat[2:(nx+1)] - off)/2, labels = xaxlabels, tick = FALSE)
-    else
-        axis(1, at = xat, labels = xaxlabels)
+    if(axes) {
+        ## side --
+        ## 1: either numeric or level names
+        if(x.categorical)
+            axis(1, at = (xat[1L:nx] + xat[2:(nx+1)] - off)/2,
+                 labels = xaxlabels, tick = FALSE)
+        else
+            axis(1, at = xat, labels = xaxlabels)
 
-    ## 2: axis with level names of y
-    yat <- yat[,1]
-    equidist <- any(diff(yat) < tol.ylab)
-    yat <- if(equidist) seq.int(1/(2*ny), 1-1/(2*ny), by = 1/ny)
-    else (yat[-1] + yat[-length(yat)])/2
-    axis(2, at = yat, labels = yaxlabels, tick = FALSE)
+        ## 2: axis with level names of y
+        yat <- yat[,1]
+        equidist <- any(diff(yat) < tol.ylab)
+        yat <- if(equidist) seq.int(1/(2*ny), 1-1/(2*ny), by = 1/ny)
+        else (yat[-1L] + yat[-length(yat)])/2
+        axis(2, at = yat, labels = yaxlabels, tick = FALSE)
 
-    ## 3: none
-    ## 4: simple numeric
-    axis(4)
+        ## 3: none
+        ## 4: simple numeric
+        axis(4)
+    }
     if(!x.categorical) box()
 
     ## return table visualized
