@@ -1,8 +1,26 @@
+#  File src/library/methods/R/SClasses.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 setClass <-
     ## Define Class to be an S-style class.
     function(Class, representation = list(), prototype = NULL,
              contains = character(), validity = NULL, access = list(),
-             where = topenv(parent.frame()), version = .newExternalptr(), sealed = FALSE, package = getPackageName(where))
+             where = topenv(parent.frame()), version = .newExternalptr(),
+             sealed = FALSE, package = getPackageName(where),
+             S3methods = FALSE)
 {
     oldDef <- getClassDef(Class, where)
     if(is(oldDef, "classRepresentation") && oldDef@sealed)
@@ -13,16 +31,16 @@ setClass <-
         if(!(missing(prototype) && missing(contains) && missing(validity) && missing(access)
              && missing(version) && missing(package)))
             stop("only arguments 'Class' and 'where' can be supplied when argument 'representation' is a 'classRepresentation' object")
-        if(length(classDef@package)==0)
+        if(length(classDef@package) == 0L)
             classDef@package <- package # the default
         superClasses <- allNames(classDef@contains)
     }
     else {
         ## catch the special case of a single class name as the representation
-        if(is.character(representation) && length(representation) == 1 &&
+        if(is.character(representation) && length(representation) == 1L &&
            is.null(names(representation)))
             representation <- list(representation)
-        slots <- nchar(allNames(representation)) > 0
+        slots <- nzchar(allNames(representation))
         superClasses <- c(as.character(representation[!slots]), contains)
         properties <- representation[slots]
         classDef <- makeClassRepresentation(Class, properties,superClasses, prototype, package,
@@ -31,37 +49,49 @@ setClass <-
     }
     classDef <- completeClassDefinition(Class, classDef, where, doExtends = FALSE)
     oldDef <- getClassDef(Class, where)
-    if(length(superClasses) == 0)
-        assignClassDef(Class, classDef, where)
-    else {
+    if(is(oldDef, "classRepresentation"))
+      .uncacheClass(Class, oldDef)
+    if(length(superClasses) > 0L) {
         sealed <- classDef@sealed
         classDef@sealed <- FALSE # to allow setIs to work anyway; will be reset later
         assignClassDef(Class, classDef, where)
         badContains <- character()
-        for(class2 in superClasses)
+        for(class2 in superClasses) {
             if(is(try(setIs(Class, class2, classDef = classDef, where = where)), "try-error"))
                 badContains <- c(badContains, class2)
-        if(length(badContains) > 0) {
+            else { # update class definition
+                classDef <- getClassDef(Class, where = where)
+                if(is.null(classDef))
+                  stop(gettextf('Internal error: definition of class "%s" not properly assigned', Class),
+                       domain = NA)
+            }
+          }
+        if(length(badContains)) {
             msg <- paste(dQuote(badContains), collapse = ", ")
             if(is(try(removeClass(Class, where)), "try-error"))
-                stop(gettextf("error in contained classes (%s) for class \"%s\" and unable to remove definition from '%s'",
+                stop(gettextf("error in contained classes (%s) for class \"%s\" and unable to remove definition from \"%s\"",
                               msg, Class, getPackageName(where)), domain = NA)
             if(is.null(oldDef))
-                stop(gettextf("error in contained classes (%s) for class \"%s\"; class definition removed from '%s'",
+                stop(gettextf("error in contained classes (%s) for class \"%s\"; class definition removed from \"%s\"",
                               msg, Class, getPackageName(where)), domain = NA)
             else if(is(try(setClass(Class, oldDef, where=where)), "try-error"))
-                stop(gettextf("error in contained classes (%s) for class \"%s\" and unable to restore previous definition from '%s'",
+                stop(gettextf("error in contained classes (%s) for class \"%s\" and unable to restore previous definition from \"%s\"",
                               msg, Class, getPackageName(where)), domain = NA)
             else
-                stop(gettextf("error in contained classes (%s) for class \"%s\"; previous definition restored to '%s'",
+                stop(gettextf("error in contained classes (%s) for class \"%s\"; previous definition restored to \"%s\"",
                               msg, Class, getPackageName(where)), domain = NA)
         }
+        if(length(attr(classDef@contains, "conflicts")) > 0)
+          .reportSuperclassConflicts(Class, classDef@contains, where)
+        .checkRequiredGenerics(Class, classDef, where)
         if(sealed) {
             classDef@sealed <- TRUE
-            assignClassDef(Class, classDef, where)
         }
     }
-    Class
+    if(S3methods)
+      classDef <- .setS3MethodsOn(classDef)
+    assignClassDef(Class, classDef, where)
+     Class
 }
 
 representation <-
@@ -74,19 +104,19 @@ representation <-
     ## unlike the S-Plus function, this does not form the class representation,
     ## since set SClass works separately with the slots and extends arguments.
     anames <- allNames(value)
-    for(i in seq(along=value)) {
+    for(i in seq_along(value)) {
         ei <- el(value, i)
-        if(!is.character(ei) || length(ei) != 1)
+        if(!is.character(ei) || length(ei) != 1L)
             stop(gettextf("element %d of the representation was not a single character string", i), domain = NA)
     }
-    includes <- as.character(value[nchar(anames)==0])
-    if(any(duplicated(includes)))
+    includes <- as.character(value[!nzchar(anames)])
+    if(anyDuplicated(includes))
         stop(gettextf("duplicate class names among superclasses: %s",
                       paste(dQuote(includes[duplicated(includes)]),
                             collapse = ", ")),
              domain = NA)
-    slots <- anames[nchar(anames)>0]
-    if(any(duplicated(slots)))
+    slots <- anames[nzchar(anames)]
+    if(anyDuplicated(slots))
        stop(gettextf("duplicated slot names: %s",
                      paste(sQuote(slots[duplicated(slots)]), collapse="")),
             domain = NA)
@@ -102,18 +132,18 @@ prototype <- function(...)
 .prototype <- function(...) {
     props <- list(...)
     names <- allNames(props)
-    data <- nchar(names) == 0
+    data <- !nzchar(names)
     dataPart <- any(data)
     if(dataPart) {
         if(sum(data) > 1)
             stop("only one data object (unnamed argument to prototype) allowed")
-        obj <- unclass(props[[seq(along=data)[data] ]])
+        obj <- unclass(props[[seq_along(data)[data] ]])
         props <- props[!data]
         names <- names[!data]
     }
     else
-        obj <- list()
-    for(i in seq(along = names))
+        obj <- defaultPrototype()
+    for(i in seq_along(names))
         slot(obj, names[[i]], FALSE) <- props[[i]]
     new("classPrototypeDef", object = obj, slots = names, dataPart = dataPart)
 }
@@ -125,14 +155,16 @@ makeClassRepresentation <-
   ## Users should call setClass instead of this function.
   function(name, slots = list(), superClasses = character(), prototype = NULL, package, validity = NULL, access = list(), version = .newExternalptr(), sealed = FALSE, virtual = NA, where)
 {
-    if(!is.null(prototype) || length(slots)>0 || length(superClasses) >0) {
+    if(any(superClasses %in% .AbnormalTypes))
+        superClasses <- .addAbnormalDataType(superClasses)
+    if(!is.null(prototype) || length(slots) || length(superClasses)) {
         ## collect information about slots, create prototype if needed
         pp <- reconcilePropertiesAndPrototype(name, slots, prototype, superClasses, where)
         slots <- pp$properties
         prototype <- pp$prototype
     }
-    contains <- list();
-    if(nchar(package)>0)
+    contains <- list()
+    if(nzchar(package))
         packageSlot(name) <- package
     for(what in superClasses) {
         if(is(what, "classRepresentation"))
@@ -153,6 +185,11 @@ makeClassRepresentation <-
         if(virtual && !is.na(match("VIRTUAL", superClasses)))
             elNamed(contains, "VIRTUAL") <- NULL
     }
+    # new() must return an S4 object, except perhaps for basic classes
+    if(!is.null(prototype) && is.na(match(name, .BasicClasses)))
+      prototype <- .asS4(prototype)
+    if(".S3Class" %in% names(slots))
+      prototype <- .addS3Class(name, prototype, contains, where)
     newClassRepresentation(className = name, slots = slots,
                            contains = contains,
                            prototype = prototype,
@@ -166,27 +203,33 @@ makeClassRepresentation <-
 
 getClassDef <-
   ## Get the definition of the class supplied as a string.
-  function(Class, where = topenv(parent.frame()), package = packageSlot(Class))
+  function(Class, where = topenv(parent.frame()), package = packageSlot(Class),
+           inherits = TRUE)
 {
     ## FIXME:  really wants to be is(Class, "classRepresentation") but
     ## generates inf. loop in booting methods package (also for new())
     if(.identC(class(Class), "classRepresentation"))
         return(Class)
-    if(length(Class)>1)
-        ## S3 class; almost certainly has no packageSLot, but we'll continue anyway
-        cname <- classMetaName(Class[[1]])
+    if(inherits)
+      value <- .getClassFromCache(Class, where)
     else
-        cname <- classMetaName(Class)
-    value <- NULL
-    ## a string with a package slot strongly implies the class definition
-    ## should be in that package.
-    if(!is.null(package)) {
-        whereP <- .requirePackage(package)
-        if(exists(cname, whereP))
-            value <- get(cname, whereP)
+      value <- NULL
+    if(is.null(value)) {
+	cname <-
+	    classMetaName(if(length(Class) > 1L)
+			  ## S3 class; almost certainly has no packageSlot,
+			  ## but we'll continue anyway
+			  Class[[1L]] else Class)
+	## a string with a package slot strongly implies the class definition
+	## should be in that package.
+	if(identical(nzchar(package), TRUE)) {
+	    whereP <- .requirePackage(package)
+	    if(exists(cname, whereP, inherits = inherits))
+		value <- get(cname, whereP)
+	}
+	if(is.null(value) && exists(cname, where, inherits = inherits))
+	    value <- get(cname, where)
     }
-    if(is.null(value) && exists(cname, where))
-        value <- get(cname, where)
     value
 }
 
@@ -237,9 +280,10 @@ checkSlotAssignment <- function(obj, name, value)
        return(value)
     ## check the value, but be careful to use the definition of the slot's class from
     ## the class environment of obj (change validObject too if a better way is found)
-    ok <- possibleExtends(valueClass, slotClass, ClassDef2 = getClassDef(slotClass, where = .classEnv(ClassDef)))
+    ok <- possibleExtends(valueClass, slotClass,
+                          ClassDef2 = getClassDef(slotClass, where = .classEnv(ClassDef)))
     if(identical(ok, FALSE))
-       stop(gettextf("assignment of an object of class \"%s\" is not valid for slot '%s' in an object of class \"%s\"; is(value, \"%s\") is not TRUE",
+       stop(gettextf("assignment of an object of class \"%s\" is not valid for slot \"%s\" in an object of class \"%s\"; is(value, \"%s\") is not TRUE",
                      class(value),  name, class(obj), slotClass),
             domain = NA)
     else if(identical(ok, TRUE))
@@ -273,7 +317,7 @@ slotNames <- function(x)
 .slotNames <- function(x)
 {
     classDef <-
-	getClassDef(if(is.character(x) && length(x) == 1) x else class(x))
+	getClassDef(if(identical(class(x), "character") && length(x) == 1L) x else class(x))
     if(is.null(classDef))
 	character()
     else
@@ -281,20 +325,32 @@ slotNames <- function(x)
 }
 
 
-removeClass <-  function(Class, where) {
+removeClass <-  function(Class, where = topenv(parent.frame())) {
     if(missing(where)) {
-        where <- findClass(Class, topenv(parent.frame()))
-        if(length(where) == 0) {
-            warning(gettextf("\"%s\" is not a class (no action taken)", Class),
+       classEnv <- .classEnv(Class, where, FALSE)
+        classWhere <- findClass(Class, where = classEnv)
+        if(length(classWhere) == 0L) {
+            warning(gettextf("Class definition for \"%s\" not found  (no action taken)", Class),
                     domain = NA)
             return(FALSE)
         }
-        if(length(where) > 1)
+        if(length(classWhere) > 1L)
             warning(gettextf("class \"%s\" has multiple definitions visible; only the first removed", Class), domain = NA)
-        where <- where[[1]]
+        classWhere <- classWhere[[1L]]
     }
+    else classWhere <- where
+    classDef <- getClassDef(Class, where=classWhere)
+    if(length(classDef@subclasses)) {
+      subclasses <- names(classDef@subclasses)
+      found <- sapply(subclasses, isClass, where = where)
+      for(what in subclasses[found])
+          .removeSuperClass(what, Class)
+    }
+    .removeSuperclassBackRefs(Class, classDef, classWhere)
+    .uncacheClass(Class, classDef)
+    .undefineMethod("initialize", Class, classWhere)
     what <- classMetaName(Class)
-    rm(list=what, pos=where)
+    rm(list=what, pos=classWhere)
     TRUE
 }
 
@@ -331,12 +387,7 @@ new <-
         ClassDef <- Class
     else
         ClassDef <- getClass(Class, where = topenv(parent.frame()))
-    if(identical(ClassDef@virtual, TRUE)) {
-        stop("trying to use new() on a virtual class")
-    }
-    else
-        value <- ClassDef@prototype
-    class(value) <- ClassDef@className
+    value <- .Call("R_do_new_object", ClassDef, PACKAGE = "base")
     initialize(value, ...)
 }
 
@@ -359,7 +410,7 @@ getClasses <-
         clNames <- objects(where, pattern = pat, all.names = TRUE)
     ## strip off the leading pattern (this implicitly assumes the characters
     ## in classMetaName("") are either "." or not metacharacters
-    substring(clNames, nchar(pat))
+    substring(clNames, nchar(pat, "c"))
 }
 
 
@@ -374,94 +425,118 @@ validObject <- function(object, test = FALSE, complete = FALSE)
     errors <- character()
     slotTypes <- classDef@slots
     slotNames <- names(slotTypes)
-    for(i in seq(along=slotTypes)) {
-        classi <- slotTypes[[i]]
-        sloti <- slot(object, slotNames[[i]])
-        classDefi <- getClassDef(classi, where = where)
-        if(is.null(classDefi)) {
-          errors <- c(errors, paste("class for slot \"", slotNames[[i]],
-                                    "\" (\"", classi, "\")", sep=""))
-          next
+    attrNames <- c(".Data", ".S3Class", names(attributes(object)))
+    if(any(is.na(match(slotNames, attrNames)))) {
+        badSlots <- is.na(match(slotNames, attrNames))
+        errors <- c(errors, paste("slots in class definition but not in object:", paste('"', slotNames[badSlots], '"', sep="", collapse = ", ")))
+        slotTypes <- slotTypes[!badSlots]
+        slotNames <- slotNames[!badSlots]
+    }
+    for(i in seq_along(slotTypes)) {
+	classi <- slotTypes[[i]]
+	classDefi <- getClassDef(classi, where = where)
+	if(is.null(classDefi)) {
+	    errors <- c(errors, paste("undefined class for slot \"", slotNames[[i]],
+				      "\" (\"", classi, "\")", sep=""))
+	    next
+	}
+        namei <- slotNames[[i]]
+        sloti <- try(switch(namei,
+                            ## .S3Class for S3 objects (e.g., "factor")
+                            .S3Class = S3Class(object),
+                            slot(object, namei)
+                            ), silent = TRUE)
+        if(inherits(sloti, "try-error")) {
+           errors <- c(errors, sloti)
+           next
         }
-        ## note that the use of possibleExtends is shared with checkSlotAssignment(), in case a
-        ## future revision improves on it!
-        ok <- possibleExtends(class(sloti), classi, ClassDef2 = classDefi)
-        if(identical(ok, FALSE)) {
-            errors <- c(errors,
-                        paste("invalid object for slot \"", slotNames[[i]],
-                              "\" in class \"", Class,
-                              "\": got class \"", class(sloti),
-                              "\", should be or extend class \"", classi, "\"", sep = ""))
-            next
-        }
-        if(!complete)
+	## note that the use of possibleExtends is shared with checkSlotAssignment(), in case a
+	## future revision improves on it!
+	ok <- possibleExtends(class(sloti), classi, ClassDef2 = classDefi)
+	if(identical(ok, FALSE)) {
+	    errors <- c(errors,
+			paste("invalid object for slot \"", slotNames[[i]],
+			      "\" in class \"", Class,
+			      "\": got class \"", class(sloti),
+			      "\", should be or extend class \"", classi, "\"",
+			      sep = ""))
+	    next
+	}
+	if(!complete)
           next
-        validityMethod <- classDefi@validity
-        if(is(validityMethod, "function")) {
-            errori <- anyStrings(validityMethod(sloti))
-            if(length(errori)>0) {
-                errori <- paste("In slot \"", slotNames[[i]],
-                                "\" of class \"", class(sloti), "\": ",
-                                sep = "", errori)
-                errors <- c(errors, errori)
-            }
+        errori <- anyStrings(Recall(sloti, TRUE, TRUE))
+        if(length(errori)) {
+            errori <- paste("In slot \"", slotNames[[i]],
+                            "\" of class \"", class(sloti), "\": ",
+                            errori, sep = "")
+            errors <- c(errors, errori)
         }
     }
-    extends <- rev(classDef@contains); i <- 1
-    while(length(errors) == 0 && i <= length(extends)) {
-        exti <- extends[[i]]
-        superClass <- exti@superClass
-        i <- i+1
-        if(!exti@simple && !is(object, superClass))
-            next ## skip conditional relations that don't hold for this object
-        superDef <- getClassDef(superClass, where = where)
-        if(is.null(superDef)) {
-            errors <- c(errors, paste("superclass \"", superClass, "\" not defined in the environment of the object's class", sep=""))
-            next
-        }
-        validityMethod <- superDef@validity
-        if(is(validityMethod, "function"))
-            errors <- c(errors, anyStrings(validityMethod(as(object, superClass))))
+    extends <- rev(classDef@contains)
+    for(i in seq_along(extends)) {
+	exti <- extends[[i]]
+	superClass <- exti@superClass
+	if(!exti@simple && !is(object, superClass))
+	    next ## skip conditional relations that don't hold for this object
+	superDef <- getClassDef(superClass, where = where)
+	if(is.null(superDef)) {
+	    errors <- c(errors,
+			paste("superclass \"", superClass,
+			      "\" not defined in the environment of the object's class",
+			      sep=""))
+	    break
+	}
+	validityMethod <- superDef@validity
+	if(is(validityMethod, "function")) {
+	    errors <- c(errors, anyStrings(validityMethod(as(object, superClass))))
+	    if(length(errors))
+		break
+	}
     }
     validityMethod <- classDef@validity
-    if(length(errors) == 0 && is(validityMethod, "function")) {
-        errors <- c(errors, anyStrings(validityMethod(object)))
+    if(length(errors) == 0L && is(validityMethod, "function")) {
+	errors <- c(errors, anyStrings(validityMethod(object)))
     }
-    if(length(errors) > 0) {
-        if(test)
-            errors
-        else {
-            msg <- gettextf("invalid class \"%s\" object:", Class)
-            if(length(errors) > 1)
-                stop(paste(msg,
-                           paste(paste(1:length(errors), errors, sep=": ")),
-                           collapse = "\n"), domain = NA)
-            else stop(msg, " ", errors, domain = NA)
+    if(length(errors)) {
+	if(test)
+	    errors
+	else {
+	    msg <- gettextf("invalid class \"%s\" object:", Class)
+	    if(length(errors) > 1L)
+		stop(paste(msg,
+			   paste(paste(seq_along(errors), errors, sep=": ")),
+			   collapse = "\n"), domain = NA)
+	    else stop(msg, " ", errors, domain = NA)
+	}
     }
-}
-  else
-    TRUE
+    else
+	TRUE
 }
 
-setValidity <-
-  function(Class, method, where = topenv(parent.frame())) {
+setValidity <- function(Class, method, where = topenv(parent.frame())) {
     if(isClassDef(Class)) {
-        ClassDef <- Class
-        Class <- ClassDef@className
+	ClassDef <- Class
+	Class <- ClassDef@className
     }
     else {
-      ClassDef <- getClassDef(Class, where)
-  }
+	ClassDef <- getClassDef(Class, where)
+    }
     method <- .makeValidityMethod(Class, method)
     if(is.null(method) ||
-      (is(method, "function") && length(formalArgs(method))==1))
-      ClassDef@validity <- method
+       (is(method, "function") && length(formalArgs(method)) == 1L))
+	ClassDef@validity <- method
     else
-      stop("validity method must be NULL or a function of one argument")
+	stop("validity method must be NULL or a function of one argument")
     ## TO DO:  check the where argument against the package of the class def.
     assignClassDef(Class, ClassDef, where = where)
     resetClass(Class, ClassDef, where = where)
-  }
+}
+
+getValidity <- function (ClassDef) {
+    ## "needed" according to ../man/validObject.Rd
+    ClassDef@validity
+}
+
 
 resetClass <- function(Class, classDef, where) {
         if(is(Class, "classRepresentation")) {
@@ -473,14 +548,14 @@ resetClass <- function(Class, classDef, where) {
         else {
             if(missing(where)) {
                 if(missing(classDef))
-                    where <- findClass(Class, unique = "resetting the definition")[[1]]
+                    where <- findClass(Class, unique = "resetting the definition")[[1L]]
                 else
                     where <- .classDefEnv(classDef)
             }
             if(missing(classDef)) {
                 classDef <- getClassDef(Class, where)
                 if(is.null(classDef)) {
-                    warning(gettextf("class \"%s\" not found on '%s'; 'resetClass' will have no effect", Class, getPackageName(where)), domain = NA)
+                    warning(gettextf("class \"%s\" not found on \"%s\"; 'resetClass' will have no effect", Class, getPackageName(where)), domain = NA)
                     return(classDef)
                 }
             }
@@ -503,7 +578,7 @@ resetClass <- function(Class, classDef, where) {
 
 initialize <- function(.Object, ...) {
     args <- list(...)
-    if(length(args) > 0) {
+    if(length(args)) {
         Class <- class(.Object)
         ## the basic classes have fixed definitions
         if(!is.na(match(Class, .BasicClasses)))
@@ -511,21 +586,23 @@ initialize <- function(.Object, ...) {
         ClassDef <- getClass(Class)
         ## separate the slots, superclass objects
         snames <- allNames(args)
-        which <- nchar(snames)>0
+        which <- nzchar(snames)
         elements <- args[which]
         supers <- args[!which]
         thisExtends <- names(ClassDef@contains)
         slotDefs <- ClassDef@slots
         dataPart <- elNamed(slotDefs, ".Data")
         if(is.null(dataPart)) dataPart <- "missing"
-        if(length(supers) > 0) {
-            for(i in rev(seq(along = supers))) {
+        if(length(supers)) {
+            for(i in rev(seq_along(supers))) {
                 obj <- el(supers, i)
                 Classi <- class(obj)
+                if(length(Classi) > 1L)
+                    Classi <- Classi[[1L]] #possible S3 inheritance
                 ## test some cases that let information be copied into the
                 ## object, ordered from more to less:  all the slots in the
                 ## first two cases, some in the 3rd, just the data part in 4th
-                if(.identC(Classi[[1]], Class))
+                if(.identC(Classi, Class))
                     .Object <- obj
                 else if(extends(Classi, Class))
                     .Object <- as(obj, Class, strict=FALSE)
@@ -536,23 +613,25 @@ initialize <- function(.Object, ...) {
                 else {
                     ## is there a class to which we can coerce obj
                     ## that is then among the superclasses of Class?
-                    extendsi <- extends(Classi)[-1]
+                    extendsi <- extends(Classi)[-1L]
                     ## look for the common extensions, choose the first
                     ## one in the extensions of Class
                     which <- match(thisExtends, extendsi)
-                    which <- seq(along=which)[!is.na(which)]
-                    if(length(which) >0 ) {
-                        Classi <- thisExtends[which[1]]
-                        as(.Object, Classi) <- as(obj, Classi, strict = FALSE)
+                    which <- seq_along(which)[!is.na(which)]
+                    if(length(which)) {
+                        Classi <- thisExtends[which[1L]]
+###                    was:    as(.Object, Classi) <- as(obj, Classi, strict = FALSE)
+                        ## but   as<- does an as(....) to its value argument
+                        as(.Object, Classi) <- obj
                     }
                     else
                         stop(gettextf("cannot use object of class \"%s\" in new():  class \"%s\" does not extend that class", Classi, Class), domain = NA)
                 }
             }
         }
-        if(length(elements)>0) {
+        if(length(elements)) {
             snames <- names(elements)
-            if(any(duplicated(snames)))
+	    if(anyDuplicated(snames))
                 stop(gettextf("duplicated slot names: %s",
                               paste(sQuote(snames[duplicated(snames)]),
                                     collapse = ", ")), domain = NA)
@@ -562,7 +641,8 @@ initialize <- function(.Object, ...) {
                               Class,
                               paste(snames[is.na(which)], collapse=", ")),
                      domain = NA)
-            for(i in seq(along=snames)) {
+            firstTime <- TRUE
+            for(i in seq_along(snames)) {
                 slotName <- el(snames, i)
                 slotClass <- elNamed(slotDefs, slotName)
                 slotClassDef <- getClassDef(slotClass, package=ClassDef@package)
@@ -578,7 +658,14 @@ initialize <- function(.Object, ...) {
                                          valClassDef, slotClassDef), FALSE))
                         slotVal <- as(slotVal, slotClass, strict = FALSE)
                 }
-                slot(.Object, slotName, check = FALSE) <- slotVal
+                if (firstTime) {
+                    ## force a copy of .Object
+                    slot(.Object, slotName, check = FALSE) <- slotVal
+                    firstTime <- FALSE
+                } else {
+                    ## XXX: do the assignment in-place
+                    "slot<-"(.Object, slotName, check = FALSE, slotVal)
+                }
             }
         }
         validObject(.Object)
@@ -589,27 +676,48 @@ initialize <- function(.Object, ...) {
 findClass <- function(Class, where = topenv(parent.frame()), unique = "") {
     if(is(Class, "classRepresentation")) {
         pkg <- Class@package
+        classDef <- Class
         Class <- Class@className
     }
-    else
-        pkg <- ""
-    if(missing(where) && nchar(pkg))
+    else {
+        pkg <- packageSlot(Class)
+        if(is.null(pkg))
+          pkg <- ""
+        classDef <- getClassDef(Class, where, pkg)
+    }
+    if(missing(where) && nzchar(pkg))
             where <- .requirePackage(pkg)
     else
         where <- as.environment(where)
     what <- classMetaName(Class)
     where <- .findAll(what, where)
-    if(length(where) != 1 && nchar(unique)>0) {
-            if(length(where) == 0)
-                stop(gettextf("no definition of \"%s\" to use for %s",
+    if(length(where) > 1L && nzchar(pkg)) {
+        pkgs <- sapply(where, function(db)get(what, db)@package)
+        where <- where[match(pkg, pkgs, 0L)]
+    }
+    else
+      pkgs <- pkg
+    if(length(where) != 1L) {
+            if(length(where) == 0L) {
+                if(is.null(classDef))
+                  classDef <- getClassDef(Class) # but won't likely succeed over previous
+                if(nzchar(unique)) {
+                    if(is(classDef, "classRepresentation"))
+                      stop(gettextf('Class "%s" is defined, with package "%s", but no corresponding metadata object was found (not exported?)',
+                                  Class, classDef@package), domain = NA)
+                    else
+                      stop(gettextf("no definition of \"%s\" to use for %s",
                               Class, unique), domain = NA)
-            if(length(where) > 1) {
-                where <- where[1]
+                }
+            }
+            else if(nzchar(unique)) {
+                where <- where[1L]
                 ## problem: 'unique'x is text passed in, so do not translate
-                warning(sprintf("multiple definitions of class \"%s\" visible; using the definition on package '%s' for %s",
-                                 Class, getPackageName(where[[1]]), unique),
+                warning(gettextf("multiple definitions of class \"%s\" visible; using the definition\n   in package \"%s\" for %s",
+                                 Class, getPackageName(where[[1L]]), unique),
                         domain = NA)
             }
+            ## else returns a list of >1 places, for the caller to sort out (e.g., .findOrCopyClass)
     }
     where
 }
@@ -634,3 +742,80 @@ sealClass <- function(Class, where = topenv(parent.frame())) {
     invisible(classDef)
 }
 
+## see $RHOME/src/main/duplicate.c for the corresponding datatypes
+## not copied by duplicate1
+.AbnormalTypes <- c("environment", "name", "primitive", "externalptr", "weakref", "NULL", "bytecode")
+
+.indirectAbnormalClasses <- c(environment = ".environment",
+                              name = ".name",
+                              primitive = ".primitive",
+                              weakref = ".weakref",
+                              "NULL" = ".NULL",
+                              bytecode = ".bytecode"
+                              )
+
+.addAbnormalDataType <- function(classes) {
+  types <- match(classes, .AbnormalTypes, 0) > 0
+  type = classes[types]
+  if(length(type) == 0)
+    return(classes)
+  if(length(type) > 1)
+    stop(gettextf("Class definition cannot extend more than one of these data types: %s", paste('"',type, '"', sep="", collapse = ", ")),
+         domain = NA)
+  class <- .indirectAbnormalClasses[type]
+  if(is.na(class) || is.null(getClassDef(class, .methodsNamespace)))
+    stop(gettextf("Sorry, abnormal type \"%s\" is not supported as a superclass of a class definition", type),
+         domain = NA)
+  ## this message is not strictly needed, but reminds programmers that
+  ## they will see an unexpected superclass
+  message(gettextf('Defining type "%s" as a superclass via class "%s"',
+                  type, class), domain = NA)
+  c(class, classes[!types])
+}
+
+.checkRequiredGenerics <- function(Class, classDef, where) {}
+
+..checkRequiredGenerics <- function(Class, classDef, where) {
+  ## If any of the superclasses are in the .NeedPrimitiveMethods
+  ## list, cache the corresponding generics now and also save their names in
+  ## .requireCachedGenerics to be used when the environment
+  ## where= is loaded.
+  supers <- names(classDef@contains)
+  allNeeded <- get(".NeedPrimitiveMethods", envir = .methodsNamespace)
+  specials <- names(allNeeded)
+  needed <- match(specials, supers, 0) > 0
+  if(any(needed)) {
+    generics <- unique(allNeeded[needed])
+    packages <- character()
+    for(g in generics) {
+      def <- getGeneric(g)
+      packages <- c(packages, def@package) # must be "methods" ?
+      cacheGenericsMetaData(g, def, TRUE, where, def@package)
+    }
+    if(exists(".requireCachedGenerics",  where, inherits = FALSE))
+      previous <- get(".requireCachedGenerics",  where)
+    else
+      previous <- character()
+    packages <- c(attr(previous, "package"), packages)
+    gg <- c(previous, generics)
+    attr(gg, "package") <- packages
+    assign(".requireCachedGenerics", gg, where)
+  }
+}
+
+.setS3MethodsOn <- function(classDef) {
+    ext <- extends(classDef)
+    slots <- classDef@slots
+    if(is.na(match(".S3Class", names(slots)))) {
+        ## add the slot if it's not there
+        slots$.S3Class <- getClass("oldClass")@slots$.S3Class
+        classDef@slots <- slots
+    }
+    ## in any case give the prototype the full extends as .S3Class
+    proto <- classDef@prototype
+    if(is.null(proto)) # simple virtual class--unlikely but valid
+        proto <- defaultPrototype()
+    attr(proto, ".S3Class") <- ext
+    classDef@prototype <- proto
+    classDef
+  }
