@@ -14,6 +14,20 @@
 #  A copy of the GNU General Public License is available at
 #  http://www.r-project.org/Licenses/
 
+## version to return NA for df = 0, as R did before 2.7.0
+safe_pchisq <- function(q, df, ...)
+{
+    df[df <= 0] <- NA
+    pchisq(q=q, df=df, ...)
+}
+## and to avoid a warning
+safe_pf <- function(q, df1, ...)
+{
+    df1[df1 <= 0] <- NA
+    pf(q=q, df1=df1, ...)
+}
+
+
 add1 <- function(object, scope, ...) UseMethod("add1")
 
 add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
@@ -29,10 +43,11 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 #     data <- model.frame(update(object, newform)) # remove NAs
 #     object <- update(object, data = data)
     ns <- length(scope)
-    ans <- matrix(nrow = ns + 1, ncol = 2,
+    ans <- matrix(nrow = ns + 1L, ncol = 2L,
                   dimnames = list(c("<none>", scope), c("df", "AIC")))
-    ans[1,  ] <- extractAIC(object, scale, k = k, ...)
+    ans[1L,  ] <- extractAIC(object, scale, k = k, ...)
     n0 <- length(object$residuals)
+    env <- environment(formula(object))
     for(i in seq(ns)) {
 	tt <- scope[i]
 	if(trace > 1) {
@@ -41,21 +56,21 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 	}
 	nfit <- update(object, as.formula(paste("~ . +", tt)),
                        evaluate = FALSE)
-        nfit <- eval.parent(nfit)
-	ans[i+1, ] <- extractAIC(nfit, scale, k = k, ...)
+	nfit <- eval(nfit, envir=env) # was  eval.parent(nfit)
+	ans[i+1L, ] <- extractAIC(nfit, scale, k = k, ...)
         if(length(nfit$residuals) != n0)
             stop("number of rows in use has changed: remove missing values?")
     }
-    dfs <- ans[,1] - ans[1,1]
-    dfs[1] <- NA
-    aod <- data.frame(Df = dfs, AIC = ans[,2])
+    dfs <- ans[, 1L] - ans[1L, 1L]
+    dfs[1L] <- NA
+    aod <- data.frame(Df = dfs, AIC = ans[, 2L])
     test <- match.arg(test)
     if(test == "Chisq") {
-	dev <- ans[,2] - k*ans[, 1]
-	dev <- dev[1] - dev; dev[1] <- NA
+	dev <- ans[, 2L] - k*ans[, 1L]
+	dev <- dev[1L] - dev; dev[1L] <- NA
 	nas <- !is.na(dev)
 	P <- dev
-	P[nas] <- pchisq(dev[nas], dfs[nas], lower.tail=FALSE)
+	P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail=FALSE)
 	aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
     }
     head <- c("Single term additions", "\nModel:",
@@ -66,6 +81,20 @@ add1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
     aod
 }
 
+check_exact <- function(object)
+{
+    w <- object$weights
+    if(is.null(w)) {
+        mss <- sum(object$fitted.values^2)
+        rss <- sum(object$residuals^2)
+    } else {
+        mss <- sum(w * object$fitted.values^2)
+        rss <- sum(w * object$residuals^2)
+    }
+    if(rss < 1e-10*mss)
+        warning("attempting model selection on an essentially perfect fit is nonsense", call. = FALSE)
+}
+
 add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 		    x = NULL, k = 2,...)
 {
@@ -77,10 +106,11 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 	Fs[df < .Machine$double.eps] <- NA
 	P <- Fs
 	nnas <- !is.na(Fs)
-	P[nnas] <- pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
+	P[nnas] <- safe_pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
 	list(Fs=Fs, P=P)
     }
 
+    check_exact(object)
     if(missing(scope) || is.null(scope)) stop("no terms in scope")
     if(!is.character(scope))
 	scope <- add.scope(object, update.formula(object, scope))
@@ -89,7 +119,8 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     oTerms <- attr(object$terms, "term.labels")
     int <- attr(object$terms, "intercept")
     ns <- length(scope)
-    y <- object$residuals + predict(object)
+    y <- object$residuals + object$fitted.values
+    ## predict(object) applies na.action where na.exclude results in too long
     dfs <- numeric(ns+1)
     RSS <- numeric(ns+1)
     names(dfs) <- names(RSS) <- c("<none>", scope)
@@ -121,21 +152,21 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     n <- nrow(x)
     Terms <- attr(Terms, "term.labels")
     asgn <- attr(x, "assign")
-    ousex <- match(asgn, match(oTerms, Terms), 0) > 0
-    if(int) ousex[1] <- TRUE
+    ousex <- match(asgn, match(oTerms, Terms), 0L) > 0L
+    if(int) ousex[1L] <- TRUE
     iswt <- !is.null(wt)
     X <- x[, ousex, drop = FALSE]
     z <- if(iswt) lm.wfit(X, y, wt, offset=offset)
     else lm.fit(X, y, offset=offset)
-    dfs[1] <- z$rank
+    dfs[1L] <- z$rank
     class(z) <- "lm" # needed as deviance.lm calls generic residuals()
-    RSS[1] <- deviance(z)
+    RSS[1L] <- deviance(z)
     ## workaround for PR#7842. terms.formula may have flipped interactions
     sTerms <- sapply(strsplit(Terms, ":", fixed=TRUE),
                      function(x) paste(sort(x), collapse=":"))
     for(tt in scope) {
-        stt <- paste(sort(strsplit(tt, ":")[[1]]), collapse=":")
-	usex <- match(asgn, match(stt, sTerms), 0) > 0
+        stt <- paste(sort(strsplit(tt, ":")[[1L]]), collapse=":")
+	usex <- match(asgn, match(stt, sTerms), 0L) > 0L
 	X <- x[, usex|ousex, drop = FALSE]
 	z <- if(iswt) lm.wfit(X, y, wt, offset=offset)
         else lm.fit(X, y, offset=offset)
@@ -145,9 +176,9 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     }
     if(scale > 0) aic <- RSS/scale - n + k*dfs
     else aic <- n * log(RSS/n) + k*dfs
-    dfs <- dfs - dfs[1]
-    dfs[1] <- NA
-    aod <- data.frame(Df = dfs, "Sum of Sq" = c(NA, RSS[1] - RSS[-1]),
+    dfs <- dfs - dfs[1L]
+    dfs[1L] <- NA
+    aod <- data.frame(Df = dfs, "Sum of Sq" = c(NA, RSS[1L] - RSS[-1L]),
 		      RSS = RSS, AIC = aic,
                       row.names = names(dfs), check.names = FALSE)
     if(scale > 0) names(aod) <- c("Df", "Sum of Sq", "RSS", "Cp")
@@ -156,16 +187,16 @@ add1.lm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
         dev <- aod$"Sum of Sq"
         if(scale == 0) {
             dev <- n * log(RSS/n)
-            dev <- dev[1] - dev
-            dev[1] <- NA
+            dev <- dev[1L] - dev
+            dev[1L] <- NA
         } else dev <- dev/scale
         df <- aod$Df
         nas <- !is.na(df)
-        dev[nas] <- pchisq(dev[nas], df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
 	rdf <- object$df.residual
-	aod[, c("F value", "Pr(F)")] <- Fstat(aod, aod$RSS[1], rdf)
+	aod[, c("F value", "Pr(F)")] <- Fstat(aod, aod$RSS[1L], rdf)
     }
     head <- c("Single term additions", "\nModel:",
 	      deparse(as.vector(formula(object))),
@@ -181,12 +212,12 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     Fstat <- function(table, rdf) {
 	dev <- table$Deviance
 	df <- table$Df
-	diff <- pmax(0, (dev[1] - dev)/df)
+	diff <- pmax(0, (dev[1L] - dev)/df)
 	Fs <- (diff/df)/(dev/(rdf-df))
 	Fs[df < .Machine$double.eps] <- NA
 	P <- Fs
 	nnas <- !is.na(Fs)
-	P[nnas] <- pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
+	P[nnas] <- safe_pf(Fs[nnas], df[nnas], rdf - df[nnas], lower.tail=FALSE)
 	list(Fs=Fs, P=P)
     }
     if(!is.character(scope))
@@ -214,7 +245,8 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
         wt <- model.weights(m)
 	x <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
         oldn <- length(y)
-        y <- model.response(m, "numeric")
+        y <- model.response(m)
+        if(!is.factor(y)) storage.mode(y) <- "double"
         ## binomial case has adjusted y and weights
         if(NCOL(y) == 2) {
             n <- y[, 1] + y[, 2]
@@ -235,19 +267,19 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     if(is.null(wt)) wt <- rep.int(1, n)
     Terms <- attr(Terms, "term.labels")
     asgn <- attr(x, "assign")
-    ousex <- match(asgn, match(oTerms, Terms), 0) > 0
-    if(int) ousex[1] <- TRUE
+    ousex <- match(asgn, match(oTerms, Terms), 0L) > 0L
+    if(int) ousex[1L] <- TRUE
     X <- x[, ousex, drop = FALSE]
     z <-  glm.fit(X, y, wt, offset=offset,
                   family=object$family, control=object$control)
-    dfs[1] <- z$rank
-    dev[1] <- z$deviance
+    dfs[1L] <- z$rank
+    dev[1L] <- z$deviance
     ## workaround for PR#7842. terms.formula may have flipped interactions
     sTerms <- sapply(strsplit(Terms, ":", fixed=TRUE),
                      function(x) paste(sort(x), collapse=":"))
     for(tt in scope) {
-        stt <- paste(sort(strsplit(tt, ":")[[1]]), collapse=":")
-	usex <- match(asgn, match(stt, sTerms), 0) > 0
+        stt <- paste(sort(strsplit(tt, ":")[[1L]]), collapse=":")
+	usex <- match(asgn, match(stt, sTerms), 0L) > 0L
 	X <- x[, usex|ousex, drop = FALSE]
 	z <-  glm.fit(X, y, wt, offset=offset,
 		      family=object$family, control=object$control)
@@ -263,20 +295,20 @@ add1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
 	else loglik <- n * log(dev/n)
     } else loglik <- dev/dispersion
     aic <- loglik + k * dfs
-    aic <- aic + (extractAIC(object, k = k)[2] - aic[1])
-    dfs <- dfs - dfs[1]
-    dfs[1] <- NA
+    aic <- aic + (extractAIC(object, k = k)[2L] - aic[1L])
+    dfs <- dfs - dfs[1L]
+    dfs[1L] <- NA
     aod <- data.frame(Df = dfs, Deviance = dev, AIC = aic,
 		      row.names = names(dfs), check.names = FALSE)
     if(all(is.na(aic))) aod <- aod[, -3]
     test <- match.arg(test)
     if(test == "Chisq") {
-        dev <- pmax(0, loglik[1] - loglik)
-        dev[1] <- NA
+        dev <- pmax(0, loglik[1L] - loglik)
+        dev[1L] <- NA
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
         nas <- !is.na(dev)
-        dev[nas] <- pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
@@ -312,10 +344,11 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 #    data <- model.frame(object) # remove NAs
 #    object <- update(object, data = data)
     ns <- length(scope)
-    ans <- matrix(nrow = ns + 1, ncol = 2,
+    ans <- matrix(nrow = ns + 1L, ncol = 2L,
                   dimnames =  list(c("<none>", scope), c("df", "AIC")))
-    ans[1,  ] <- extractAIC(object, scale, k = k, ...)
+    ans[1, ] <- extractAIC(object, scale, k = k, ...)
     n0 <- length(object$residuals)
+    env <- environment(formula(object))
     for(i in seq(ns)) {
 	tt <- scope[i]
 	if(trace > 1) {
@@ -324,21 +357,21 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
         }
         nfit <- update(object, as.formula(paste("~ . -", tt)),
                        evaluate = FALSE)
-        nfit <- eval.parent(nfit)
+	nfit <- eval(nfit, envir=env) # was  eval.parent(nfit)
 	ans[i+1, ] <- extractAIC(nfit, scale, k = k, ...)
         if(length(nfit$residuals) != n0)
             stop("number of rows in use has changed: remove missing values?")
     }
-    dfs <- ans[1,1] - ans[,1]
-    dfs[1] <- NA
+    dfs <- ans[1L , 1L] - ans[, 1L]
+    dfs[1L] <- NA
     aod <- data.frame(Df = dfs, AIC = ans[,2])
     test <- match.arg(test)
     if(test == "Chisq") {
-        dev <- ans[, 2] - k*ans[, 1]
-        dev <- dev - dev[1] ; dev[1] <- NA
+        dev <- ans[, 2L] - k*ans[, 1L]
+        dev <- dev - dev[1L] ; dev[1L] <- NA
         nas <- !is.na(dev)
         P <- dev
-        P[nas] <- pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
+        P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
         aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
     }
     head <- c("Single term deletions", "\nModel:",
@@ -352,6 +385,7 @@ drop1.default <- function(object, scope, scale = 0, test=c("none", "Chisq"),
 drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
 		     test=c("none", "Chisq", "F"), k = 2, ...)
 {
+    check_exact(object)
     x <- model.matrix(object)
     offset <- model.offset(model.frame(object))
     iswt <- !is.null(wt <- object$weights)
@@ -371,12 +405,12 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
     chisq <- deviance.lm(object)
     dfs <- numeric(ns)
     RSS <- numeric(ns)
-    y <- object$residuals + predict(object)
-    na.coef <- (1:length(object$coefficients))[!is.na(object$coefficients)]
-    for(i in 1:ns) {
+    y <- object$residuals + object$fitted.values
+    ## predict(object) applies na.action where na.exclude results in too long
+    na.coef <- seq_along(object$coefficients)[!is.na(object$coefficients)]
+    for(i in 1L:ns) {
 	ii <- seq_along(asgn)[asgn == ndrop[i]]
-	if(all.cols) jj <- setdiff(seq(ncol(x)), ii)
-	else jj <- setdiff(na.coef, ii)
+	jj <- setdiff(if(all.cols) seq(ncol(x)) else na.coef, ii)
 	z <- if(iswt) lm.wfit(x[, jj, drop = FALSE], y, wt, offset=offset)
 	else lm.fit(x[, jj, drop = FALSE], y, offset=offset)
 	dfs[i] <- z$rank
@@ -388,9 +422,9 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
     RSS <- c(chisq, RSS)
     if(scale > 0) aic <- RSS/scale - n + k*dfs
     else aic <- n * log(RSS/n) + k*dfs
-    dfs <- dfs[1] - dfs
-    dfs[1] <- NA
-    aod <- data.frame(Df = dfs, "Sum of Sq" = c(NA, RSS[-1] - RSS[1]),
+    dfs <- dfs[1L] - dfs
+    dfs[1L] <- NA
+    aod <- data.frame(Df = dfs, "Sum of Sq" = c(NA, RSS[-1L] - RSS[1L]),
 		      RSS = RSS, AIC = aic,
                       row.names = scope, check.names = FALSE)
     if(scale > 0) names(aod) <- c("Df", "Sum of Sq", "RSS", "Cp")
@@ -399,23 +433,23 @@ drop1.lm <- function(object, scope, scale = 0, all.cols = TRUE,
         dev <- aod$"Sum of Sq"
         if(scale == 0) {
             dev <- n * log(RSS/n)
-            dev <- dev - dev[1]
-            dev[1] <- NA
+            dev <- dev - dev[1L]
+            dev[1L] <- NA
         } else dev <- dev/scale
         df <- aod$Df
         nas <- !is.na(df)
-        dev[nas] <- pchisq(dev[nas], df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
 	dev <- aod$"Sum of Sq"
 	dfs <- aod$Df
 	rdf <- object$df.residual
-	rms <- aod$RSS[1]/rdf
+	rms <- aod$RSS[1L]/rdf
 	Fs <- (dev/dfs)/rms
 	Fs[dfs < 1e-4] <- NA
 	P <- Fs
 	nas <- !is.na(Fs)
-	P[nas] <- pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
+	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
 	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
     }
     head <- c("Single term deletions", "\nModel:",
@@ -451,11 +485,14 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
     dfs <- numeric(ns)
     dev <- numeric(ns)
     y <- object$y
-    if(is.null(y)) y <- model.response(model.frame(object), "numeric")
-#    na.coef <- (1:length(object$coefficients))[!is.na(object$coefficients)]
+    if(is.null(y)) {
+        y <- model.response(model.frame(object))
+        if(!is.factor(y)) storage.mode(y) <- "double"
+    }
+#    na.coef <- seq_along(object$coefficients)[!is.na(object$coefficients)]
     wt <- object$prior.weights
     if(is.null(wt)) wt <- rep.int(1, n)
-    for(i in 1:ns) {
+    for(i in 1L:ns) {
 	ii <- seq_along(asgn)[asgn == ndrop[i]]
 	jj <- setdiff(seq(ncol(x)), ii)
 	z <-  glm.fit(x[, jj, drop = FALSE], y, wt, offset=object$offset,
@@ -475,35 +512,35 @@ drop1.glm <- function(object, scope, scale = 0, test=c("none", "Chisq", "F"),
             if(scale > 0) dev/scale - n else n * log(dev/n)
         } else dev/dispersion
     aic <- loglik + k * dfs
-    dfs <- dfs[1] - dfs
-    dfs[1] <- NA
-    aic <- aic + (extractAIC(object, k = k)[2] - aic[1])
+    dfs <- dfs[1L] - dfs
+    dfs[1L] <- NA
+    aic <- aic + (extractAIC(object, k = k)[2L] - aic[1L])
     aod <- data.frame(Df = dfs, Deviance = dev, AIC = aic,
 		      row.names = scope, check.names = FALSE)
     if(all(is.na(aic))) aod <- aod[, -3]
     test <- match.arg(test)
     if(test == "Chisq") {
-        dev <- pmax(0, loglik - loglik[1])
-        dev[1] <- NA
+        dev <- pmax(0, loglik - loglik[1L])
+        dev[1L] <- NA
         nas <- !is.na(dev)
         LRT <- if(dispersion == 1) "LRT" else "scaled dev."
         aod[, LRT] <- dev
-        dev[nas] <- pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
+        dev[nas] <- safe_pchisq(dev[nas], aod$Df[nas], lower.tail=FALSE)
         aod[, "Pr(Chi)"] <- dev
     } else if(test == "F") {
         if(fam == "binomial" || fam == "poisson")
             warning(gettextf("F test assumes 'quasi%s' family", fam),
                     domain = NA)
 	dev <- aod$Deviance
-	rms <- dev[1]/rdf
-        dev <- pmax(0, dev - dev[1])
+	rms <- dev[1L]/rdf
+        dev <- pmax(0, dev - dev[1L])
 	dfs <- aod$Df
 	rdf <- object$df.residual
 	Fs <- (dev/dfs)/rms
 	Fs[dfs < 1e-4] <- NA
 	P <- Fs
 	nas <- !is.na(Fs)
-	P[nas] <- pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
+	P[nas] <- safe_pf(Fs[nas], dfs[nas], rdf, lower.tail=FALSE)
 	aod[, c("F value", "Pr(F)")] <- list(Fs, P)
     }
     head <- c("Single term deletions", "\nModel:",
@@ -526,7 +563,7 @@ add.scope <- function(terms1, terms2)
 drop.scope <- function(terms1, terms2)
 {
     terms1 <- terms(terms1)
-    f2 <- if(missing(terms2)) numeric(0)
+    f2 <- if(missing(terms2)) numeric(0L)
     else attr(terms(terms2), "factors")
     factor.scope(attr(terms1, "factors"), list(drop = f2))$drop
 }
@@ -547,7 +584,7 @@ factor.scope <- function(factor, scope)
                              function(x) paste(sort(x), collapse=":"))
             nmdrop0 <- sapply(strsplit(nmdrop, ":", fixed=TRUE),
                              function(x) paste(sort(x), collapse=":"))
-	    where <- match(nmdrop0, nmfac0, 0)
+	    where <- match(nmdrop0, nmfac0, 0L)
 	    if(any(!where))
                 stop(gettextf("lower scope has term(s) %s not included in model",
                               paste(sQuote(nmdrop[where==0]), collapse=", ")),
@@ -562,9 +599,9 @@ factor.scope <- function(factor, scope)
 	    for(i in seq(keep)) keep[i] <- max(f[i, - i]) != f[i, i]
 	    nmdrop <- nmdrop[keep]
 	}
-    } else nmdrop <- character(0)
+    } else nmdrop <- character(0L)
 
-    if(!length(add)) nmadd <- character(0)
+    if(!length(add)) nmadd <- character(0L)
     else {
 	nmfac <- colnames(factor)
 	nmadd <- colnames(add)
@@ -575,10 +612,10 @@ factor.scope <- function(factor, scope)
                              function(x) paste(sort(x), collapse=":"))
             nmadd0 <- sapply(strsplit(nmadd, ":", fixed=TRUE),
                              function(x) paste(sort(x), collapse=":"))
-	    where <- match(nmfac0, nmadd0, 0)
+	    where <- match(nmfac0, nmadd0, 0L)
 	    if(any(!where))
                 stop(gettextf("upper scope does not include model term(s) %s",
-                              paste(sQuote(nmfac[where==0]), collapse=", ")),
+                              paste(sQuote(nmfac[where==0L]), collapse=", ")),
                      domain = NA)
 	    nmadd <- nmadd[-where]
 	    add <- add[, -where, drop = FALSE]
@@ -606,7 +643,7 @@ step <- function(object, scope, scale = 0,
 # 	    tmp <- "1"
 #         tmp <- paste("~", paste(tmp, collapse = " + "))
 #         form <- formula(object) # some formulae have no lhs
-#         tmp <- if(length(form) > 2) paste(deparse(form[[2]]), tmp)
+#         tmp <- if(length(form) > 2) paste(deparse(form[[2L]]), tmp)
 #         ## must be as.character as deparse gives spurious ()
 # 	if (length(offset <- attr(tt, "offset")))
 # 	    tmp <- paste(tmp, as.character(attr(tt, "variables")[offset + 1]),
@@ -618,18 +655,18 @@ step <- function(object, scope, scale = 0,
     mydeviance <- function(x, ...)
     {
         dev <- deviance(x)
-        if(!is.null(dev)) dev else extractAIC(x, k=0)[2]
+        if(!is.null(dev)) dev else extractAIC(x, k=0)[2L]
     }
 
     cut.string <- function(string)
     {
 	if(length(string) > 1)
-	    string[-1] <- paste("\n", string[-1], sep = "")
+	    string[-1L] <- paste("\n", string[-1L], sep = "")
 	string
     }
     re.arrange <- function(keep)
     {
-	namr <- names(k1 <- keep[[1]])
+	namr <- names(k1 <- keep[[1L]])
 	namc <- names(keep)
 	nc <- length(keep)
 	nr <- length(k1)
@@ -669,7 +706,7 @@ step <- function(object, scope, scale = 0,
     backward <- direction == "both" | direction == "backward"
     forward  <- direction == "both" | direction == "forward"
     if(missing(scope)) {
-	fdrop <- numeric(0)
+	fdrop <- numeric(0L)
         fadd <- attr(Terms, "factors")
         if(md) forward <- FALSE
     }
@@ -677,14 +714,14 @@ step <- function(object, scope, scale = 0,
 	if(is.list(scope)) {
 	    fdrop <- if(!is.null(fdrop <- scope$lower))
 		attr(terms(update.formula(object, fdrop)), "factors")
-	    else numeric(0)
+	    else numeric(0L)
 	    fadd <- if(!is.null(fadd <- scope$upper))
 		attr(terms(update.formula(object, fadd)), "factors")
 	}
         else {
 	    fadd <- if(!is.null(fadd <- scope))
 		attr(terms(update.formula(object, scope)), "factors")
-	    fdrop <- numeric(0)
+	    fdrop <- numeric(0L)
 	}
     }
     models <- vector("list", steps)
@@ -692,8 +729,8 @@ step <- function(object, scope, scale = 0,
     n <- length(object$residuals)
     fit <- object
     bAIC <- extractAIC(fit, scale, k = k, ...)
-    edf <- bAIC[1]
-    bAIC <- bAIC[2]
+    edf <- bAIC[1L]
+    bAIC <- bAIC[2L]
     if(is.na(bAIC))
         stop("AIC is not defined for this model, so 'step' cannot proceed")
     nm <- 1
@@ -717,12 +754,12 @@ step <- function(object, scope, scale = 0,
 	    aod <- drop1(fit, scope$drop, scale = scale,
                          trace = trace, k = k, ...)
 	    rn <- row.names(aod)
-	    row.names(aod) <- c(rn[1], paste("-", rn[-1], sep=" "))
+	    row.names(aod) <- c(rn[1L], paste("-", rn[-1L], sep=" "))
             ## drop zero df terms first: one at time since they
             ## may mask each other
 	    if(any(aod$Df == 0, na.rm=TRUE)) {
 		zdf <- aod$Df == 0 & !is.na(aod$Df)
-		change <- rev(rownames(aod)[zdf])[1]
+		change <- rev(rownames(aod)[zdf])[1L]
 	    }
 	}
 	if(is.null(change)) {
@@ -730,7 +767,7 @@ step <- function(object, scope, scale = 0,
 		aodf <- add1(fit, scope$add, scale = scale,
                              trace = trace, k = k, ...)
 		rn <- row.names(aodf)
-		row.names(aodf) <- c(rn[1], paste("+", rn[-1], sep=" "))
+		row.names(aodf) <- c(rn[1L], paste("+", rn[-1L], sep=" "))
 		aod <-
                     if(is.null(aod)) aodf
                     else rbind(aod, aodf[-1, , drop = FALSE])
@@ -742,13 +779,13 @@ step <- function(object, scope, scale = 0,
 	    aod <- aod[nzdf, ]
 	    if(is.null(aod) || ncol(aod) == 0) break
 	    nc <- match(c("Cp", "AIC"), names(aod))
-	    nc <- nc[!is.na(nc)][1]
+	    nc <- nc[!is.na(nc)][1L]
 	    o <- order(aod[, nc])
 	    if(trace) print(aod[o, ])
-	    if(o[1] == 1) break
-	    change <- rownames(aod)[o[1]]
+	    if(o[1L] == 1) break
+	    change <- rownames(aod)[o[1L]]
 	}
-	usingCp <- match("Cp", names(aod), 0) > 0
+	usingCp <- match("Cp", names(aod), 0L) > 0L
         ## may need to look for a `data' argument in parent
 	fit <- update(fit, paste("~ .", change), evaluate = FALSE)
         fit <- eval.parent(fit)
@@ -756,8 +793,8 @@ step <- function(object, scope, scale = 0,
             stop("number of rows in use has changed: remove missing values?")
         Terms <- terms(fit)
 	bAIC <- extractAIC(fit, scale, k = k, ...)
-	edf <- bAIC[1]
-	bAIC <- bAIC[2]
+	edf <- bAIC[1L]
+	bAIC <- bAIC[2L]
 	if(trace)
 	    cat("\nStep:  AIC=", format(round(bAIC, 2)), "\n",
 		cut.string(deparse(as.vector(formula(fit)))), "\n\n", sep='')
@@ -787,7 +824,7 @@ extractAIC.coxph <- function(fit, scale, k = 2, ...)
 extractAIC.survreg <- function(fit, scale, k = 2, ...)
 {
     edf <- sum(fit$df)
-    c(edf, -2 * fit$loglik[2] + k * edf)
+    c(edf, -2 * fit$loglik[2L] + k * edf)
 }
 
 extractAIC.glm <- function(fit, scale = 0, k = 2, ...)
