@@ -1,3 +1,19 @@
+#  File src/library/stats/R/glm.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ### This function fits a generalized linear model via
 ### iteratively reweighted least squares for any family.
 ### Written by Simon Davies, Dec 1995
@@ -17,41 +33,52 @@ glm <- function(formula, family = gaussian, data, weights,
     if(is.function(family)) family <- family()
     if(is.null(family$family)) {
 	print(family)
-	stop("`family' not recognized")
+	stop("'family' not recognized")
     }
 
     ## extract x, y, etc from the model formula and frame
-#    mt <- terms(formula, data = data)
     if(missing(data)) data <- environment(formula)
     mf <- match.call(expand.dots = FALSE)
-#     mf$family <- mf$start <- mf$control <- mf$maxit <- NULL
-#     mf$model <- mf$method <- mf$x <- mf$y <- mf$contrasts <- NULL
-#     mf$... <- NULL
     m <- match(c("formula", "data", "subset", "weights", "na.action",
-                 "etastart", "mustart", "offset"), names(mf), 0)
+                 "etastart", "mustart", "offset"), names(mf), 0L)
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
-    mf[[1]] <- as.name("model.frame")
+    mf[[1L]] <- as.name("model.frame")
     mf <- eval(mf, parent.frame())
-    switch(method,
-	   "model.frame" = return(mf),
-	   "glm.fit" = 1,
-	   "glm.fit.null" = 1,
-	   ## else
-	   stop("invalid `method': ", method))
-    mt <- attr(mf, "terms") # allow model.frame to update it
+    if(identical(method, "model.frame")) return(mf)
+    ## we use 'glm.fit' for the sake of error messages.
+    if(identical(method, "glm.fit")) {
+        ## OK
+    } else if(is.function(method)) {
+        glm.fit <- method
+    } else if(is.character(method)) {
+        if(exists(method)) glm.fit <- get(method)
+        else stop(gettextf("invalid 'method': %s", method), domain = NA)
+    } else stop("invalid 'method'")
+    mt <- attr(mf, "terms") # allow model.frame to have updated it
 
-    Y <- model.response(mf, "numeric")
+    Y <- model.response(mf, "any") # e.g. factors are allowed
+    ## avoid problems with 1D arrays, but keep names
+    if(length(dim(Y)) == 1L) {
+        nm <- rownames(Y)
+        dim(Y) <- NULL
+        if(!is.null(nm)) names(Y) <- nm
+    }
     ## null model support
-    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y),0)
-    weights <- model.weights(mf)
-    offset <- model.offset(mf)
+    X <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else matrix(,NROW(Y), 0L)
+    ## avoid any problems with 1D or nx1 arrays by as.vector.
+    weights <- as.vector(model.weights(mf))
+    if(!is.null(weights) && !is.numeric(weights))
+        stop("'weights' must be a numeric vector")
     ## check weights and offset
     if( !is.null(weights) && any(weights < 0) )
-	stop("Negative wts not allowed")
-    if(!is.null(offset) && length(offset) != NROW(Y))
-	stop("Number of offsets is ", length(offset),
-	     ", should equal ", NROW(Y), " (number of observations)")
+	stop("negative weights not allowed")
+
+    offset <- as.vector(model.offset(mf))
+    if(!is.null(offset)) {
+        if(length(offset) != NROW(Y))
+            stop(gettextf("number of offsets is %d should equal %d (number of observations)", length(offset), NROW(Y)), domain = NA)
+    }
     ## these allow starting values to be expressed in terms of other vars.
     mustart <- model.extract(mf, "mustart")
     etastart <- model.extract(mf, "etastart")
@@ -62,8 +89,16 @@ glm <- function(formula, family = gaussian, data, weights,
                    offset = offset, family = family, control = control,
                    intercept = attr(mt, "intercept") > 0)
 
-    ## empty models don't have an intercept!
-    if(any(offset) && attr(mt, "intercept") > 0) {
+    ## This calculated the null deviance from the intercept-only model
+    ## if there is one, otherwise from the offset-only model.
+    ## We need to recalculate by a proper fit if there is intercept and
+    ## offset.
+    ##
+    ## The glm.fit calculation could be wrong if the link depends on the
+    ## observations, so we allow the null deviance to be forced to be
+    ## re-calculated by setting an offset (provided there is an intercept).
+    ## Prior to 2.4.0 this was only done for non-zero offsets.
+    if(length(offset) && attr(mt, "intercept") > 0L) {
 	fit$null.deviance <-
 	    glm.fit(x = X[,"(Intercept)",drop=FALSE], y = Y, weights = weights,
                     offset = offset, family = family,
@@ -86,7 +121,7 @@ glm <- function(formula, family = gaussian, data, weights,
 glm.control <- function(epsilon = 1e-8, maxit = 25, trace = FALSE)
 {
     if(!is.numeric(epsilon) || epsilon <= 0)
-	stop("value of epsilon must be > 0")
+	stop("value of 'epsilon' must be > 0")
     if(!is.numeric(maxit) || maxit <= 0)
 	stop("maximum number of iterations must be > 0")
     list(epsilon = epsilon, maxit = maxit, trace = trace)
@@ -103,7 +138,7 @@ glm.fit <-
 	      family = gaussian(), control = glm.control(), intercept = TRUE)
 {
     x <- as.matrix(x)
-    xnames <- dimnames(x)[[2]]
+    xnames <- dimnames(x)[[2L]]
     ynames <- if(is.matrix(y)) rownames(y) else names(y)
     conv <- FALSE
     nobs <- NROW(y)
@@ -114,20 +149,18 @@ glm.fit <-
 	weights <- rep.int(1, nobs)
     if (is.null(offset))
 	offset <- rep.int(0, nobs)
+
     ## get family functions:
     variance <- family$variance
+    linkinv  <- family$linkinv
+    if (!is.function(variance) || !is.function(linkinv) )
+	stop("'family' argument seems not to be a valid family object")
     dev.resids <- family$dev.resids
     aic <- family$aic
-    linkinv <- family$linkinv
     mu.eta <- family$mu.eta
-    if (!is.function(variance) || !is.function(linkinv) )
-	stop("illegal `family' argument")
-    valideta <- family$valideta
-    if (is.null(valideta))
-	valideta <- function(eta) TRUE
-    validmu <- family$validmu
-    if (is.null(validmu))
-	validmu <- function(mu) TRUE
+    unless.null <- function(x, if.null) if(is.null(x)) if.null else x
+    valideta <- unless.null(family$valideta, function(eta) TRUE)
+    validmu  <- unless.null(family$validmu,  function(mu) TRUE)
     if(is.null(mustart)) {
         ## calculates mustart and may change y and weights and set n (!)
         eval(family$initialize)
@@ -139,27 +172,26 @@ glm.fit <-
     if(EMPTY) {
         eta <- rep.int(0, nobs) + offset
         if (!valideta(eta))
-            stop("Invalid linear predictor values in empty model")
+            stop("invalid linear predictor values in empty model")
         mu <- linkinv(eta)
         ## calculate initial deviance and coefficient
         if (!validmu(mu))
-            stop("Invalid fitted means in empty model")
+            stop("invalid fitted means in empty model")
         dev <- sum(dev.resids(y, mu, weights))
         w <- ((weights * mu.eta(eta)^2)/variance(mu))^0.5
         residuals <- (y - mu)/mu.eta(eta)
         good <- rep(TRUE, length(residuals))
         boundary <- conv <- TRUE
-        coef <- numeric(0)
-        iter <- 0
+        coef <- numeric(0L)
+        iter <- 0L
     } else {
         coefold <- NULL
         eta <-
             if(!is.null(etastart)) etastart
             else if(!is.null(start))
                 if (length(start) != nvars)
-                    stop("Length of start should equal ", nvars,
-                         " and correspond to initial coefs for ",
-                         deparse(xnames))
+                    stop(gettextf("length of 'start' should equal %d and correspond to initial coefs for %s", nvars, paste(deparse(xnames), collapse=", ")),
+                         domain = NA)
                 else {
                     coefold <- start
                     offset + as.vector(if (NCOL(x) == 1) x * start else x %*% start)
@@ -167,13 +199,13 @@ glm.fit <-
             else family$linkfun(mustart)
         mu <- linkinv(eta)
         if (!(validmu(mu) && valideta(eta)))
-            stop("Can't find valid starting values: please specify some")
+            stop("cannot find valid starting values: please specify some")
         ## calculate initial deviance and coefficient
         devold <- sum(dev.resids(y, mu, weights))
         boundary <- conv <- FALSE
 
         ##------------- THE Iteratively Reweighting L.S. iteration -----------
-        for (iter in 1:control$maxit) {
+        for (iter in 1L:control$maxit) {
             good <- weights > 0
             varmu <- variance(mu)[good]
             if (any(is.na(varmu)))
@@ -188,7 +220,7 @@ glm.fit <-
 
             if (all(!good)) {
                 conv <- FALSE
-                warning("No observations informative at iteration ", iter)
+                warning("no observations informative at iteration ", iter)
                 break
             }
             z <- (eta - offset)[good] + (y - mu)[good]/mu.eta.val[good]
@@ -197,24 +229,24 @@ glm.fit <-
             ## call Fortran code
             fit <- .Fortran("dqrls",
                             qr = x[good, ] * w, n = ngoodobs,
-                            p = nvars, y = w * z, ny = as.integer(1),
+                            p = nvars, y = w * z, ny = 1L,
                             tol = min(1e-7, control$epsilon/1000),
                             coefficients = double(nvars),
                             residuals = double(ngoodobs),
                             effects = double(ngoodobs),
-                            rank = integer(1),
-                            pivot = 1:nvars, qraux = double(nvars),
+                            rank = integer(1L),
+                            pivot = 1L:nvars, qraux = double(nvars),
                             work = double(2 * nvars),
                             PACKAGE = "base")
             if (any(!is.finite(fit$coefficients))) {
                 conv <- FALSE
-                warning("Non-finite coefficients at iteration ", iter)
+                warning("non-finite coefficients at iteration ", iter)
                 break
             }
             ## stop if not enough parameters
             if (nobs < fit$rank)
-                stop("X matrix has rank ", fit$rank,
-                     " but only ", nobs, " observations")
+                stop(gettextf("X matrix has rank %d, but only %d observations",
+                              fit$rank, nobs), domain = NA)
             ## calculate updated values of eta and mu with the new coef:
             start[fit$pivot] <- fit$coefficients
             eta <- drop(x %*% start)
@@ -226,12 +258,12 @@ glm.fit <-
             boundary <- FALSE
             if (!is.finite(dev)) {
                 if(is.null(coefold))
-                    stop("no valid set of coefficients has been found:please supply starting values", call. = FALSE)
-                warning("Step size truncated due to divergence", call. = FALSE)
+                    stop("no valid set of coefficients has been found: please supply starting values", call. = FALSE)
+                warning("step size truncated due to divergence", call. = FALSE)
                 ii <- 1
                 while (!is.finite(dev)) {
                     if (ii > control$maxit)
-                        stop("inner loop 1; can't correct step size")
+                        stop("inner loop 1; cannot correct step size")
                     ii <- ii + 1
                     start <- (start + coefold)/2
                     eta <- drop(x %*% start)
@@ -244,11 +276,13 @@ glm.fit <-
             }
             ## check for fitted values outside domain.
             if (!(valideta(eta) && validmu(mu))) {
-                warning("Step size truncated: out of bounds", call. = FALSE)
+                if(is.null(coefold))
+                    stop("no valid set of coefficients has been found: please supply starting values", call. = FALSE)
+                warning("step size truncated: out of bounds", call. = FALSE)
                 ii <- 1
                 while (!(valideta(eta) && validmu(mu))) {
                     if (ii > control$maxit)
-                        stop("inner loop 2; can't correct step size")
+                        stop("inner loop 2; cannot correct step size")
                     ii <- ii + 1
                     start <- (start + coefold)/2
                     eta <- drop(x %*% start)
@@ -270,8 +304,8 @@ glm.fit <-
             }
         } ##-------------- end IRLS iteration -------------------------------
 
-        if (!conv) warning("Algorithm did not converge")
-        if (boundary) warning("Algorithm stopped at boundary value")
+        if (!conv) warning("algorithm did not converge")
+        if (boundary) warning("algorithm stopped at boundary value")
         eps <- 10*.Machine$double.eps
         if (family$family == "binomial") {
             if (any(mu > 1 - eps) || any(mu < eps))
@@ -285,17 +319,19 @@ glm.fit <-
         ## hence we need to re-label the names ...
         ## Original code changed as suggested by BDR---give NA rather
         ## than 0 for non-estimable parameters
-        if (fit$rank < nvars) coef[fit$pivot][seq(fit$rank+1, nvars)] <- NA
+        if (fit$rank < nvars) coef[fit$pivot][seq.int(fit$rank+1, nvars)] <- NA
         xxnames <- xnames[fit$pivot]
-        residuals <- rep.int(NA, nobs)
-        residuals[good] <- z - (eta - offset)[good] # z does not have offset in.
+        ## update by accurate calculation, including 0-weight cases.
+        residuals <-  (y - mu)/mu.eta(eta)
+##        residuals <- rep.int(NA, nobs)
+##        residuals[good] <- z - (eta - offset)[good] # z does not have offset in.
         fit$qr <- as.matrix(fit$qr)
         nr <- min(sum(good), nvars)
         if (nr < nvars) {
             Rmat <- diag(nvars)
-            Rmat[1:nr, 1:nvars] <- fit$qr[1:nr, 1:nvars]
+            Rmat[1L:nr, 1L:nvars] <- fit$qr[1L:nr, 1L:nvars]
         }
-        else Rmat <- fit$qr[1:nvars, 1:nvars]
+        else Rmat <- fit$qr[1L:nvars, 1L:nvars]
         Rmat <- as.matrix(Rmat)
         Rmat[row(Rmat) > col(Rmat)] <- 0
         names(coef) <- xnames
@@ -313,7 +349,7 @@ glm.fit <-
     names(y) <- ynames
     if(!EMPTY)
         names(fit$effects) <-
-            c(xxnames[seq(len=fit$rank)], rep.int("", sum(good) - fit$rank))
+            c(xxnames[seq_len(fit$rank)], rep.int("", sum(good) - fit$rank))
     ## calculate null deviance -- corrected in glm() if offset and intercept
     wtdmu <-
 	if (intercept) sum(weights * y)/sum(weights) else linkinv(offset)
@@ -345,13 +381,14 @@ print.glm <- function(x, digits= max(3, getOption("digits") - 3), ...)
         cat("Coefficients")
         if(is.character(co <- x$contrasts))
             cat("  [contrasts: ",
-                apply(cbind(names(co),co), 1, paste, collapse="="), "]")
+                apply(cbind(names(co),co), 1L, paste, collapse="="), "]")
         cat(":\n")
         print.default(format(x$coefficients, digits=digits),
                       print.gap = 2, quote = FALSE)
     } else cat("No coefficients\n\n")
     cat("\nDegrees of Freedom:", x$df.null, "Total (i.e. Null); ",
         x$df.residual, "Residual\n")
+    if(nzchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep="")
     cat("Null Deviance:	   ",	format(signif(x$null.deviance, digits)),
 	"\nResidual Deviance:", format(signif(x$deviance, digits)),
 	"\tAIC:", format(signif(x$aic, digits)), "\n")
@@ -366,12 +403,12 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
     named <- if (is.null(names(dotargs)))
 	rep(FALSE, length(dotargs)) else (names(dotargs) != "")
     if(any(named))
-	warning("The following arguments to anova.glm(..) are invalid and dropped: ",
+	warning("the following arguments to 'anova.glm' are invalid and dropped: ",
 		paste(deparse(dotargs[named]), collapse=", "))
     dotargs <- dotargs[!named]
     is.glm <- unlist(lapply(dotargs,function(x) inherits(x,"glm")))
     dotargs <- dotargs[is.glm]
-    if (length(dotargs) > 0)
+    if (length(dotargs))
 	return(anova.glmlist(c(list(object), dotargs),
 			     dispersion = dispersion, test=test))
 
@@ -380,7 +417,7 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
     varlist <- attr(object$terms, "variables")
     ## must avoid partial matching here.
     x <-
-	if (n <- match("x", names(object), 0))
+	if (n <- match("x", names(object), 0L))
 	    object[[n]]
 	else model.matrix(object)
     varseq <- attr(x, "assign")
@@ -394,12 +431,19 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
 	method <- object$method
 	if(!is.function(method))
 	    method <- get(method, mode = "function", envir=parent.frame())
-	for(i in 1:(nvars-1)) {
+        ## allow for 'y = FALSE' in the call (PR#13098)
+        y <- object$y
+        if(is.null(y)) { ## code from residuals.glm
+            mu.eta <- object$family$mu.eta
+            eta <- object$linear.predictors
+            y <-   object$fitted.values + object$residuals * mu.eta(eta)
+        }
+	for(i in 1L:(nvars-1)) {
 	    ## explanatory variables up to i are kept in the model
 	    ## use method from glm to find residual deviance
 	    ## and df for each sequential fit
 	    fit <- method(x=x[, varseq <= i, drop = FALSE],
-			  y=object$y,
+			  y=y,
 			  weights=object$prior.weights,
 			  start	 =object$start,
 			  offset =object$offset,
@@ -420,12 +464,12 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
     table <- data.frame(c(NA, -diff(resdf)),
 			c(NA, pmax(0, -diff(resdev))), resdf, resdev)
     tl <- attr(object$terms, "term.labels")
-    if (length(tl) == 0) table <- table[1,,drop=FALSE] # kludge for null model
+    if (length(tl) == 0L) table <- table[1,,drop=FALSE] # kludge for null model
     dimnames(table) <- list(c("NULL", tl),
 			    c("Df", "Deviance", "Resid. Df", "Resid. Dev"))
     title <- paste("Analysis of Deviance Table", "\n\nModel: ",
 		   object$family$family, ", link: ", object$family$link,
-		   "\n\nResponse: ", as.character(varlist[-1])[1],
+		   "\n\nResponse: ", as.character(varlist[-1L])[1L],
 		   "\n\nTerms added sequentially (first to last)\n\n", sep="")
 
     ## calculate test statistics if needed
@@ -435,9 +479,19 @@ anova.glm <- function(object, ..., dispersion=NULL, test=NULL)
 	dispersion <- summary(object, dispersion=dispersion)$dispersion
 	df.dispersion <- if (dispersion == 1) Inf else object$df.residual
     }
-    if(!is.null(test))
+    if(!is.null(test)) {
+        if(test == "F" && df.dispersion == Inf) {
+            fam <- object$family$family
+            if(fam == "binomial" || fam == "poisson")
+                warning(gettextf("using F test with a %s family is inappropriate",
+                                 fam),
+                        domain = NA)
+            else
+                warning("using F test with a fixed dispersion is inappropriate")
+        }
 	table <- stat.anova(table=table, test=test, scale=dispersion,
 			    df.scale=df.dispersion, n=NROW(x))
+    }
     structure(table, heading = title, class= c("anova", "data.frame"))
 }
 
@@ -449,23 +503,23 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
     ## any models with a different response
 
     responses <- as.character(lapply(object, function(x) {
-	deparse(formula(x)[[2]])} ))
-    sameresp <- responses==responses[1]
+	deparse(formula(x)[[2L]])} ))
+    sameresp <- responses==responses[1L]
     if(!all(sameresp)) {
 	object <- object[sameresp]
-	warning("Models with response ", deparse(responses[!sameresp]),
-		" removed because response differs from ", "model 1")
+	warning("models with response ", deparse(responses[!sameresp]),
+		" removed because response differs from model 1")
     }
 
     ns <- sapply(object, function(x) length(x$residuals))
-    if(any(ns != ns[1]))
+    if(any(ns != ns[1L]))
 	stop("models were not all fitted to the same size of dataset")
 
     ## calculate the number of models
 
     nmodels <- length(object)
     if(nmodels==1)
-	return(anova.glm(object[[1]], dispersion=dispersion, test=test))
+	return(anova.glm(object[[1L]], dispersion=dispersion, test=test))
 
     ## extract statistics
 
@@ -478,18 +532,27 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
 			c(NA, -diff(resdev)) )
     variables <- lapply(object, function(x)
 			paste(deparse(formula(x)), collapse="\n") )
-    dimnames(table) <- list(1:nmodels, c("Resid. Df", "Resid. Dev", "Df",
+    dimnames(table) <- list(1L:nmodels, c("Resid. Df", "Resid. Dev", "Df",
 					 "Deviance"))
     title <- "Analysis of Deviance Table\n"
-    topnote <- paste("Model ", format(1:nmodels),": ",
+    topnote <- paste("Model ", format(1L:nmodels),": ",
 		     variables, sep="", collapse="\n")
 
     ## calculate test statistic if needed
 
     if(!is.null(test)) {
-	bigmodel <- object[[order(resdf)[1]]]
+	bigmodel <- object[[order(resdf)[1L]]]
 	dispersion <- summary(bigmodel, dispersion=dispersion)$dispersion
 	df.dispersion <- if (dispersion == 1) Inf else min(resdf)
+        if(test == "F" && df.dispersion == Inf) {
+            fam <- bigmodel$family$family
+            if(fam == "binomial" || fam == "poisson")
+                warning(gettextf("using F test with a '%s' family is inappropriate",
+                                 fam),
+                        domain = NA, call. = FALSE)
+            else
+                warning("using F test with a fixed dispersion is inappropriate")
+        }
 	table <- stat.anova(table = table, test = test,
 			    scale = dispersion, df.scale = df.dispersion,
 			    n = length(bigmodel$residuals))
@@ -499,30 +562,6 @@ anova.glmlist <- function(object, ..., dispersion=NULL, test=NULL)
 }
 
 
-## utility for anova.FOO(), FOO in "lmlist", "glm", "glmlist":
-stat.anova <- function(table, test=c("Chisq", "F", "Cp"), scale, df.scale, n)
-{
-    test <- match.arg(test)
-    dev.col <- match("Deviance", colnames(table))
-    if(is.na(dev.col)) dev.col <- match("Sum of Sq", colnames(table))
-    switch(test,
-	   "Chisq" = {
-	       cbind(table,"P(>|Chi|)"= pchisq(abs(table[, dev.col]/scale),
-			     abs(table[, "Df"]), lower.tail=FALSE))
-	   },
-	   "F" = {
-	       Fvalue <- abs((table[, dev.col]/table[, "Df"])/scale)
-	       Fvalue[table[, "Df"] %in% 0] <- NA
-	       cbind(table, F = Fvalue,
-		     "Pr(>F)" = pf(Fvalue, abs(table[, "Df"]),
-		     abs(df.scale), lower.tail=FALSE))
-	   },
-	   "Cp" = {
-	       cbind(table, Cp = table[,"Resid. Dev"] +
-		     2*scale*(n - table[,"Resid. Df"]))
-	   })
-}
-
 summary.glm <- function(object, dispersion = NULL,
 			correlation = FALSE, symbolic.cor = FALSE, ...)
 {
@@ -530,23 +569,25 @@ summary.glm <- function(object, dispersion = NULL,
     df.r <- object$df.residual
     if(is.null(dispersion))	# calculate dispersion if needed
 	dispersion <-
-	    if(any(object$family$family == c("poisson", "binomial")))  1
+	    if(object$family$family %in% c("poisson", "binomial"))  1
 	    else if(df.r > 0) {
-		est.disp <- TRUE
+                est.disp <- TRUE
 		if(any(object$weights==0))
-		    warning("observations with zero weight ",
-			    "not used for calculating dispersion")
-		sum(object$weights*object$residuals^2)/ df.r
-	    } else Inf
+		    warning("observations with zero weight not used for calculating dispersion")
+		sum((object$weights*object$residuals^2)[object$weights > 0])/ df.r
+	    } else {
+                est.disp <- TRUE
+                NaN
+            }
 
     ## calculate scaled and unscaled covariance matrix
 
+    aliased <- is.na(coef(object))  # used in print method
     p <- object$rank
     if (p > 0) {
-        p1 <- 1:p
+        p1 <- 1L:p
         Qr <- object$qr
-        aliased <- is.na(coef(object))  # used in print method
-        ## WATCHIT! doesn't this rely on pivoting not permuting 1:p? -- that's quaranteed
+        ## WATCHIT! doesn't this rely on pivoting not permuting 1L:p? -- that's quaranteed
         coef.p <- object$coefficients[Qr$pivot[p1]]
         covmat.unscaled <- chol2inv(Qr$qr[p1,p1,drop=FALSE])
         dimnames(covmat.unscaled) <- list(names(coef.p),names(coef.p))
@@ -559,7 +600,7 @@ summary.glm <- function(object, dispersion = NULL,
         tvalue <- coef.p/s.err
 
         dn <- c("Estimate", "Std. Error")
-        if(!est.disp) {
+        if(!est.disp) { # known dispersion
             pvalue <- 2*pnorm(-abs(tvalue))
             coef.table <- cbind(coef.p, s.err, tvalue, pvalue)
             dimnames(coef.table) <- list(names(coef.p),
@@ -569,24 +610,26 @@ summary.glm <- function(object, dispersion = NULL,
             coef.table <- cbind(coef.p, s.err, tvalue, pvalue)
             dimnames(coef.table) <- list(names(coef.p),
                                          c(dn, "t value","Pr(>|t|)"))
-        } else { ## df.r == 0
-            coef.table <- cbind(coef.p, Inf)
-            dimnames(coef.table) <- list(names(coef.p), dn)
+        } else { # df.r == 0
+            coef.table <- cbind(coef.p, NaN, NaN, NaN)
+            dimnames(coef.table) <- list(names(coef.p),
+                                         c(dn, "t value","Pr(>|t|)"))
         }
         df.f <- NCOL(Qr$qr)
     } else {
-        coef.table <- matrix(, 0, 4)
+        coef.table <- matrix(, 0L, 4L)
         dimnames(coef.table) <-
             list(NULL, c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
-        covmat.unscaled <- covmat <- matrix(, 0, 0)
-        aliased <- is.na(coef(object))
+        covmat.unscaled <- covmat <- matrix(, 0L, 0L)
         df.f <- length(aliased)
     }
     ## return answer
 
-    ans <- c(object[c("call","terms","family","deviance", "aic",
-		      "contrasts",
-		      "df.residual","null.deviance","df.null","iter")],
+    ## these need not all exist, e.g. na.action.
+    keep <- match(c("call","terms","family","deviance", "aic",
+		      "contrasts", "df.residual","null.deviance","df.null",
+                      "iter", "na.action"), names(object), 0L)
+    ans <- c(object[keep],
 	     list(deviance.resid = residuals(object, type = "deviance"),
 		  coefficients = coef.table,
                   aliased = aliased,
@@ -617,20 +660,23 @@ print.summary.glm <-
 	x$deviance.resid <- quantile(x$deviance.resid,na.rm=TRUE)
 	names(x$deviance.resid) <- c("Min", "1Q", "Median", "3Q", "Max")
     }
-    print.default(x$deviance.resid, digits=digits, na = "", print.gap = 2)
+    print.default(x$deviance.resid, digits=digits, na.print = "",
+                  print.gap = 2)
 
-    if(length(x$aliased) == 0) {
+    if(length(x$aliased) == 0L) {
         cat("\nNo Coefficients\n")
     } else {
         ## df component added in 1.8.0
-        if (!is.null(df<- x$df) && (nsingular <- df[3] - df[1]))
+        ## partial matching problem here.
+        df <- if ("df" %in% names(x)) x[["df"]] else NULL
+        if (!is.null(df) && (nsingular <- df[3L] - df[1L]))
             cat("\nCoefficients: (", nsingular,
                 " not defined because of singularities)\n", sep = "")
         else cat("\nCoefficients:\n")
         coefs <- x$coefficients
         if(!is.null(aliased <- x$aliased) && any(aliased)) {
             cn <- names(aliased)
-            coefs <- matrix(NA, length(aliased), 4,
+            coefs <- matrix(NA, length(aliased), 4L,
                             dimnames=list(cn, colnames(coefs)))
             coefs[!aliased, ] <- x$coefficients
         }
@@ -640,14 +686,15 @@ print.summary.glm <-
     ##
     cat("\n(Dispersion parameter for ", x$family$family,
 	" family taken to be ", format(x$dispersion), ")\n\n",
-	apply(cbind(paste(format.char(c("Null","Residual"),width=8,flag=""),
-			  "deviance:"),
+	apply(cbind(paste(format(c("Null","Residual"), justify="right"),
+                          "deviance:"),
 		    format(unlist(x[c("null.deviance","deviance")]),
 			   digits= max(5, digits+1)), " on",
 		    format(unlist(x[c("df.null","df.residual")])),
 		    " degrees of freedom\n"),
-	      1, paste, collapse=" "),
-	"AIC: ", format(x$aic, digits= max(4, digits+1)),"\n\n",
+	      1L, paste, collapse=" "), sep="")
+    if(nzchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep="")
+    cat("AIC: ", format(x$aic, digits= max(4, digits+1)),"\n\n",
 	"Number of Fisher Scoring iterations: ", x$iter,
 	"\n", sep="")
 
@@ -663,7 +710,7 @@ print.summary.glm <-
 	if(p > 1) {
 	    cat("\nCorrelation of Coefficients:\n")
 	    if(is.logical(symbolic.cor) && symbolic.cor) {# NULL < 1.7.0 objects
-		print(symnum(correl, abbr.col = NULL))
+		print(symnum(correl, abbr.colnames = NULL))
 	    } else {
 		correl <- format(round(correl, 2), nsmall = 2, digits = digits)
 		correl[!lower.tri(correl)] <- ""
@@ -693,8 +740,15 @@ residuals.glm <-
     r <- object$residuals
     mu	<- object$fitted.values
     wts <- object$prior.weights
+    switch(type,
+           deviance=,pearson=,response=
+           if(is.null(y)) {
+               mu.eta <- object$family$mu.eta
+               eta <- object$linear.predictors
+               y <-  mu + r * mu.eta(eta)
+           })
     res <- switch(type,
-		  deviance = if(object$df.res > 0) {
+		  deviance = if(object$df.residual > 0) {
 		      d.res <- sqrt(pmax((object$family$dev.resids)(y, mu, wts), 0))
 		      ifelse(y > mu, d.res, -d.res)
 		  } else rep.int(0, length(mu)),
@@ -704,22 +758,24 @@ residuals.glm <-
 		  partial = r
 		  )
     if(!is.null(object$na.action))
-      res<- naresid(object$na.action, res)
-    if (type=="partial") ## need to avoid doing naresid() twice.
-      res<-res+predict(object, type="terms")
+        res <- naresid(object$na.action, res)
+    if (type == "partial") ## need to avoid doing naresid() twice.
+        res <- res+predict(object, type="terms")
     res
 }
 
+## For influence.glm() ... --> ./lm.influence.R
+
 ## KH on 1998/06/22: update.default() is now used ...
 
 model.frame.glm <- function (formula, ...)
 {
     dots <- list(...)
-    nargs <- dots[match(c("data", "na.action", "subset"), names(dots), 0)]
+    nargs <- dots[match(c("data", "na.action", "subset"), names(dots), 0L)]
     if (length(nargs) || is.null(formula$model)) {
 	fcall <- formula$call
 	fcall$method <- "model.frame"
-	fcall[[1]] <- as.name("glm")
+	fcall[[1L]] <- as.name("glm")
         fcall[names(nargs)] <- nargs
 #	env <- environment(fcall$formula)  # always NULL
         env <- environment(formula$terms)
