@@ -1,3 +1,19 @@
+#  File src/library/stats/R/lm.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 
 lm <- function (formula, data, subset, weights, na.action,
 		method = "qr", model = TRUE, x = FALSE, y = FALSE,
@@ -9,10 +25,10 @@ lm <- function (formula, data, subset, weights, na.action,
     cl <- match.call()
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "weights", "na.action", "offset"),
-	       names(mf), 0)
-    mf <- mf[c(1, m)]
+	       names(mf), 0L)
+    mf <- mf[c(1L, m)]
     mf$drop.unused.levels <- TRUE
-    mf[[1]] <- as.name("model.frame")
+    mf[[1L]] <- as.name("model.frame")
     mf <- eval(mf, parent.frame())
     if (method == "model.frame")
 	return(mf)
@@ -27,8 +43,7 @@ lm <- function (formula, data, subset, weights, na.action,
         stop("'weights' must be a numeric vector")
     offset <- as.vector(model.offset(mf))
     if(!is.null(offset)) {
-        if(length(offset) == 1) offset <- rep(offset, NROW(y))
-        else if(length(offset) != NROW(y))
+        if(length(offset) != NROW(y))
             stop(gettextf("number of offsets is %d, should equal %d (number of observations)",
                           length(offset), NROW(y)), domain = NA)
     }
@@ -36,8 +51,8 @@ lm <- function (formula, data, subset, weights, na.action,
     if (is.empty.model(mt)) {
 	x <- NULL
 	z <- list(coefficients = if (is.matrix(y))
-                    matrix(,0,3) else numeric(0), residuals = y,
-		  fitted.values = 0 * y, weights = w, rank = 0,
+                    matrix(,0,3) else numeric(0L), residuals = y,
+		  fitted.values = 0 * y, weights = w, rank = 0L,
 		  df.residual = if (is.matrix(y)) nrow(y) else length(y))
         if(!is.null(offset)) {
             z$fitted.values <- offset
@@ -72,11 +87,11 @@ lm.fit <- function (x, y, offset = NULL, method = "qr", tol = 1e-07,
                     singular.ok = TRUE, ...)
 {
     if (is.null(n <- nrow(x))) stop("'x' must be a matrix")
-    if(n == 0) stop("0 (non-NA) cases")
+    if(n == 0L) stop("0 (non-NA) cases")
     p <- ncol(x)
-    if (p == 0) {
+    if (p == 0L) {
         ## oops, null model
-        return(list(coefficients = numeric(0), residuals = y,
+        return(list(coefficients = numeric(0L), residuals = y,
                     fitted.values = 0 * y, rank = 0,
                     df.residual = length(y)))
     }
@@ -101,17 +116,17 @@ lm.fit <- function (x, y, offset = NULL, method = "qr", tol = 1e-07,
 		  y = y, ny = ny,
 		  tol = as.double(tol),
 		  coefficients = mat.or.vec(p, ny),
-		  residuals = y, effects = y, rank = integer(1),
-		  pivot = 1:p, qraux = double(p), work = double(2*p),
+		  residuals = y, effects = y, rank = integer(1L),
+		  pivot = 1L:p, qraux = double(p), work = double(2*p),
                   PACKAGE="base")
     if(!singular.ok && z$rank < p) stop("singular fit encountered")
     coef <- z$coefficients
     pivot <- z$pivot
     ## careful here: the rank might be 0
     r1 <- seq_len(z$rank)
-    dn <- colnames(x); if(is.null(dn)) dn <- paste("x", 1:p, sep="")
+    dn <- colnames(x); if(is.null(dn)) dn <- paste("x", 1L:p, sep="")
     nmeffects <- c(dn[pivot[r1]], rep.int("", n - z$rank))
-    r2 <- if(z$rank < p) (z$rank+1):p else integer(0)
+    r2 <- if(z$rank < p) (z$rank+1L):p else integer(0L)
     if (is.matrix(y)) {
 	coef[r2, ] <- NA
 	coef[pivot, ] <- coef
@@ -140,7 +155,7 @@ lm.wfit <- function (x, y, w, offset = NULL, method = "qr", tol = 1e-7,
     if(n == 0) stop("0 (non-NA) cases")
     ny <- NCOL(y)
     ## treat one-col matrix as vector
-    if(is.matrix(y) && ny == 1)
+    if(is.matrix(y) && ny == 1L)
         y <- drop(y)
     if(!is.null(offset))
         y <- y - offset
@@ -166,14 +181,14 @@ lm.wfit <- function (x, y, w, offset = NULL, method = "qr", tol = 1e-7,
 	x0 <- x[!ok, , drop = FALSE]
 	x <- x[ok,  , drop = FALSE]
 	n <- nrow(x)
-	y0 <- if (ny > 1) y[!ok, , drop = FALSE] else y[!ok]
-	y  <- if (ny > 1) y[ ok, , drop = FALSE] else y[ok]
+	y0 <- if (ny > 1L) y[!ok, , drop = FALSE] else y[!ok]
+	y  <- if (ny > 1L) y[ ok, , drop = FALSE] else y[ok]
     }
     p <- ncol(x)
     if (p == 0) {
         ## oops, null model
-        return(list(coefficients = numeric(0), residuals = y,
-                    fitted.values = 0 * y, weights = w, rank = 0,
+        return(list(coefficients = numeric(0L), residuals = y,
+                    fitted.values = 0 * y, weights = w, rank = 0L,
                     df.residual = length(y)))
     }
     storage.mode(y) <- "double"
@@ -184,16 +199,16 @@ lm.wfit <- function (x, y, w, offset = NULL, method = "qr", tol = 1e-7,
 		  tol = as.double(tol),
 		  coefficients = mat.or.vec(p, ny), residuals = y,
 		  effects = mat.or.vec(n, ny),
-		  rank = integer(1), pivot = 1:p, qraux = double(p),
+		  rank = integer(1L), pivot = 1L:p, qraux = double(p),
 		  work = double(2 * p),
                   PACKAGE="base")
     if(!singular.ok && z$rank < p) stop("singular fit encountered")
     coef <- z$coefficients
     pivot <- z$pivot
     r1 <- seq_len(z$rank)
-    dn <- colnames(x); if(is.null(dn)) dn <- paste("x", 1:p, sep="")
+    dn <- colnames(x); if(is.null(dn)) dn <- paste("x", 1L:p, sep="")
     nmeffects <- c(dn[pivot[r1]], rep.int("", n - z$rank))
-    r2 <- if(z$rank < p) (z$rank+1):p else integer(0)
+    r2 <- if(z$rank < p) (z$rank+1L):p else integer(0L)
     if (is.matrix(y)) {
 	coef[r2, ] <- NA
 	coef[pivot, ] <- coef
@@ -270,8 +285,8 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
         class(ans) <- "summary.lm"
         ans$aliased <- is.na(coef(object))  # used in print method
         ans$residuals <- r
-        ans$df <- c(0, n, length(ans$aliased))
-        ans$coefficients <- matrix(NA, 0, 4)
+        ans$df <- c(0L, n, length(ans$aliased))
+        ans$coefficients <- matrix(NA, 0L, 4L)
         dimnames(ans$coefficients)<-
             list(NULL, c("Estimate", "Std. Error", "t value", "Pr(>|t|)"))
         ans$sigma <- sqrt(resvar)
@@ -285,10 +300,10 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
     rdf <- n - p
     if(is.na(z$df.residual) || rdf != z$df.residual)
         warning("residual degrees of freedom in object suggest this is not an \"lm\" fit")
-    p1 <- 1:p
+    p1 <- 1L:p
     ## do not want missing values substituted here
     r <- z$residuals
-    f <- z$fitted
+    f <- z$fitted.values
     w <- z$weights
     if (is.null(w)) {
         mss <- if (attr(z$terms, "intercept"))
@@ -318,7 +333,7 @@ summary.lm <- function (object, correlation = FALSE, symbolic.cor = FALSE, ...)
     ans$sigma <- sqrt(resvar)
     ans$df <- c(p, rdf, NCOL(Qr$qr))
     if (p != attr(z$terms, "intercept")) {
-	df.int <- if (attr(z$terms, "intercept")) 1 else 0
+	df.int <- if (attr(z$terms, "intercept")) 1L else 0L
 	ans$r.squared <- mss/(mss + rss)
 	ans$adj.r.squared <- 1 - (1 - ans$r.squared) * ((n - df.int)/rdf)
 	ans$fstatistic <- c(value = (mss/(p - df.int))/resvar,
@@ -345,26 +360,26 @@ print.summary.lm <-
     cat(paste(deparse(x$call), sep="\n", collapse = "\n"), "\n\n", sep="")
     resid <- x$residuals
     df <- x$df
-    rdf <- df[2]
+    rdf <- df[2L]
     cat(if(!is.null(x$w) && diff(range(x$w))) "Weighted ",
         "Residuals:\n", sep="")
-    if (rdf > 5) {
+    if (rdf > 5L) {
 	nam <- c("Min", "1Q", "Median", "3Q", "Max")
 	rq <- if (length(dim(resid)) == 2)
-	    structure(apply(t(resid), 1, quantile),
-		      dimnames = list(nam, dimnames(resid)[[2]]))
+	    structure(apply(t(resid), 1L, quantile),
+		      dimnames = list(nam, dimnames(resid)[[2L]]))
 	else  structure(quantile(resid), names = nam)
 	print(rq, digits = digits, ...)
     }
-    else if (rdf > 0) {
+    else if (rdf > 0L) {
 	print(resid, digits = digits, ...)
     } else { # rdf == 0 : perfect fit!
-	cat("ALL", df[1], "residuals are 0: no residual degrees of freedom!\n")
+	cat("ALL", df[1L], "residuals are 0: no residual degrees of freedom!\n")
     }
-    if (length(x$aliased) == 0) {
+    if (length(x$aliased) == 0L) {
         cat("\nNo Coefficients\n")
     } else {
-        if (nsingular <- df[3] - df[1])
+        if (nsingular <- df[3L] - df[1L])
             cat("\nCoefficients: (", nsingular,
                 " not defined because of singularities)\n", sep = "")
         else cat("\nCoefficients:\n")
@@ -380,24 +395,24 @@ print.summary.lm <-
     ##
     cat("\nResidual standard error:",
 	format(signif(x$sigma, digits)), "on", rdf, "degrees of freedom\n")
-    if(nchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep="")
+    if(nzchar(mess <- naprint(x$na.action))) cat("  (",mess, ")\n", sep="")
     if (!is.null(x$fstatistic)) {
-	cat("Multiple R-Squared:", formatC(x$r.squared, digits=digits))
+	cat("Multiple R-squared:", formatC(x$r.squared, digits=digits))
 	cat(",\tAdjusted R-squared:",formatC(x$adj.r.squared,digits=digits),
-	    "\nF-statistic:", formatC(x$fstatistic[1], digits=digits),
-	    "on", x$fstatistic[2], "and",
-	    x$fstatistic[3], "DF,  p-value:",
-	    format.pval(pf(x$fstatistic[1], x$fstatistic[2],
-                           x$fstatistic[3], lower.tail = FALSE), digits=digits),
+	    "\nF-statistic:", formatC(x$fstatistic[1L], digits=digits),
+	    "on", x$fstatistic[2L], "and",
+	    x$fstatistic[3L], "DF,  p-value:",
+	    format.pval(pf(x$fstatistic[1L], x$fstatistic[2L],
+                           x$fstatistic[3L], lower.tail = FALSE), digits=digits),
 	    "\n")
     }
     correl <- x$correlation
     if (!is.null(correl)) {
 	p <- NCOL(correl)
-	if (p > 1) {
+	if (p > 1L) {
 	    cat("\nCorrelation of Coefficients:\n")
 	    if(is.logical(symbolic.cor) && symbolic.cor) {# NULL < 1.7.0 objects
-		print(symnum(correl, abbr.col = NULL))
+		print(symnum(correl, abbr.colnames = NULL))
 	    } else {
                 correl <- format(round(correl, 2), nsmall = 2, digits = digits)
                 correl[!lower.tri(correl)] <- ""
@@ -422,16 +437,17 @@ residuals.lm <-
                   if(is.null(object$weights)) r else r * sqrt(object$weights),
                   partial = r
            )
-    res<-naresid(object$na.action, res)
+    res <- naresid(object$na.action, res)
     if (type=="partial") ## predict already does naresid
-      res<-res+predict(object,type="terms")
+      res <- res + predict(object,type="terms")
     res
 }
 
+## The lm method includes objects of class "glm"
 simulate.lm <- function(object, nsim = 1, seed = NULL, ...)
 {
     if(!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
-        runif(1)                       # initialize the RNG if necessary
+        runif(1)                     # initialize the RNG if necessary
     if(is.null(seed))
         RNGstate <- get(".Random.seed", envir = .GlobalEnv)
     else {
@@ -440,16 +456,30 @@ simulate.lm <- function(object, nsim = 1, seed = NULL, ...)
         RNGstate <- structure(seed, kind = as.list(RNGkind()))
         on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
     }
-    ftd <- fitted(object)
-    ans <-
-        as.data.frame(ftd +
-                      matrix(rnorm(length(ftd) * nsim,
-                                   sd = sqrt(deviance(object)/
-                                   df.residual(object))),
-                             nr = length(ftd)))
+    ftd <- fitted(object)             # == napredict(*, object$fitted)
+    nm <- names(ftd)
+    n <- length(ftd)
+    ntot <- n * nsim
+    fam <- if(inherits(object, "glm")) object$family$family else "gaussian"
+    val <- switch(fam,
+                  "gaussian" = {
+                      vars <- deviance(object)/ df.residual(object)
+                      if (!is.null(object$weights)) vars <- vars/object$weights
+                      ftd + rnorm(ntot, sd = sqrt(vars))
+                  },
+                  if(!is.null(object$family$simulate))
+                      object$family$simulate(object, nsim)
+                  else stop("family '", fam, "' not implemented"))
 
-    attr(ans, "seed") <- RNGstate
-    ans
+    if(!is.list(val)) {
+        dim(val) <- c(n, nsim)
+        val <- as.data.frame(val)
+    } else
+        class(val) <- "data.frame"
+    names(val) <- paste("sim", seq_len(nsim), sep="_")
+    if (!is.null(nm)) row.names(val) <- nm
+    attr(val, "seed") <- RNGstate
+    val
 }
 
 #fitted.lm <- function(object, ...)
@@ -484,7 +514,7 @@ model.frame.lm <- function(formula, ...)
     if (length(nargs) || is.null(formula$model)) {
         fcall <- formula$call
         fcall$method <- "model.frame"
-        fcall[[1]] <- as.name("lm")
+        fcall[[1L]] <- as.name("lm")
         fcall[names(nargs)] <- nargs
 #	env <- environment(fcall$formula)  # always NULL
         env <- environment(formula$terms)
@@ -496,9 +526,9 @@ model.frame.lm <- function(formula, ...)
 
 variable.names.lm <- function(object, full = FALSE, ...)
 {
-    if(full)	dimnames(object$qr$qr)[[2]]
-    else if(object$rank) dimnames(object$qr$qr)[[2]][seq_len(object$rank)]
-    else character(0)
+    if(full)	dimnames(object$qr$qr)[[2L]]
+    else if(object$rank) dimnames(object$qr$qr)[[2L]][seq_len(object$rank)]
+    else character(0L)
 }
 
 case.names.lm <- function(object, full = FALSE, ...)
@@ -510,14 +540,17 @@ case.names.lm <- function(object, full = FALSE, ...)
 
 anova.lm <- function(object, ...)
 {
-    if(length(list(object, ...)) > 1)
+    if(length(list(object, ...)) > 1L)
 	return(anova.lmlist(object, ...))
     w <- object$weights
-    ssr <- sum(if(is.null(w)) object$resid^2 else w*object$resid^2)
+    ssr <- sum(if(is.null(w)) object$residuals^2 else w*object$residuals^2)
+    mss <- sum(if(is.null(w)) object$fitted.values^2 else w*object$fitted.values^2)
+    if(ssr < 1e-10*mss)
+        warning("ANOVA F-tests on an essentially perfect fit are unreliable")
     dfr <- df.residual(object)
     p <- object$rank
-    if(p > 0) {
-        p1 <- 1:p
+    if(p > 0L) {
+        p1 <- 1L:p
         comp <- object$effects[p1]
         asgn <- object$assign[object$qr$pivot][p1]
         nmeffects <- c("(Intercept)", attr(object$terms, "term.labels"))
@@ -527,7 +560,7 @@ anova.lm <- function(object, ...)
     } else {
         ss <- ssr
         df <- dfr
-        tlabels <- character(0)
+        tlabels <- character(0L)
     }
     ms <- ss/df
     f <- ms/(ssr/dfr)
@@ -538,7 +571,7 @@ anova.lm <- function(object, ...)
                             c("Df","Sum Sq", "Mean Sq", "F value", "Pr(>F)"))
     if(attr(object$terms,"intercept")) table <- table[-1, ]
     structure(table, heading = c("Analysis of Variance Table\n",
-		     paste("Response:", deparse(formula(object)[[2]]))),
+		     paste("Response:", deparse(formula(object)[[2L]]))),
 	      class= c("anova", "data.frame"))# was "tabular"
 }
 
@@ -546,8 +579,8 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
 {
     objects <- list(object, ...)
     responses <- as.character(lapply(objects,
-				     function(x) deparse(x$terms[[2]])))
-    sameresp <- responses == responses[1]
+				     function(x) deparse(x$terms[[2L]])))
+    sameresp <- responses == responses[1L]
     if (!all(sameresp)) {
 	objects <- objects[sameresp]
 	warning("models with response ",
@@ -556,7 +589,7 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
     }
 
     ns <- sapply(objects, function(x) length(x$residuals))
-    if(any(ns != ns[1]))
+    if(any(ns != ns[1L]))
         stop("models were not all fitted to the same size of dataset")
 
     ## calculate the number of models
@@ -575,17 +608,17 @@ anova.lmlist <- function (object, ..., scale = 0, test = "F")
                         c(NA, -diff(resdev)) )
     variables <- lapply(objects, function(x)
                         paste(deparse(formula(x)), collapse="\n") )
-    dimnames(table) <- list(1:nmodels,
+    dimnames(table) <- list(1L:nmodels,
                             c("Res.Df", "RSS", "Df", "Sum of Sq"))
 
     title <- "Analysis of Variance Table\n"
-    topnote <- paste("Model ", format(1:nmodels),": ",
+    topnote <- paste("Model ", format(1L:nmodels),": ",
 		     variables, sep="", collapse="\n")
 
     ## calculate test statistic if needed
 
     if(!is.null(test)) {
-	bigmodel <- order(resdf)[1]
+	bigmodel <- order(resdf)[1L]
         scale <- if(scale > 0) scale else resdev[bigmodel]/resdf[bigmodel]
 	table <- stat.anova(table = table, test = test,
 			    scale = scale,
@@ -615,11 +648,13 @@ predict.lm <-
         m <- model.frame(Terms, newdata, na.action = na.action,
                          xlev = object$xlevels)
         if(!is.null(cl <- attr(Terms, "dataClasses"))) .checkMFClasses(cl, m)
-        X <- model.matrix(Terms, m, contrasts = object$contrasts)
-	offset <- if (!is.null(off.num <- attr(tt, "offset")))
-	    eval(attr(tt, "variables")[[off.num+1]], newdata)
-	else if (!is.null(object$offset))
-	    eval(object$call$offset, newdata)
+        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
+        offset <- rep(0, nrow(X))
+        if (!is.null(off.num <- attr(tt, "offset")))
+            for(i in off.num)
+                offset <- offset + eval(attr(tt, "variables")[[i+1]], newdata)
+	if (!is.null(object$call$offset))
+	    offset <- offset + eval(object$call$offset, newdata)
 	mmDone <- FALSE
     }
     n <- length(object$residuals) # NROW(object$qr$qr)
@@ -649,13 +684,13 @@ predict.lm <-
         if (!missing(newdata) && missing(weights) && !is.null(object$weights) && missing(pred.var))
             warning("Assuming constant prediction variance even though model fit is weighted\n")
         if (inherits(weights, "formula")){
-            if (length(weights) != 2)
+            if (length(weights) != 2L)
                 stop("'weights' as formula should be one-sided")
             d <- if(missing(newdata) || is.null(newdata))
                 model.frame(object)
             else
                 newdata
-            weights <- eval(weights[[2]], d, environment(weights))
+            weights <- eval(weights[[2L]], d, environment(weights))
         }
     }
 
@@ -692,7 +727,7 @@ predict.lm <-
 	## asgn <- attrassign(mm, tt) :
 	aa <- attr(mm, "assign")
 	ll <- attr(tt, "term.labels")
-	hasintercept <- attr(tt, "intercept") > 0
+	hasintercept <- attr(tt, "intercept") > 0L
 	if (hasintercept)
 	    ll <- c("(Intercept)", ll)
 	aaa <- factor(aa, labels = ll)
@@ -714,22 +749,22 @@ predict.lm <-
                 Rinv <- qr.solve(qr.R(object$qr)[p1, p1])
             }
             if(hasintercept)
-                X <- sweep(X, 2, avx)
-            unpiv <- rep.int(0, NCOL(X))
+                X <- sweep(X, 2L, avx, check.margin=FALSE)
+            unpiv <- rep.int(0L, NCOL(X))
             unpiv[piv] <- p1
             ## Predicted values will be set to 0 for any term that
             ## corresponds to columns of the X-matrix that are
             ## completely aliased with earlier columns.
-            for (i in seq(1, nterms, length = nterms)) {
+            for (i in seq.int(1L, nterms, length.out = nterms)) {
                 iipiv <- asgn[[i]]      # Columns of X, ith term
                 ii <- unpiv[iipiv]      # Corresponding rows of Rinv
-                iipiv[ii == 0] <- 0
+                iipiv[ii == 0L] <- 0L
                 predictor[, i] <-
-                    if(any(iipiv) > 0) X[, iipiv, drop = FALSE] %*% beta[iipiv]
+                    if(any(iipiv > 0L)) X[, iipiv, drop = FALSE] %*% beta[iipiv]
                     else 0
                 if (se.fit || interval != "none")
                     ip[, i] <-
-                        if(any(iipiv) > 0)
+                        if(any(iipiv > 0L))
                             as.matrix(X[, iipiv, drop = FALSE] %*%
                                       Rinv[ii, , drop = FALSE])^2 %*% rep.int(res.var, p)
                         else 0
@@ -740,7 +775,7 @@ predict.lm <-
                     ip <- ip[, terms, drop = FALSE]
             }
         } else { # no terms
-            predictor <- ip <- matrix(0, n,0)
+            predictor <- ip <- matrix(0, n, 0L)
         }
 	attr(predictor, 'constant') <- if (hasintercept) termsconst else 0
     }
@@ -787,10 +822,10 @@ effects.lm <- function(object, set.sign = FALSE, ...)
     if(set.sign) {
 	dd <- coef(object)
 	if(is.matrix(eff)) {
-	    r <- 1:dim(dd)[1]
+	    r <- 1L:dim(dd)[1L]
 	    eff[r,  ] <- sign(dd) * abs(eff[r,	])
 	} else {
-	    r <- 1:length(dd)
+	    r <- seq_along(dd)
 	    eff[r] <- sign(dd) * abs(eff[r])
 	}
     }
@@ -801,10 +836,11 @@ effects.lm <- function(object, set.sign = FALSE, ...)
 
 model.matrix.lm <- function(object, ...)
 {
-    if(n_match <- match("x", names(object), 0)) object[[n_match]]
+    if(n_match <- match("x", names(object), 0L)) object[[n_match]]
     else {
         data <- model.frame(object, xlev = object$xlevels, ...)
-        NextMethod("model.matrix", data = data, contrasts = object$contrasts)
+        NextMethod("model.matrix", data = data,
+                   contrasts.arg = object$contrasts)
     }
 }
 
@@ -812,7 +848,7 @@ model.matrix.lm <- function(object, ...)
 predict.mlm <-
     function(object, newdata, se.fit = FALSE, na.action = na.pass, ...)
 {
-    if(missing(newdata)) return(object$fitted)
+    if(missing(newdata)) return(object$fitted.values)
     if(se.fit)
 	stop("the 'se.fit' argument is not yet implemented for \"mlm\" objects")
     if(missing(newdata)) {
@@ -825,7 +861,7 @@ predict.mlm <-
         m <- model.frame(Terms, newdata, na.action = na.action,
                          xlev = object$xlevels)
         if(!is.null(cl <- attr(Terms, "dataClasses"))) .checkMFClasses(cl, m)
-        X <- model.matrix(Terms, m, contrasts = object$contrasts)
+        X <- model.matrix(Terms, m, contrasts.arg = object$contrasts)
 	offset <- if (!is.null(off.num <- attr(tt, "offset")))
 	    eval(attr(tt, "variables")[[off.num+1]], newdata)
 	else if (!is.null(object$offset))
@@ -834,13 +870,13 @@ predict.mlm <-
     piv <- object$qr$pivot[seq(object$rank)]
     pred <- X[, piv, drop = FALSE] %*% object$coefficients[piv,]
     if ( !is.null(offset) ) pred <- pred + offset
-    if(inherits(object, "mlm")) pred else pred[, 1]
+    if(inherits(object, "mlm")) pred else pred[, 1L]
 }
 
 ## from base/R/labels.R
 labels.lm <- function(object, ...)
 {
     tl <- attr(object$terms, "term.labels")
-    asgn <- object$assign[object$qr$pivot[1:object$rank]]
+    asgn <- object$assign[object$qr$pivot[1L:object$rank]]
     tl[unique(asgn)]
 }
