@@ -1,6 +1,23 @@
+#  File src/library/utils/R/str.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 ####------ str : show STRucture of an R object
 str <- function(object, ...) UseMethod("str")
 
+## FIXME: convert to use sQuote
 str.data.frame <- function(object, ...)
 {
     ## Method to 'str' for  'data.frame' objects
@@ -26,21 +43,50 @@ str.data.frame <- function(object, ...)
 
 str.POSIXt <- function(object, ...) {
     cl <- oldClass(object)
-    cat("'", cl[min(2, length(cl))],"', format:", sep = "")
-    str(format(object), ...)
+    ## be careful to be fast for large object:
+    n <- length(object)
+    if(n >= 1000L) object <- object[seq_len(1000L)]
+
+    give.length <- TRUE ## default
+    ## use 'give.length' when specified, else default = give.head
+    if(length(larg <- list(...))) {
+	nl <- names(larg)
+	iGiveHead <- which(nl == "give.head")
+	if (any(Bgl <- nl == "give.length"))
+	    give.length <- larg[[which(Bgl)]]
+	else if(length(iGiveHead))
+	    give.length <- larg[[iGiveHead]]
+	if(length(iGiveHead)) # eliminate it from arg.list
+	    larg <- larg[ - iGiveHead ]
+	if(is.numeric(larg[["nest.lev"]]) &&
+	   is.numeric(v.len <- larg[["vec.len"]])) # typical call from data.frame
+	    ## diminuish length for typical call:
+	    larg[["vec.len"]] <-
+		min(larg[["vec.len"]],
+		    (larg[["width"]]- nchar(larg[["indent.str"]]) -31)%/% 19)
+    }
+
+    le.str <- if(give.length) paste("[1:",as.character(n),"]", sep="")
+    cat(" ", cl[min(2, length(cl))], le.str,", format: ", sep = "")
+    do.call(str, c(list(format(object), give.head = FALSE), larg))
 }
 
-strOptions <- function(strict.width = "no", digits.d = 3, vec.len = 4)
-    list(strict.width = strict.width, digits.d = digits.d, vec.len = vec.len)
+strOptions <- function(strict.width = "no", digits.d = 3, vec.len = 4,
+		       formatNum = function(x, ...)
+		       format(x, trim=TRUE, drop0trailing=TRUE, ...))
+    list(strict.width = strict.width, digits.d = digits.d, vec.len = vec.len,
+	 formatNum = match.fun(formatNum))
 
 str.default <-
     function(object, max.level = NA, vec.len = strO$vec.len,
-             digits.d = strO$digits.d,
-	     nchar.max = 128, give.attr = TRUE, give.length = TRUE,
+	     digits.d = strO$digits.d,
+	     nchar.max = 128, give.attr = TRUE,
+	     give.head = TRUE, give.length = give.head,
 	     width = getOption("width"), nest.lev = 0,
 	     indent.str= paste(rep.int(" ", max(0,nest.lev+1)), collapse= ".."),
 	     comp.str="$ ", no.list = FALSE, envir = baseenv(),
-             strict.width = strO$strict.width,
+	     strict.width = strO$strict.width,
+	     formatNum = strO$formatNum,
 	     ...)
 {
     ## Purpose: Display STRucture of any R - object (in a compact form).
@@ -49,21 +95,26 @@ str.default <-
     ## Author: Martin Maechler <maechler@stat.math.ethz.ch>	1990--1997
 
     ## Get defaults for these
-    oDefs <- c("vec.len", "digits.d", "strict.width")
+    oDefs <- c("vec.len", "digits.d", "strict.width", "formatNum")
     ## from
     strO <- getOption("str")
-    if(!is.list(strO) || !all(names(strO) %in% oDefs)) {
+    if (!is.list(strO)) {
 	warning("invalid options('str') -- using defaults instead")
 	strO <- strOptions()
     }
-
+    else {
+        if (!all(names(strO) %in% oDefs))
+            warning("invalid components in options('str'): ",
+                    paste(setdiff(names(strO), oDefs), collapse = ", "))
+        strO <- modifyList(strOptions(), strO)
+    }
     strict.width <- match.arg(strict.width, choices = c("no", "cut", "wrap"))
     if(strict.width != "no") {
 	## using eval() would be cleaner, but fails inside capture.output():
-	ss <- capture.output(str(object, max.level = max.level,
+	ss <- capture.output(str.default(object, max.level = max.level,
 				 vec.len = vec.len, digits.d = digits.d,
 				 nchar.max = nchar.max,
-				 give.attr= give.attr, give.length= give.length,
+				 give.attr= give.attr, give.head= give.head, give.length= give.length,
 				 width = width, nest.lev = nest.lev,
 				 indent.str = indent.str, comp.str= comp.str,
 				 no.list= no.list || is.data.frame(object),
@@ -91,6 +142,9 @@ str.default <-
 	if(any(ii)) x[ii] <- P0(trimmed[ii], Sep, ch)
 	x
     }
+    pClass <- function(cls)
+	paste("Class", if(length(cls) > 1) "es",
+              " '", paste(cls, collapse = "', '"), "' ", sep="")
 
     ## le.str: not used for arrays:
     le.str <-
@@ -102,9 +156,7 @@ str.default <-
     ## NON interesting attributes:
     std.attr <- "names"
 
-    cl <-
-	if((S4 <- isS4(object)))
-	    class(object) else attr(object, "class")
+    cl <- if((S4 <- isS4(object))) class(object) else oldClass(object)
     has.class <- S4 || !is.null(cl) # S3 or S4
     mod <- ""; char.like <- FALSE
     if(give.attr) a <- attributes(object)#-- save for later...
@@ -133,7 +185,11 @@ str.default <-
 	if(le == 0) {
 	    if(is.d.f) std.attr <- c(std.attr, "class", "row.names")
 	    else cat(" ", if(i.pl)"pair", "list()\n",sep="")
-	} else {
+	} else { # list, length >= 1 :
+	    if(irregCl <- has.class && identical(object[[1L]], object)) {
+		le <- length(object <- unclass(object))
+		std.attr <- c(std.attr, "class")
+	    }
 	    if(no.list || (has.class &&
 			   any(sapply(paste("str", cl, sep="."),
 					#use sys.function(.) ..
@@ -142,7 +198,8 @@ str.default <-
 		## str.default is a 'NextMethod' : omit the 'List of ..'
 		std.attr <- c(std.attr, "class", if(is.d.f) "row.names")
 	    } else {
-		cat(if(i.pl) "Dotted pair list" else "List",
+		cat(if(i.pl) "Dotted pair list" else
+		    if(irregCl) paste(pClass(cl), "hidden list") else "List",
 		    " of ", le, "\n", sep="")
 	    }
 	    if (is.na(max.level) || nest.lev < max.level) {
@@ -151,7 +208,7 @@ str.default <-
 		    else { max.ncnam <- max(nchar(nam.ob, type="w"))
 			   format(nam.ob, width = max.ncnam, justify="left")
 		       }
-		for(i in 1:le) {
+		for(i in seq_len(le)) {
 		    cat(indent.str, comp.str, nam.ob[i], ":", sep="")
 		    envir <- # pass envir for 'promise' components:
 			if(typeof(object[[i]]) == "promise") {
@@ -161,7 +218,7 @@ str.default <-
                         indent.str = paste(indent.str,".."),
                         nchar.max = nchar.max, max.level = max.level,
                         vec.len = vec.len, digits.d = digits.d,
-                        give.attr = give.attr, give.length = give.length,
+                        give.attr = give.attr, give.head= give.head, give.length = give.length,
                         width = width, envir = envir)
 		}
 	    }
@@ -177,15 +234,16 @@ str.default <-
 		mod <- substr(mode(object), 1, 4)
 		if     (mod == "nume")
 		    mod <- if(is.integer(object)) "int"
-		    else if(has.class) cl[1] else "num"
+		    else if(has.class) cl[1L] else "num"
 		else if(mod == "char") { mod <- "chr"; char.like <- TRUE }
 		else if(mod == "comp") mod <- "cplx" #- else: keep 'logi'
 		if(is.array(object)) {
-		    di. <- dim(object)
+		    rnk <- length(di. <- dim(object))
 		    di <- P0(ifelse(di. > 1, "1:",""), di.,
 			     ifelse(di. > 0, "" ," "))
-		    le.str <- paste(c("[", P0(di[-length(di)], ", "),
-				      di[length(di)], "]"), collapse = "")
+		    pDi <- function(...) paste(c("[", ..., "]"), collapse = "")
+		    le.str <- (if(rnk == 1) pDi(di[1L], "(1d)") else
+			       pDi(P0(di[-rnk], ", "), di[rnk]))
 		    std.attr <- "dim" #- "names"
 		} else if(!is.null(names(object))) {
 		    mod <- paste("Named", mod)
@@ -234,23 +292,23 @@ str.default <-
 #	    std.attr <- c(std.attr, "tspar")
 	} else if(stats::is.ts(object)) {
 	    tsp.a <- stats::tsp(object)
-	    str1 <- P0(" Time-Series ", le.str, " from ", format(tsp.a[1]),
-		       " to ", format(tsp.a[2]), ":")
+	    str1 <- P0(" Time-Series ", le.str, " from ", format(tsp.a[1L]),
+		       " to ", format(tsp.a[2L]), ":")
 	    std.attr <- c("tsp","class") #- "names"
 	} else if (is.factor(object)) {
 	    nl <- length(lev.att <- levels(object))
 	    if(!is.character(lev.att)) {# should not happen..
 		warning("'object' does not have valid levels()")
 		nl <- 0
-	    } else lev.att <- encodeString(lev.att, na = FALSE, quote = '"')
+	    } else lev.att <- encodeString(lev.att, na.encode = FALSE, quote = '"')
 	    ord <- is.ordered(object)
 	    object <- unclass(object)
 	    if(nl) {
 		## as from 2.1.0, quotes are included ==> '-2':
 		lenl <- cumsum(3 + (nchar(lev.att, type="w") - 2))# level space
 		ml <- if(nl <= 1 || lenl[nl] <= 13)
-		    nl else which(lenl > 13)[1]
-		lev.att <- maybe_truncate(lev.att[1:ml])
+		    nl else which(lenl > 13)[1L]
+		lev.att <- maybe_truncate(lev.att[seq_len(ml)])
 ##		if((d <- lenl[ml] - if(ml>1)18 else 14) >= 3)# truncate last
 ##		    lev.att[ml] <-
 ##			P0(substring(lev.att[ml],1, nchar(lev.att[ml])-d), "..")
@@ -270,15 +328,11 @@ str.default <-
 		  c("externalptr", "weakref", "environment")) {
 	    ## Careful here, we don't want to change pointer objects
 	    if(has.class)
-		cat("Class", if(length(cl) > 1) "es",
-		" '", paste(cl, collapse = "', '"), "' ", sep="")
+                cat(pClass(cl))
 	    le <- v.len <- 0
-	    str1 <- paste("<", typeof(object), ">", sep="")
-	    if(typeof(object) == "environment") {
-                nm <- environmentName(object)
-                str1 <- if(nchar(nm)[1]) paste("<", nm , ">", sep="")
-		else paste("length", length(object), str1)
-            }
+	    str1 <-
+		if(is.environment(object)) format(object)
+		else paste("<", typeof(object), ">", sep="")
 	    has.class <- TRUE # fake for later
 	    std.attr <- "class"
 	    ## ideally we would figure out if as.character has a
@@ -287,7 +341,16 @@ str.default <-
 	    cat("Class", if(length(cl) > 1) "es",
 		" '", paste(cl, collapse = "', '"), "' ", sep="")
 	    ## If there's a str.<method>, it should have been called before!
-	    str(unclass(object),
+	    uo <- unclass(object)
+	    if(!is.null(attributes(uo)$class)) {
+		## another irregular case
+		xtr <- c(if(identical(uo, object)) { # trap infinite loop
+		    class(uo) <- NULL
+		    "unclass()-immune"
+		} else if(!is.object(object)) "not-object")
+		if(!is.null(xtr)) cat("{",xtr,"} ", sep="")
+	    }
+	    str(uo,
 		max.level = max.level, vec.len = vec.len, digits.d = digits.d,
 		indent.str = paste(indent.str,".."), nest.lev = nest.lev + 1,
 		nchar.max = nchar.max, give.attr = give.attr, width=width)
@@ -332,10 +395,10 @@ str.default <-
 		le <- length(object) # is > 1 e.g. for {A;B} language
 		format.fun <- function(x)x
 		v.len <- round(.5 * v.len)
-		if(le > 1 && typ=="language" && object[1] == "{" && object[le]=="}") {
+		if(le > 1 && typ=="language" && object[1L] == "{" && object[le]=="}") {
 		    v.len <- v.len + 2
 		    if(le >= 3) {
-			object <- c(object[1],
+			object <- c(object[1L],
 				    paste(sub("^ +", " ", object[2:(le-1)]),
 					  collapse = ";"),
 				    object[le])
@@ -356,7 +419,7 @@ str.default <-
 
 	} else if(is.logical(object)) {
 	    v.len <- 1.5 * v.len # was '3' originally (but S prints 'T' 'F' ..)
-	    format.fun <- format
+	    format.fun <- formatNum
 	} else if(is.numeric(object)) {
 	    iv.len <- round(2.5 * v.len)
 	    if(iSurv <- inherits(object, "Surv"))
@@ -369,18 +432,18 @@ str.default <-
 	    else if(iSurv)
 		le <- length(object <- as.character(object))
 	    if(int.surv || (all(ao > 1e-10 | ao==0) && all(ao < 1e10| ao==0) &&
-			    all(ob == signif(ob, digits.d)))) {
-		if(!iSurv || di.[2] == 2)
+			    all(abs(ob - signif(ob, digits.d)) <= 9e-16*ao))) {
+		if(!iSurv || di.[2L] == 2) # "Surv" : implemented as matrix
 		    ## use integer-like length
 		    v.len <- iv.len
-		format.fun <- function(x)x
+		format.fun <- formatNum
 	    } else {
 		v.len <- round(1.25 * v.len)
-		format.fun <- format
+		format.fun <- formatNum
 	    }
 	} else if(is.complex(object)) {
 	    v.len <- round(.75 * v.len)
-	    format.fun <- format
+	    format.fun <- formatNum
 	}
 
 	## Not sure, this is ever triggered:
@@ -389,8 +452,8 @@ str.default <-
 	if(char.like) {
 	    ## if object is very long, drop the rest which won't be used anyway:
 	    max.len <- max(100, width %/% 3 + 1, if(!missing(vec.len)) vec.len)
-	    if(le > max.len) object <- object[1:max.len]
-	    encObj <- encodeString(object, quote= '"', na= FALSE)
+	    if(le > max.len) object <- object[seq_len(max.len)]
+	    encObj <- encodeString(object, quote= '"', na.encode= FALSE)
 					#O: encodeString(object)
 	    v.len <-
 		if(missing(vec.len)) {
@@ -400,26 +463,26 @@ str.default <-
 		else round(v.len)
 	    ile <- min(le, v.len)
 	    if(ile >= 1) ## truncate if LONG char:
-		object <- maybe_truncate(encObj[1:ile])
-					#O: encodeString(object, quote= '"', na= FALSE)
+		object <- maybe_truncate(encObj[seq_len(ile)])
+					#O: encodeString(object, quote= '"', na.encode= FALSE)
 	    formObj <- function(x) paste(as.character(x), collapse=" ")
 	}
 	else {
 	    if(!exists("format.fun", inherits=TRUE)) #-- define one --
 		format.fun <-
-		    if(mod == 'num' || mod == 'cplx') format else as.character
+		    if(mod == "num" || mod == "cplx") format else as.character
 	    ## v.len <- max(1,round(v.len))
 	    ile <- min(v.len, le)
 	    formObj <- function(x) paste(format.fun(x), collapse = " ")
 	}
 
-	cat(str1, " ", formObj(if(ile >= 1) object[1:ile] else
-			       if(v.len > 0) object),
+	cat(if(give.head) P0(str1, " "),
+	    formObj(if(ile >= 1) object[seq_len(ile)] else if(v.len > 0) object),
 	    if(le > v.len) " ...", "\n", sep="")
 
     } ## else (not function nor list)----------------------------------------
 
-    if(give.attr) { ## possible: || has.class && any(cl == 'terms')
+    if(give.attr) { ## possible: || has.class && any(cl == "terms")
 	nam <- names(a)
 	for (i in seq_along(a))
 	    if (all(nam[i] != std.attr)) {# only `non-standard' attributes:
@@ -429,7 +492,7 @@ str.default <-
 		    max.level = max.level, digits.d = digits.d,
 		    nchar.max = nchar.max,
 		    vec.len = if(nam[i] == "source") 1 else vec.len,
-		    give.attr= give.attr, give.length= give.length, width= width)
+		    give.attr= give.attr, give.head= give.head, give.length= give.length, width= width)
 	    }
     }
     invisible()	 ## invisible(object)#-- is SLOOOOW on large objects
@@ -437,26 +500,59 @@ str.default <-
 
 ## An extended `ls()' whose print method will use str() :
 ls.str <-
-    function(pos = 1, pattern, ..., envir = as.environment(pos), mode = "any")
+    function (pos = -1, name, envir, all.names = FALSE, pattern, mode = "any")
 {
-    nms <- ls(..., envir = envir, pattern = pattern)
+    if(missing(envir)) ## [for "lazy" reasons, this fails as default]
+        envir <- as.environment(pos)
+    nms <- ls(name, envir = envir, all.names=all.names, pattern=pattern)
     r <- unlist(lapply(nms, function(n)
                        exists(n, envir= envir, mode= mode, inherits=FALSE)))
     structure(nms[r], envir = envir, mode = mode, class = "ls_str")
 }
 
-lsf.str <- function(pos = 1, ..., envir = as.environment(pos))
-    ls.str(pos = pos, envir = envir, mode = "function", ...)
+lsf.str <- function(pos = -1, envir, ...) {
+    if(missing(envir)) ## [for "lazy" reasons, this fails as default]
+        envir <- as.environment(pos)
+    ls.str(pos=pos, envir=envir, mode = "function", ...)
+}
 
-print.ls_str <- function(x, max.level = 1, give.attr = FALSE, ...)
+print.ls_str <- function(x, max.level = 1, give.attr = FALSE,
+                         ..., digits = max(1, getOption("str")$digits.d))
 {
     E <- attr(x, "envir")
     stopifnot(is.environment(E))
     M <- attr(x, "mode")
+    args <- list(...)
+    if(length(args) && "digits.d" %in% names(args)) {
+        if(missing(digits))
+            digits <- args$digits.d
+        else
+            warning("'digits' and 'digits.d' are both specified and the latter is not used")
+        args$digits.d <- NULL
+    }
+    strargs <- c(list(max.level = max.level, give.attr = give.attr,
+                      digits = digits), args)
     for(nam in x) {
 	cat(nam, ": ")
-	str(get(nam, envir = E, mode = M),
-	    max.level = max.level, give.attr = give.attr, ...)
+	## check missingness, e.g. inside debug(.) :
+
+##__ Why does this give	 too many <missing> in some case?
+##__	if(eval(substitute(missing(.), list(. = as.name(nam))),
+##__		envir = E))
+##__	    cat("<missing>\n")
+##__	else
+##__	    str(get(nam, envir = E, mode = M),
+##__		max.level = max.level, give.attr = give.attr, ...)
+
+	o <- tryCatch(get(nam, envir = E, mode = M), error = function(e)e)
+	if(inherits(o, "error")) {
+	    cat(## FIXME: only works with "C" (or English) LC_MESSAGES locale!
+		if(length(grep("missing|not found", o$message)))
+		"<missing>" else o$message, "\n", sep='')
+	}
+	else
+	    do.call(str, c(list(o), strargs),
+		    quote = is.call(o)) # protect calls from evaluation
     }
     invisible(x)
 }
