@@ -1,3 +1,19 @@
+#  File src/library/utils/R/unix/download.file.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 download.file <- function(url, destfile, method,
                           quiet = FALSE, mode = "w", cacheOK = TRUE)
 {
@@ -11,18 +27,21 @@ download.file <- function(url, destfile, method,
     if(method == "auto") {
         if(capabilities("http/ftp"))
             method <- "internal"
-        else if(length(grep("^file:", url)))
+        else if(length(grep("^file:", url))) {
             method <- "internal"
-        else if(system("wget --help > /dev/null")==0)
+            url <- URLdecode(url)
+        } else if(system("wget --help > /dev/null")==0)
             method <- "wget"
         else if(system("lynx -help > /dev/null")==0)
             method <- "lynx"
         else
             stop("no download method found")
     }
-    if(method == "internal")
+    if(method == "internal") {
         status <- .Internal(download(url, destfile, quiet, mode, cacheOK))
-    else if(method == "wget") {
+        ## needed for Mac GUI from download.packages etc
+        if(!quiet) flush.console()
+    } else if(method == "wget") {
         extra <- if(quiet) " --quiet" else ""
         if(!cacheOK) extra <- paste(extra, "--cache=off")
         status <- system(paste("wget", extra, " ", shQuote(url),
@@ -31,8 +50,7 @@ download.file <- function(url, destfile, method,
         status <- system(paste("lynx -dump ", shQuote(url), " > ",
                                path.expand(destfile), sep=""))
 
-    if(status > 0)
-        warning("download had nonzero exit status")
+    if(status) warning("download had nonzero exit status")
 
     invisible(status)
 }
