@@ -24,15 +24,16 @@ download.file <- function(url, destfile, method,
     else
         match.arg(method, c("auto", "internal", "wget", "lynx"))
 
+    if(missing(mode) & length(grep("\\.(gz|bz2|tgz|zip)$", url))) mode <- "wb"
     if(method == "auto") {
         if(capabilities("http/ftp"))
             method <- "internal"
         else if(length(grep("^file:", url))) {
             method <- "internal"
             url <- URLdecode(url)
-        } else if(system("wget --help", invisible=TRUE)==0)
+        } else if(system("wget --help", invisible=TRUE) == 0L)
             method <- "wget"
-        else if(shell("lynx -help", invisible=TRUE)==0)
+        else if(shell("lynx -help", invisible=TRUE) == 0L)
             method <- "lynx"
         else
             stop("no download method found")
@@ -48,9 +49,8 @@ download.file <- function(url, destfile, method,
         status <- shell(paste("lynx -dump", url, ">",
                               path.expand(destfile)))
 
-    if(status > 0)
+    if(status > 0L)
         warning("download had nonzero exit status")
 
     invisible(status)
 }
-
