@@ -2,6 +2,7 @@
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995-2007  Robert Gentleman, Ross Ihaka and the
  *			     R Development Core Team
+ *  Copyright (C) 2003-2009 The R Foundation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,13 +19,13 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <Defn.h> /*-- Maybe modularize into own Coerce.h ..*/
+#define R_MSG_mode	_("invalid 'mode' argument")
+#define R_MSG_list_vec	_("applies only to lists and vectors")
 #include <Rmath.h>
 #include <Print.h>
 
@@ -296,12 +297,36 @@ SEXP attribute_hidden StringFromInteger(int x, int *warn)
     else return mkChar(EncodeInteger(x, w));
 }
 
+static const char* dropTrailing0(char *s, char cdec)
+{
+    /* Note that  's'  is modified */
+    char *p = s;
+    for (p = s; *p; p++) {
+	if(*p == cdec) {
+	    char *replace = p++;
+	    while ('0' <= *p  &&  *p <= '9')
+		if(*(p++) != '0')
+		    replace = p;
+	    if(replace != p)
+		while((*(replace++) = *(p++)))
+		    ;
+	    break;
+	}
+    }
+    return s;
+}
+
 SEXP attribute_hidden StringFromReal(double x, int *warn)
 {
     int w, d, e;
     formatReal(&x, 1, &w, &d, &e, 0);
     if (ISNA(x)) return NA_STRING;
-    else return mkChar(EncodeReal(x, w, d, e, OutDec));
+    else
+	/* Note that we recast EncodeReal()'s value to possibly modify it
+	 * destructively; this is harmless here (in a sequential
+	 * environment), as mkChar() creates a copy */
+	return mkChar(dropTrailing0((char *)EncodeReal(x, w, d, e, OutDec),
+				     OutDec));
 }
 
 SEXP attribute_hidden StringFromComplex(Rcomplex x, int *warn)
@@ -309,7 +334,7 @@ SEXP attribute_hidden StringFromComplex(Rcomplex x, int *warn)
     int wr, dr, er, wi, di, ei;
     formatComplex(&x, 1, &wr, &dr, &er, &wi, &di, &ei, 0);
     if (ISNA(x.r) || ISNA(x.i)) return NA_STRING;
-    else
+    else /* EncodeComplex has its own anti-trailing-0 care :*/
 	return mkChar(EncodeComplex(x, wr, dr, er, wi, di, ei, OutDec));
 }
 
@@ -378,7 +403,7 @@ static SEXP coerceToSymbol(SEXP v)
     SEXP ans = R_NilValue;
     int warn = 0;
     if (length(v) <= 0)
-	error(_("invalid data of mode \"%s\" (too short)"),
+	error(_("invalid data of mode '%s' (too short)"),
 	      type2char(TYPEOF(v)));
     PROTECT(v);
     switch(TYPEOF(v)) {
@@ -415,9 +440,9 @@ static SEXP coerceToLogical(SEXP v)
     int i, n, warn = 0;
     PROTECT(ans = allocVector(LGLSXP, n = length(v)));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     DUPLICATE_ATTRIB(ans, v);
@@ -456,9 +481,9 @@ static SEXP coerceToInteger(SEXP v)
     int i, n, warn = 0;
     PROTECT(ans = allocVector(INTSXP, n = LENGTH(v)));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     DUPLICATE_ATTRIB(ans, v);
@@ -497,9 +522,9 @@ static SEXP coerceToReal(SEXP v)
     int i, n, warn = 0;
     PROTECT(ans = allocVector(REALSXP, n = LENGTH(v)));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     DUPLICATE_ATTRIB(ans, v);
@@ -538,9 +563,9 @@ static SEXP coerceToComplex(SEXP v)
     int i, n, warn = 0;
     PROTECT(ans = allocVector(CPLXSXP, n = LENGTH(v)));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     DUPLICATE_ATTRIB(ans, v);
@@ -580,9 +605,9 @@ static SEXP coerceToRaw(SEXP v)
 
     PROTECT(ans = allocVector(RAWSXP, n = LENGTH(v)));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     DUPLICATE_ATTRIB(ans, v);
@@ -651,9 +676,9 @@ static SEXP coerceToString(SEXP v)
     int i, n, savedigits, warn = 0;
     PROTECT(ans = allocVector(STRSXP, n = LENGTH(v)));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     DUPLICATE_ATTRIB(ans, v);
@@ -700,9 +725,9 @@ static SEXP coerceToExpression(SEXP v)
 	n = LENGTH(v);
 	PROTECT(ans = allocVector(EXPRSXP, n));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
 	switch (TYPEOF(v)) {
@@ -749,9 +774,9 @@ static SEXP coerceToVectorList(SEXP v)
     n = length(v);
     PROTECT(ans = allocVector(VECSXP, n));
 #ifdef R_MEMORY_PROFILING
-    if (TRACE(v)){
+    if (RTRACE(v)){
        memtrace_report(v,ans);
-       SET_TRACE(ans,1);
+       SET_RTRACE(ans,1);
     }
 #endif
     switch (TYPEOF(v)) {
@@ -954,9 +979,9 @@ static SEXP coerceVectorList(SEXP v, SEXPTYPE type)
 	n = length(v);
 	PROTECT(rval = allocVector(type, n));
 #ifdef R_MEMORY_PROFILING
-	if (TRACE(v)){
+	if (RTRACE(v)){
 	   memtrace_report(v, rval);
-	   SET_TRACE(rval,1);
+	   SET_RTRACE(rval,1);
 	}
 #endif
 	for (i = 0; i < n;  i++) {
@@ -1042,6 +1067,15 @@ SEXP coerceVector(SEXP v, SEXPTYPE type)
 
     if (TYPEOF(v) == type)
 	return v;
+    /* code to allow classes to extend ENVSXP, SYMSXP, etc */
+    if(IS_S4_OBJECT(v) && TYPEOF(v) == S4SXP) {
+        SEXP vv = R_getS4DataSlot(v, ANYSXP);
+	if(vv == R_NilValue)
+	  error(_("no method for coercing this S4 class to a vector"));
+	else if(TYPEOF(vv) == type)
+	  return vv;
+	v = vv;
+    }
 
     switch (TYPEOF(v)) {
 #ifdef NOTYET
@@ -1234,7 +1268,29 @@ static SEXP ascommon(SEXP call, SEXP u, SEXPTYPE type)
     return u;/* -Wall */
 }
 
-/* A historical anomaly: as.character is primitive, the other ops are not */
+SEXP asCharacterFactor(SEXP x)
+{
+    SEXP ans;
+
+    if( !inherits(x, "factor") )
+        error(_("attempting to coerce non-factor"));
+
+    int i, n = LENGTH(x);
+    SEXP labels = getAttrib(x, install("levels"));
+    PROTECT(ans = allocVector(STRSXP, n));
+    for(i = 0; i < n; i++) {
+      int ii = INTEGER(x)[i];
+      SET_STRING_ELT(ans, i,
+		   (ii == NA_INTEGER) ? NA_STRING
+		   : STRING_ELT(labels, ii - 1));
+    }
+    UNPROTECT(1);
+    return ans;
+}
+
+
+/* the "ascharacter" name is a historical anomaly: as.character used to be the
+ * only primitive;  now, all these ops are : */
 SEXP attribute_hidden do_ascharacter(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP ans, x;
@@ -1292,6 +1348,7 @@ SEXP attribute_hidden do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
     else
 	type = str2type(CHAR(STRING_ELT(CADR(args), 0))); /* ASCII */
 
+
     if(TYPEOF(x) == type) {
 	switch(type) {
 	case LGLSXP:
@@ -1309,6 +1366,13 @@ SEXP attribute_hidden do_asvector(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
     }
 
+    if(IS_S4_OBJECT(x) && TYPEOF(x) == S4SXP) {
+        SEXP v = R_getS4DataSlot(x, ANYSXP);
+	if(v == R_NilValue)
+	  error(_("no method for coercing this S4 class to a vector"));
+	x = v;
+    }
+
     switch(type) {/* only those are valid : */
     case SYMSXP:
     case LGLSXP:
@@ -1445,18 +1509,20 @@ int asLogical(SEXP x)
 	case LGLSXP:
 	    return LOGICAL(x)[0];
 	case INTSXP:
-	    return Rf_LogicalFromInteger(INTEGER(x)[0], &warn);
+	    return LogicalFromInteger(INTEGER(x)[0], &warn);
 	case REALSXP:
-	    return Rf_LogicalFromReal(REAL(x)[0], &warn);
+	    return LogicalFromReal(REAL(x)[0], &warn);
 	case CPLXSXP:
-	    return Rf_LogicalFromComplex(COMPLEX(x)[0], &warn);
+	    return LogicalFromComplex(COMPLEX(x)[0], &warn);
 	case STRSXP:
-	    return Rf_LogicalFromString(STRING_ELT(x, 0), &warn);
+	    return LogicalFromString(STRING_ELT(x, 0), &warn);
+	case RAWSXP:
+	    return LogicalFromInteger((int)RAW(x)[0], &warn);
 	default:
 	    UNIMPLEMENTED_TYPE("asLogical", x);
 	}
     } else if(TYPEOF(x) == CHARSXP) {
-	    return Rf_LogicalFromString(x, &warn);
+	    return LogicalFromString(x, &warn);
     }
     return NA_LOGICAL;
 }
@@ -1468,27 +1534,27 @@ int asInteger(SEXP x)
     if (isVectorAtomic(x) && LENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
 	case LGLSXP:
-	    return Rf_IntegerFromLogical(LOGICAL(x)[0], &warn);
+	    return IntegerFromLogical(LOGICAL(x)[0], &warn);
 	case INTSXP:
 	    return INTEGER(x)[0];
 	case REALSXP:
-	    res = Rf_IntegerFromReal(REAL(x)[0], &warn);
-	    Rf_CoercionWarning(warn);
+	    res = IntegerFromReal(REAL(x)[0], &warn);
+	    CoercionWarning(warn);
 	    return res;
 	case CPLXSXP:
-	    res = Rf_IntegerFromComplex(COMPLEX(x)[0], &warn);
-	    Rf_CoercionWarning(warn);
+	    res = IntegerFromComplex(COMPLEX(x)[0], &warn);
+	    CoercionWarning(warn);
 	    return res;
 	case STRSXP:
-	    res = Rf_IntegerFromString(STRING_ELT(x, 0), &warn);
-	    Rf_CoercionWarning(warn);
+	    res = IntegerFromString(STRING_ELT(x, 0), &warn);
+	    CoercionWarning(warn);
 	    return res;
 	default:
 	    UNIMPLEMENTED_TYPE("asInteger", x);
 	}
     } else if(TYPEOF(x) == CHARSXP) {
-	res = Rf_IntegerFromString(x, &warn);
-	Rf_CoercionWarning(warn);
+	res = IntegerFromString(x, &warn);
+	CoercionWarning(warn);
 	return res;
     }
     return NA_INTEGER;
@@ -1502,29 +1568,29 @@ double asReal(SEXP x)
     if (isVectorAtomic(x) && LENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
 	case LGLSXP:
-	    res = Rf_RealFromLogical(LOGICAL(x)[0], &warn);
-	    Rf_CoercionWarning(warn);
+	    res = RealFromLogical(LOGICAL(x)[0], &warn);
+	    CoercionWarning(warn);
 	    return res;
 	case INTSXP:
-	    res = Rf_RealFromInteger(INTEGER(x)[0], &warn);
-	    Rf_CoercionWarning(warn);
+	    res = RealFromInteger(INTEGER(x)[0], &warn);
+	    CoercionWarning(warn);
 	    return res;
 	case REALSXP:
 	    return REAL(x)[0];
 	case CPLXSXP:
-	    res = Rf_RealFromComplex(COMPLEX(x)[0], &warn);
-	    Rf_CoercionWarning(warn);
+	    res = RealFromComplex(COMPLEX(x)[0], &warn);
+	    CoercionWarning(warn);
 	    return res;
 	case STRSXP:
-	    res = Rf_RealFromString(STRING_ELT(x, 0), &warn);
-	    Rf_CoercionWarning(warn);
+	    res = RealFromString(STRING_ELT(x, 0), &warn);
+	    CoercionWarning(warn);
 	    return res;
 	default:
 	    UNIMPLEMENTED_TYPE("asReal", x);
 	}
     } else if(TYPEOF(x) == CHARSXP) {
-	res = Rf_RealFromString(x, &warn);
-	Rf_CoercionWarning(warn);
+	res = RealFromString(x, &warn);
+	CoercionWarning(warn);
 	return res;
     }
     return NA_REAL;
@@ -1535,26 +1601,36 @@ Rcomplex asComplex(SEXP x)
     int warn = 0;
     Rcomplex z;
 
-    z.r = NA_REAL;
-    z.i = NA_REAL;
     if (isVectorAtomic(x) && LENGTH(x) >= 1) {
 	switch (TYPEOF(x)) {
 	case LGLSXP:
-	    return Rf_ComplexFromLogical(LOGICAL(x)[0], &warn);
+	    z = ComplexFromLogical(LOGICAL(x)[0], &warn);
+	    CoercionWarning(warn);
+	    return z;
 	case INTSXP:
-	    return Rf_ComplexFromInteger(INTEGER(x)[0], &warn);
+	    z = ComplexFromInteger(INTEGER(x)[0], &warn);
+	    CoercionWarning(warn);
+	    return z;
 	case REALSXP:
-	    return Rf_ComplexFromReal(REAL(x)[0], &warn);
+	    z = ComplexFromReal(REAL(x)[0], &warn);
+	    CoercionWarning(warn);
+	    return z;
 	case CPLXSXP:
 	    return COMPLEX(x)[0];
 	case STRSXP:
-	    return Rf_ComplexFromString(STRING_ELT(x, 0), &warn);
+	    z = ComplexFromString(STRING_ELT(x, 0), &warn);
+	    CoercionWarning(warn);
+	    return z;
 	default:
 	    UNIMPLEMENTED_TYPE("asComplex", x);
 	}
     } else if(TYPEOF(x) == CHARSXP) {
-	return Rf_ComplexFromString(x, &warn);
+	z = ComplexFromString(x, &warn);
+	CoercionWarning(warn);
+	return z;
     }
+    z.r = NA_REAL;
+    z.i = NA_REAL;
     return z;
 }
 
@@ -1577,10 +1653,19 @@ SEXP attribute_hidden do_is(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* These are all builtins, so we do not need to worry about
        evaluating arguments in DispatchOrEval */
     if(PRIMVAL(op) >= 100 && PRIMVAL(op) < 200 &&
-       isObject(CAR(args)) &&
-       DispatchOrEval(call, op, CHAR(PRINTNAME(CAR(call))), /* ASCII */
-		      args, rho, &ans, 0,1))
-	return(ans);
+       isObject(CAR(args))) {
+	/* This used CHAR(PRINTNAME(CAR(call))), but that is not
+	   necessarily correct, e.g. when called from lapply() */
+	const char *nm;
+	switch(PRIMVAL(op)) {
+	case 100: nm = "is.numeric"; break;
+	case 101: nm = "is.matrix"; break;
+	case 102: nm = "is.array"; break;
+	default: nm = ""; /* -Wall */
+	}
+	if(DispatchOrEval(call, op, nm, args, rho, &ans, 0, 1))
+	    return(ans);
+    }
 
     PROTECT(ans = allocVector(LGLSXP, 1));
     switch (PRIMVAL(op)) {
@@ -2078,7 +2163,7 @@ SEXP attribute_hidden do_call(SEXP call, SEXP op, SEXP args, SEXP rho)
        better error message.
      */
     if (!isString(rfun) || length(rfun) != 1)
-	errorcall_return(call, R_MSG_A1_char);
+	errorcall_return(call, _("first argument must be a character string"));
     PROTECT(rfun = install(translateChar(STRING_ELT(rfun, 0))));
     PROTECT(evargs = duplicate(CDR(args)));
     for (rest = evargs; rest != R_NilValue; rest = CDR(rest))
@@ -2108,7 +2193,7 @@ SEXP attribute_hidden do_docall(SEXP call, SEXP op, SEXP args, SEXP rho)
 	error(_("'what' must be a character string or a function"));
 
     if (!isNull(args) && !isNewList(args))
-	error(R_MSG_A2_list);
+	error(_("'args' must be a list"));
 
     if (!isEnvironment(envir))
 	error(_("'envir' must be an environment"));
@@ -2181,6 +2266,8 @@ SEXP substitute(SEXP lang, SEXP rho)
 		    do {
 			t = PREXPR(t);
 		    } while(TYPEOF(t) == PROMSXP);
+		    /* make sure code will not be modified: */
+		    if (NAMED(t) < 2) SET_NAMED(t, 2);
 		    return t;
 		}
 		else if (TYPEOF(t) == DOTSXP)
@@ -2323,6 +2410,18 @@ static int class2type(const char *s)
     /* cannot get here return -1; */
 }
 
+static SEXP do_unsetS4(SEXP obj, SEXP newClass) {
+  if(isNull(newClass))  { /* NULL class is only valid for S3 objects */
+    warning(_("Setting class(x) to NULL;   result will no longer be an S4 object"));
+  }
+  else if(length(newClass) > 1)
+    warning(_("Setting class(x) to multiple strings (\"%s\", \"%s\", ...); result will no longer be an S4 object"), translateChar(STRING_ELT(newClass, 0)), translateChar(STRING_ELT(newClass, 1)));
+  else
+    warning(_("Setting class(x) to \"%s\" sets attribute to NULL; result will no longer be an S4 object"), CHAR(asChar(newClass)));
+  UNSET_S4_OBJECT(obj);
+  return obj;
+}
+
 /* set the class to value, and return the modified object.  This is
    NOT a primitive assignment operator , because there is no code in R
    that changes type in place. */
@@ -2331,6 +2430,8 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
     int nProtect = 0;
     if(isNull(value)) {
 	setAttrib(obj, R_ClassSymbol, value);
+	if(IS_S4_OBJECT(obj)) /* NULL class is only valid for S3 objects */
+	  do_unsetS4(obj, value);
 	return obj;
     }
     if(TYPEOF(value) != STRSXP) {
@@ -2339,8 +2440,11 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	PROTECT(value = coerceVector(duplicate(value), STRSXP));
 	nProtect++;
     }
-    if(length(value) > 1)
+    if(length(value) > 1) {
 	setAttrib(obj, R_ClassSymbol, value);
+	if(IS_S4_OBJECT(obj)) /*  multiple strings only valid for S3 objects */
+	  do_unsetS4(obj, value);
+    }
     else if(length(value) == 0) {
 	UNPROTECT(nProtect); nProtect = 0;
 	error(_("invalid replacement object to be a class string"));
@@ -2356,6 +2460,8 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	/*  assigning type as a class deletes an explicit class attribute. */
 	if(valueType != -1) {
 	    setAttrib(obj, R_ClassSymbol, R_NilValue);
+	    if(IS_S4_OBJECT(obj)) /* NULL class is only valid for S3 objects */
+	      do_unsetS4(obj, value);
 	    if(classTable[whichType].canChange) {
 		PROTECT(obj = ascommon(call, obj, valueType));
 		nProtect++;
@@ -2367,6 +2473,8 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 	}
 	else if(!strcmp("numeric", valueString)) {
 	    setAttrib(obj, R_ClassSymbol, R_NilValue);
+	    if(IS_S4_OBJECT(obj)) /* NULL class is only valid for S3 objects */
+	      do_unsetS4(obj, value);
 	    switch(TYPEOF(obj)) {
 	    case INTSXP: case REALSXP: break;
 	    default: PROTECT(obj = coerceVector(obj, REALSXP));
@@ -2380,11 +2488,15 @@ static SEXP R_set_class(SEXP obj, SEXP value, SEXP call)
 		error(_("invalid to set the class to matrix unless the dimension attribute is of length 2 (was %d)"),
 		 length(getAttrib(obj, R_DimSymbol)));
 	    setAttrib(obj, R_ClassSymbol, R_NilValue);
+	    if(IS_S4_OBJECT(obj))
+	      do_unsetS4(obj, value);
 	}
 	else if(!strcmp("array", valueString)) {
 	    if(length(getAttrib(obj, R_DimSymbol))<= 0)
 		error(_("cannot set class to \"array\" unless the dimension attribute has length > 0"));
 	    setAttrib(obj, R_ClassSymbol, R_NilValue);
+	    if(IS_S4_OBJECT(obj)) /* NULL class is only valid for S3 objects */
+	      UNSET_S4_OBJECT(obj);
 	}
 	else { /* set the class but don't do the coercion; that's
 		  supposed to be done by an as() method */
