@@ -19,8 +19,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -34,8 +32,6 @@
 #define __MAIN__
 #include "Defn.h"
 #include "Rinterface.h"
-#include "Graphics.h"
-#include <Rdevices.h>		/* for InitGraphics */
 #include "IOStuff.h"
 #include "Fileio.h"
 #include "Parse.h"
@@ -45,12 +41,8 @@
 # include <locale.h>
 #endif
 
-#ifdef HAVE_LANGINFO_CODESET
-# include <langinfo.h>
-#endif
-
 #ifdef ENABLE_NLS
-void attribute_hidden nl_Rdummy()
+void attribute_hidden nl_Rdummy(void)
 {
     /* force this in as packages use it */
     dgettext("R", "dummy - do not translate");
@@ -79,25 +71,30 @@ void Rf_callToplevelHandlers(SEXP expr, SEXP value, Rboolean succeeded,
 
 static int ParseBrowser(SEXP, SEXP);
 
- 
-extern void InitDynload();
+
+extern void InitDynload(void);
 
 	/* Read-Eval-Print Loop [ =: REPL = repl ] with input from a file */
 
-static void R_ReplFile(FILE *fp, SEXP rho, int savestack, int browselevel)
+static void R_ReplFile(FILE *fp, SEXP rho)
 {
     ParseStatus status;
     int count=0;
-
+    SrcRefState ParseState;
+    int savestack;
+    
+    R_InitSrcRefState(&ParseState);
+    savestack = R_PPStackTop;    
     for(;;) {
 	R_PPStackTop = savestack;
-	R_CurrentExpr = R_Parse1File(fp, 1, &status);
+	R_CurrentExpr = R_Parse1File(fp, 1, &status, &ParseState);
 	switch (status) {
 	case PARSE_NULL:
 	    break;
 	case PARSE_OK:
 	    R_Visible = FALSE;
 	    R_EvalDepth = 0;
+	    resetTimeLimits();
 	    count++;
 	    PROTECT(R_CurrentExpr);
 	    R_CurrentExpr = eval(R_CurrentExpr, rho);
@@ -109,9 +106,10 @@ static void R_ReplFile(FILE *fp, SEXP rho, int savestack, int browselevel)
 		PrintWarnings();
 	    break;
 	case PARSE_ERROR:
-	    parseError(R_NilValue, count);
+	    parseError(R_NilValue, R_ParseError);
 	    break;
 	case PARSE_EOF:
+	    R_FinalizeSrcRefState(&ParseState);
 	    return;
 	    break;
 	case PARSE_INCOMPLETE:
@@ -214,7 +212,7 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
 		return(-1);
 	    state->bufp = state->buf;
     }
-#ifdef SHELL_ESCAPE
+#ifdef SHELL_ESCAPE /* not default */
     if (*state->bufp == '!') {
 #ifdef HAVE_SYSTEM
 	    R_system(&(state->buf[1]));
@@ -231,13 +229,13 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
     }
 
     R_PPStackTop = savestack;
-    R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 0, &state->status);
-
+    R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 0, &state->status, NULL);
+    
     switch(state->status) {
 
     case PARSE_NULL:
 
-	/* The intention here is to break on CR but not on other 
+	/* The intention here is to break on CR but not on other
 	   null statements: see PR#9063 */
 	if (browselevel && !strcmp((char *) state->buf, "\n")) return -1;
 	R_IoBufferWriteReset(&R_ConsoleIob);
@@ -247,7 +245,7 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
     case PARSE_OK:
 
 	R_IoBufferReadReset(&R_ConsoleIob);
-	R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 1, &state->status);
+	R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 1, &state->status, NULL);
 	if (browselevel) {
 	    browsevalue = ParseBrowser(R_CurrentExpr, rho);
 	    if(browsevalue == 1) return(-1);
@@ -258,6 +256,7 @@ Rf_ReplIteration(SEXP rho, int savestack, int browselevel, R_ReplState *state)
 	}
 	R_Visible = FALSE;
 	R_EvalDepth = 0;
+	resetTimeLimits();
 	PROTECT(R_CurrentExpr);
 	R_Busy(1);
 	value = eval(R_CurrentExpr, rho);
@@ -303,7 +302,7 @@ static void R_ReplConsole(SEXP rho, int savestack, int browselevel)
 
     R_IoBufferWriteReset(&R_ConsoleIob);
     state.buf[0] = '\0';
-    state.buf[CONSOLE_BUFFER_SIZE] = '\0'; 
+    state.buf[CONSOLE_BUFFER_SIZE] = '\0';
     /* stopgap measure if line > CONSOLE_BUFFER_SIZE chars */
     state.bufp = state.buf;
     if(R_Verbose)
@@ -311,16 +310,17 @@ static void R_ReplConsole(SEXP rho, int savestack, int browselevel)
     for(;;) {
 	status = Rf_ReplIteration(rho, savestack, browselevel, &state);
 	if(status < 0)
-  	  return;
+	  return;
     }
 }
 
 
 static unsigned char DLLbuf[CONSOLE_BUFFER_SIZE+1], *DLLbufp;
 
-void R_ReplDLLinit()
+void R_ReplDLLinit(void)
 {
     R_IoBufferInit(&R_ConsoleIob);
+    SETJMP(R_Toplevel.cjmpbuf);
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
     R_IoBufferWriteReset(&R_ConsoleIob);
     prompt_type = 1;
@@ -329,7 +329,7 @@ void R_ReplDLLinit()
 }
 
 
-int R_ReplDLLdo1()
+int R_ReplDLLdo1(void)
 {
     int c;
     ParseStatus status;
@@ -347,7 +347,7 @@ int R_ReplDLLdo1()
 	if(c == ';' || c == '\n') break;
     }
     R_PPStackTop = 0;
-    R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 0, &status);
+    R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 0, &status, NULL);
 
     switch(status) {
     case PARSE_NULL:
@@ -356,9 +356,10 @@ int R_ReplDLLdo1()
 	break;
     case PARSE_OK:
 	R_IoBufferReadReset(&R_ConsoleIob);
-	R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 1, &status);
+	R_CurrentExpr = R_Parse1Buffer(&R_ConsoleIob, 1, &status, NULL);
 	R_Visible = FALSE;
 	R_EvalDepth = 0;
+	resetTimeLimits();
 	PROTECT(R_CurrentExpr);
 	R_Busy(1);
 	R_CurrentExpr = eval(R_CurrentExpr, rho);
@@ -403,7 +404,7 @@ static RETSIGTYPE handleInterrupt(int dummy)
 static RETSIGTYPE handlePipe(int dummy)
 {
     signal(SIGPIPE, handlePipe);
-    error("ignoring SIGPIPE signal");    
+    error("ignoring SIGPIPE signal");
 }
 #endif
 
@@ -432,7 +433,7 @@ static void win32_segv(int signum)
     }
     num_caught++;
     if(num_caught < 10) signal(signum, win32_segv);
-    if(signum == SIGILL)  
+    if(signum == SIGILL)
 	error("caught access violation - continue with care");
     else
 	error("caught access violation - continue with care");
@@ -448,7 +449,7 @@ static void win32_segv(int signum)
    2005-12-17 BDR */
 
 static unsigned char ConsoleBuf[CONSOLE_BUFFER_SIZE];
-extern void R_CleanTempDir();
+extern void R_CleanTempDir(void);
 
 static void sigactionSegv(int signum, siginfo_t *ip, void *context)
 {
@@ -457,7 +458,7 @@ static void sigactionSegv(int signum, siginfo_t *ip, void *context)
     /* First check for stack overflow if we know the stack position.
        We assume anything within 16Mb beyond the stack end is a stack overflow.
      */
-    if(signum == SIGSEGV && (ip != (siginfo_t *)0) && 
+    if(signum == SIGSEGV && (ip != (siginfo_t *)0) &&
        (intptr_t) R_CStackStart != -1) {
 	uintptr_t addr = (uintptr_t) ip->si_addr;
 	intptr_t diff = (R_CStackDir > 0) ? R_CStackStart - addr:
@@ -466,20 +467,20 @@ static void sigactionSegv(int signum, siginfo_t *ip, void *context)
 	if((intptr_t) R_CStackLimit != -1) upper += R_CStackLimit;
 	if(diff > 0 && diff < upper) {
 	    REprintf(_("Error: segfault from C stack overflow\n"));
-	    jump_to_toplevel();	
-	}    
+	    jump_to_toplevel();
+	}
     }
 
     /* need to take off stack checking as stack base has changed */
     R_CStackLimit = (uintptr_t)-1;
 
     /* Do not translate these messages */
-    REprintf("\n *** caught %s ***\n", 
-	     signum == SIGILL ? "illegal operation" : 
+    REprintf("\n *** caught %s ***\n",
+	     signum == SIGILL ? "illegal operation" :
 	     signum == SIGBUS ? "bus error" : "segfault");
     if(ip != (siginfo_t *)0) {
 	if(signum == SIGILL) {
-	    
+
 	    switch(ip->si_code) {
 #ifdef ILL_ILLOPC
 	    case ILL_ILLOPC:
@@ -566,13 +567,13 @@ static void sigactionSegv(int signum, siginfo_t *ip, void *context)
 	}
     }
     if(R_Interactive) {
-	REprintf("\nPossible actions:\n1: %s\n2: %s\n3: %s\n4: %s\n", 
-		 "abort (with core dump, if enabled)", 
-		 "normal R exit", 
+	REprintf("\nPossible actions:\n1: %s\n2: %s\n3: %s\n4: %s\n",
+		 "abort (with core dump, if enabled)",
+		 "normal R exit",
 		 "exit R without saving workspace",
 		 "exit R saving workspace");
 	while(1) {
-	    if(R_ReadConsole("Selection: ", ConsoleBuf, CONSOLE_BUFFER_SIZE, 
+	    if(R_ReadConsole("Selection: ", ConsoleBuf, CONSOLE_BUFFER_SIZE,
 			     0) > 0) {
 		if(ConsoleBuf[0] == '1') break;
 		if(ConsoleBuf[0] == '2') R_CleanUp(SA_DEFAULT, 0, 1);
@@ -600,16 +601,16 @@ static struct sigaltstack sigstk;
 static void *signal_stack;
 
 #define R_USAGE 100000 /* Just a guess */
-static void init_signal_handlers()
+static void init_signal_handlers(void)
 {
     /* <FIXME> may need to reinstall this if we do recover. */
     struct sigaction sa;
     signal_stack = malloc(SIGSTKSZ + R_USAGE);
     if (signal_stack != NULL) {
-        sigstk.ss_sp = signal_stack;
-        sigstk.ss_size = SIGSTKSZ + R_USAGE;
-        sigstk.ss_flags = 0;
-        if(sigaltstack(&sigstk, NULL) < 0)
+	sigstk.ss_sp = signal_stack;
+	sigstk.ss_size = SIGSTKSZ + R_USAGE;
+	sigstk.ss_flags = 0;
+	if(sigaltstack(&sigstk, NULL) < 0)
 	    warning("failed to set alternate signal stack");
     } else
 	warning("failed to allocate alternate signal stack");
@@ -629,7 +630,7 @@ static void init_signal_handlers()
 }
 
 #else /* not sigaltstack and sigaction and sigemptyset*/
-static void init_signal_handlers()
+static void init_signal_handlers(void)
 {
     signal(SIGINT,  handleInterrupt);
     signal(SIGUSR1, onsigusr1);
@@ -650,7 +651,7 @@ static void R_LoadProfile(FILE *fparg, SEXP env)
     if (fp != NULL) {
 	if (! SETJMP(R_Toplevel.cjmpbuf)) {
 	    R_GlobalContext = R_ToplevelContext = &R_Toplevel;
-	    R_ReplFile(fp, env, 0, 0);
+	    R_ReplFile(fp, env);
 	}
 	fclose(fp);
     }
@@ -672,10 +673,8 @@ void setup_Rmainloop(void)
 #ifdef ENABLE_NLS
     char localedir[PATH_MAX+20];
 #endif
-#ifdef Win32
-    char deferred_warnings[4][250];
+    char deferred_warnings[6][250];
     int ndeferred_warnings = 0;
-#endif
 
     InitConnections(); /* needed to get any output at all */
 
@@ -688,7 +687,7 @@ void setup_Rmainloop(void)
 	p = getenv("LC_ALL");
 	strncpy(Rlocale, p ? p : "", 1000);
 	if(!(p = getenv("LC_CTYPE"))) p = Rlocale;
-	/* We'd like to use warning, but need to defer. 
+	/* We'd like to use warning, but need to defer.
 	   Also cannot translate. */
 	if(!setlocale(LC_CTYPE, p))
 	    snprintf(deferred_warnings[ndeferred_warnings++], 250,
@@ -711,19 +710,30 @@ void setup_Rmainloop(void)
 	/* Windows does not have LC_MESSAGES */
     }
 #else /* not Win32 */
-    setlocale(LC_CTYPE, "");/*- make ISO-latin1 etc. work for LOCALE users */
-    setlocale(LC_COLLATE, "");/*- alphabetically sorting */
-    setlocale(LC_TIME, "");/*- names and defaults for date-time formats */
-    setlocale(LC_MONETARY, "");/*- currency units */
+    if(!setlocale(LC_CTYPE, ""))
+	snprintf(deferred_warnings[ndeferred_warnings++], 250,
+		 "Setting LC_CTYPE failed, using \"C\"\n");
+    if(!setlocale(LC_COLLATE, ""))
+	snprintf(deferred_warnings[ndeferred_warnings++], 250,
+		 "Setting LC_COLLATE failed, using \"C\"\n");
+    if(!setlocale(LC_TIME, ""))
+	snprintf(deferred_warnings[ndeferred_warnings++], 250,
+		 "Setting LC_TIME failed, using \"C\"\n");
 #ifdef ENABLE_NLS
-    setlocale(LC_MESSAGES,""); /* language for messages */
+    if(!setlocale(LC_MESSAGES, ""))
+	snprintf(deferred_warnings[ndeferred_warnings++], 250,
+		 "Setting LC_MESSAGES failed, using \"C\"\n");
 #endif
     /* NB: we do not set LC_NUMERIC */
 #ifdef LC_PAPER
-    setlocale(LC_PAPER,"");
+    if(!setlocale(LC_PAPER, ""))
+	snprintf(deferred_warnings[ndeferred_warnings++], 250,
+		 "Setting LC_PAPER failed, using \"C\"\n");
 #endif
 #ifdef LC_MEASUREMENT
-    setlocale(LC_MEASUREMENT,"");
+    if(!setlocale(LC_MEASUREMENT, ""))
+	snprintf(deferred_warnings[ndeferred_warnings++], 250,
+		 "Setting LC_MEASUREMENT failed, using \"C\"\n");
 #endif
 #endif /* not Win32 */
 #ifdef ENABLE_NLS
@@ -758,29 +768,7 @@ void setup_Rmainloop(void)
     InitColors();
     InitGraphics();
     R_Is_Running = 1;
-#ifdef HAVE_LANGINFO_CODESET
-    {
-	char  *p = nl_langinfo(CODESET);
-	if(streql(p, "UTF-8")) known_to_be_utf8 = utf8locale =TRUE;
-	if(streql(p, "ISO-8859-1")) known_to_be_latin1 = latin1locale =TRUE;
-	/* fprintf(stderr, "using %s\n", p); */
-    }
-#endif
-#ifdef SUPPORT_MBCS
-    mbcslocale = MB_CUR_MAX > 1;
-#endif
-#ifdef Win32
-    {
-	char *ctype = setlocale(LC_CTYPE, NULL), *p;
-	p = strrchr(ctype, '.');
-	if(p && isdigit(p[1])) localeCP = atoi(p+1); else localeCP = 0;
-	/* Not 100% correct, but CP1252 is a superset */
-	known_to_be_latin1 = latin1locale = (localeCP == 1252);
-    }
-#endif
-#if defined(Win32) && defined(SUPPORT_UTF8)
-    utf8locale = mbcslocale = TRUE;
-#endif
+    R_check_locale();
     /* gc_inhibit_torture = 0; */
 
     /* Initialize the global context for error handling. */
@@ -807,6 +795,7 @@ void setup_Rmainloop(void)
     R_Toplevel.intsusp = FALSE;
     R_Toplevel.handlerstack = R_HandlerStack;
     R_Toplevel.restartstack = R_RestartStack;
+    R_Toplevel.srcref = R_NilValue;
     R_GlobalContext = R_ToplevelContext = &R_Toplevel;
 
     R_Warnings = R_NilValue;
@@ -834,7 +823,7 @@ void setup_Rmainloop(void)
     if (R_SignalHandlers) init_signal_handlers();
     if (!doneit) {
 	doneit = 1;
-	R_ReplFile(fp, baseEnv, 0, 0);
+	R_ReplFile(fp, baseEnv);
     }
     fclose(fp);
 
@@ -851,7 +840,7 @@ void setup_Rmainloop(void)
     R_LockEnvironment(R_BaseEnv, TRUE);
 #endif
     /* At least temporarily unlock some bindings used in graphics */
-    R_unLockBinding(install(".Device"), R_BaseEnv);
+    R_unLockBinding(R_DeviceSymbol, R_BaseEnv);
     R_unLockBinding(install(".Devices"), R_BaseEnv);
     R_unLockBinding(install(".Library.site"), R_BaseEnv);
 
@@ -865,9 +854,9 @@ void setup_Rmainloop(void)
 	R_CurrentExpr = findVar(cmd, R_GlobalEnv);
 	if (R_CurrentExpr != R_UnboundValue &&
 	    TYPEOF(R_CurrentExpr) == CLOSXP) {
-	        PROTECT(R_CurrentExpr = lang1(cmd));
-	        R_CurrentExpr = eval(R_CurrentExpr, R_GlobalEnv);
-	        UNPROTECT(1);
+		PROTECT(R_CurrentExpr = lang1(cmd));
+		R_CurrentExpr = eval(R_CurrentExpr, R_GlobalEnv);
+		UNPROTECT(1);
 	}
 	UNPROTECT(1);
     }
@@ -882,14 +871,8 @@ void setup_Rmainloop(void)
     /* Print a platform and version dependent greeting and a pointer to
      * the copyleft.
      */
-    if(!R_Quiet) {
-	PrintGreeting();
-#ifndef SUPPORT_UTF8
-	if(utf8locale)
-	    R_ShowMessage(_("WARNING: UTF-8 locales are not supported in this build of R\n"));
-#endif
-    }
-
+    if(!R_Quiet) PrintGreeting();
+ 
     R_LoadProfile(R_OpenSiteFile(), baseEnv);
     R_LockBinding(install(".Library.site"), R_BaseEnv);
     R_LoadProfile(R_OpenInitFile(), R_GlobalEnv);
@@ -908,7 +891,7 @@ void setup_Rmainloop(void)
 	R_InitialData();
     }
     else
-    	R_Suicide(_("unable to restore saved data in .RData\n"));
+	R_Suicide(_("unable to restore saved data in .RData\n"));
 
     /* Initial Loading is done.
        At this point we try to invoke the .First Function.
@@ -923,9 +906,9 @@ void setup_Rmainloop(void)
 	R_CurrentExpr = findVar(cmd, R_GlobalEnv);
 	if (R_CurrentExpr != R_UnboundValue &&
 	    TYPEOF(R_CurrentExpr) == CLOSXP) {
-	        PROTECT(R_CurrentExpr = lang1(cmd));
-	        R_CurrentExpr = eval(R_CurrentExpr, R_GlobalEnv);
-	        UNPROTECT(1);
+		PROTECT(R_CurrentExpr = lang1(cmd));
+		R_CurrentExpr = eval(R_CurrentExpr, R_GlobalEnv);
+		UNPROTECT(1);
 	}
 	UNPROTECT(1);
     }
@@ -941,20 +924,18 @@ void setup_Rmainloop(void)
 	R_CurrentExpr = findVar(cmd, baseEnv);
 	if (R_CurrentExpr != R_UnboundValue &&
 	    TYPEOF(R_CurrentExpr) == CLOSXP) {
-	        PROTECT(R_CurrentExpr = lang1(cmd));
-	        R_CurrentExpr = eval(R_CurrentExpr, R_GlobalEnv);
-	        UNPROTECT(1);
+		PROTECT(R_CurrentExpr = lang1(cmd));
+		R_CurrentExpr = eval(R_CurrentExpr, R_GlobalEnv);
+		UNPROTECT(1);
 	}
 	UNPROTECT(1);
     }
     /* gc_inhibit_torture = 0; */
-#ifdef Win32
     {
 	int i;
 	for(i = 0 ; i < ndeferred_warnings; i++)
 	    warning(deferred_warnings[i]);
     }
-#endif
     if (R_CollectWarnings) {
 	REprintf(_("During startup - "));
 	PrintWarnings();
@@ -967,7 +948,7 @@ void end_Rmainloop(void)
 {
     /* refrain from printing trailing '\n' in slave mode */
     if (!R_Slave)
-        Rprintf("\n");
+	Rprintf("\n");
     /* run the .Last function. If it gives an error, will drop back to main
        loop. */
     R_CleanUp(SA_DEFAULT, 0, 1);
@@ -1001,62 +982,134 @@ static void printwhere(void)
   for (cptr = R_GlobalContext; cptr; cptr = cptr->nextcontext) {
     if ((cptr->callflag & (CTXT_FUNCTION | CTXT_BUILTIN)) &&
 	(TYPEOF(cptr->call) == LANGSXP)) {
-	Rprintf("where %d: ", lct++);
+	Rprintf("where %d", lct++);
+	SrcrefPrompt("", cptr->srcref);
 	PrintValue(cptr->call);
     }
   }
   Rprintf("\n");
 }
 
-
 static int ParseBrowser(SEXP CExpr, SEXP rho)
 {
     int rval = 0;
     if (isSymbol(CExpr)) {
 	const char *expr = CHAR(PRINTNAME(CExpr));
 	if (!strcmp(expr, "n")) {
-	    SET_DEBUG(rho, 1);
+	    SET_RDEBUG(rho, 1);
 	    rval = 1;
 	}
 	if (!strcmp(expr, "c")) {
 	    rval = 1;
-	    SET_DEBUG(rho, 0);
+	    SET_RDEBUG(rho, 0);
 	}
 	if (!strcmp(expr, "cont")) {
 	    rval = 1;
-	    SET_DEBUG(rho, 0);
+	    SET_RDEBUG(rho, 0);
 	}
 	if (!strcmp(expr, "Q")) {
 
 	    /* Run onexit/cend code for everything above the target.
-               The browser context is still on the stack, so any error
-               will drop us back to the current browser.  Not clear
-               this is a good thing.  Also not clear this should still
-               be here now that jump_to_toplevel is used for the
-               jump. */
+	       The browser context is still on the stack, so any error
+	       will drop us back to the current browser.  Not clear
+	       this is a good thing.  Also not clear this should still
+	       be here now that jump_to_toplevel is used for the
+	       jump. */
 	    R_run_onexits(R_ToplevelContext);
 
 	    /* this is really dynamic state that should be managed as such */
-	    R_BrowseLevel = 0;
-	    SET_DEBUG(rho, 0); /*PR#1721*/
+	    SET_RDEBUG(rho, 0); /*PR#1721*/
 
 	    jump_to_toplevel();
 	}
 	if (!strcmp(expr, "where")) {
 	    printwhere();
-	    /* SET_DEBUG(rho, 1); */
+	    /* SET_RDEBUG(rho, 1); */
 	    rval = 2;
 	}
     }
     return rval;
 }
 
-/* registering this as a cend exit procedure makes sure R_BrowseLevel
-   is maintained across LONGJMP's */
-static void browser_cend(void *data)
+/* 
+   since users are unlikely to get this right we need to try
+   to do name/position matching - this is a bit rough
+*/
+static SEXP matchargs(SEXP args)
 {
-    int *psaved = (int *) data;
-    R_BrowseLevel = *psaved - 1;
+    int i, nargs = length(args), mt = 0, mc = 0, me = 0, nmatch = 0, pos[3];
+    SEXP tmp, tsym, csym, esym, argList;
+
+    /* set up argList and defaults */
+    PROTECT(argList = allocList(3));
+    PROTECT(tmp = allocVector(STRSXP, 3));
+    SET_STRING_ELT(tmp, 0, mkChar("text"));
+    SET_STRING_ELT(tmp, 0, mkChar("condition"));
+    SET_STRING_ELT(tmp, 0, mkChar("expr"));
+    setAttrib(argList, R_NamesSymbol, tmp);
+    UNPROTECT(1);
+
+    /* set default values */
+
+    SETCAR(argList, mkString(""));
+    SETCADR(argList, R_NilValue);
+    PROTECT(tmp = allocVector(LGLSXP, 1));
+    LOGICAL(tmp)[0] = 1; /*true*/
+    SETCADDR(argList, tmp);
+    UNPROTECT(1);
+
+    /* now match  */
+    if( nargs == 0 ) {
+	UNPROTECT(1);
+	return(argList);
+    }
+
+    /* we have at least one arg */
+    tsym = install("text"); csym = install("condition"); esym = install("expr");
+    tmp = args;
+
+    for(i = 0; i < nargs; i++) { 
+	pos[i] = 0;
+	if(TAG(tmp) == tsym) {
+	    if( mt == 0 ) {
+		nmatch++; pos[i] = 1; mt = 1; 
+		SETCAR(argList, CAR(tmp));
+	    } else error(_("duplicate '%s' argument"), "text");
+	}
+	if(TAG(tmp) == csym) {
+	    if( mc == 0 ) {
+		nmatch++; pos[i] = 1; mc = 1;
+		SETCADR(argList, CAR(tmp));
+	    } else error(_("duplicate '%s' argument"), "condition");
+	}
+	if(TAG(tmp) == esym) {
+	    if( me == 0 ) {
+		nmatch++; pos[i] = 1; me = 1;
+		SETCADDR(argList, CAR(tmp));
+	    } else error(_("duplicate '%s' argument"), "expr");
+	    tmp = CDR(tmp);
+	}
+    }
+    if (nmatch == nargs) {
+	UNPROTECT(1);
+	return(argList);
+    }
+    /* otherwise match by position */
+    /* reset tmp */
+    tmp = args;
+    for(i = 0; i < 3; i++) {
+	if( pos[i] == 0 ) {
+	    if( mt == 0 ) /* first non-named is text */
+		SETCAR(argList, tmp);
+	    else if(mc == 0)  /* second is condition */
+		SETCADR(argList, tmp);
+	    else /* third is expr */
+		SETCADDR(argList, tmp);
+	    nmatch++;
+	}
+    }
+    UNPROTECT(1);
+    return(argList);
 }
 
 SEXP attribute_hidden do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
@@ -1064,26 +1117,39 @@ SEXP attribute_hidden do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
     RCNTXT *saveToplevelContext;
     RCNTXT *saveGlobalContext;
     RCNTXT thiscontext, returncontext, *cptr;
-    int savestack, savebrowselevel, tmp;
-    SEXP topExp;
+    int savestack, browselevel, tmp;
+    SEXP topExp, argList;
+
+    /* argument matching */
+    PROTECT(argList = matchargs(args)); 
+
+    /* return if the expr is not TRUE */
+    if( !asLogical(CADDR(argList)) ) {
+        UNPROTECT(1);
+        return R_NilValue;
+    }
 
     /* Save the evaluator state information */
     /* so that it can be restored on exit. */
 
-    savebrowselevel = R_BrowseLevel + 1;
+    browselevel = countContexts(CTXT_BROWSER, 1);
     savestack = R_PPStackTop;
     PROTECT(topExp = R_CurrentExpr);
     saveToplevelContext = R_ToplevelContext;
     saveGlobalContext = R_GlobalContext;
 
-    if (!DEBUG(rho)) {
+    if (!RDEBUG(rho)) {
 	cptr = R_GlobalContext;
 	while ( !(cptr->callflag & CTXT_FUNCTION) && cptr->callflag )
 	    cptr = cptr->nextcontext;
 	Rprintf("Called from: ");
 	tmp = asInteger(GetOption(install("deparse.max.lines"), R_BaseEnv));
 	if(tmp != NA_INTEGER && tmp > 0) R_BrowseLines = tmp;
-	PrintValueRec(cptr->call,rho);
+        if( cptr != R_ToplevelContext )
+	    PrintValueRec(cptr->call,rho);
+        else
+            Rprintf("top level \n");
+
 	R_BrowseLines = 0;
     }
 
@@ -1096,9 +1162,7 @@ SEXP attribute_hidden do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
     /* acts as a target for error returns. */
 
     begincontext(&returncontext, CTXT_BROWSER, call, rho,
-		 R_BaseEnv, R_NilValue, R_NilValue);
-    returncontext.cend = browser_cend;
-    returncontext.cenddata = &savebrowselevel;
+		 R_BaseEnv, argList, R_NilValue);
     if (!SETJMP(returncontext.cjmpbuf)) {
 	begincontext(&thiscontext, CTXT_RESTART, R_NilValue, rho,
 		     R_BaseEnv, R_NilValue, R_NilValue);
@@ -1109,8 +1173,7 @@ SEXP attribute_hidden do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
 	}
 	R_GlobalContext = &thiscontext;
 	R_InsertRestartHandlers(&thiscontext, TRUE);
-	R_BrowseLevel = savebrowselevel;
-	R_ReplConsole(rho, savestack, R_BrowseLevel);
+	R_ReplConsole(rho, savestack, browselevel+1);
 	endcontext(&thiscontext);
     }
     endcontext(&returncontext);
@@ -1120,10 +1183,10 @@ SEXP attribute_hidden do_browser(SEXP call, SEXP op, SEXP args, SEXP rho)
     R_CurrentExpr = topExp;
     UNPROTECT(1);
     R_PPStackTop = savestack;
+    UNPROTECT(1);
     R_CurrentExpr = topExp;
     R_ToplevelContext = saveToplevelContext;
     R_GlobalContext = saveGlobalContext;
-    R_BrowseLevel--;
     return R_ReturnedValue;
 }
 
@@ -1159,7 +1222,8 @@ SEXP attribute_hidden do_quit(SEXP call, SEXP op, SEXP args, SEXP rho)
     SA_TYPE ask=SA_DEFAULT;
     int status, runLast;
 
-    if(R_BrowseLevel) {
+    /* if there are any browser contexts active don't quit */
+    if(countContexts(CTXT_BROWSER, 1)) {
 	warning(_("cannot quit from browser"));
 	return R_NilValue;
     }
@@ -1180,12 +1244,12 @@ SEXP attribute_hidden do_quit(SEXP call, SEXP op, SEXP args, SEXP rho)
 	errorcall(call, _("unrecognized value of 'save'"));
     status = asInteger(CADR(args));
     if (status == NA_INTEGER) {
-        warning(_("invalid 'status', 0 assumed"));
+	warning(_("invalid 'status', 0 assumed"));
 	runLast = 0;
     }
     runLast = asLogical(CADDR(args));
     if (runLast == NA_LOGICAL) {
-        warning(_("invalid 'runLast', FALSE assumed"));
+	warning(_("invalid 'runLast', FALSE assumed"));
 	runLast = 0;
     }
     /* run the .Last function. If it gives an error, will drop back to main
@@ -1209,7 +1273,7 @@ static R_ToplevelCallbackEl *Rf_ToplevelTaskHandlers = NULL;
   position).
  */
 R_ToplevelCallbackEl *
-Rf_addTaskCallback(R_ToplevelCallback cb, void *data, 
+Rf_addTaskCallback(R_ToplevelCallback cb, void *data,
 		   void (*finalizer)(void *), const char *name, int *pos)
 {
     int which;
@@ -1228,19 +1292,19 @@ Rf_addTaskCallback(R_ToplevelCallback cb, void *data,
 	which = 0;
     } else {
 	R_ToplevelCallbackEl *tmp;
-        tmp = Rf_ToplevelTaskHandlers;
+	tmp = Rf_ToplevelTaskHandlers;
 	which = 1;
 	while(tmp->next) {
 	    which++;
 	    tmp = tmp->next;
 	}
-        tmp->next = el;
+	tmp->next = el;
     }
 
     if(!name) {
-        char buf[5];
+	char buf[5];
 	sprintf(buf, "%d", which+1);
-        el->name = strdup(buf);
+	el->name = strdup(buf);
     } else
 	el->name = strdup(name);
 
@@ -1261,7 +1325,7 @@ Rf_removeTaskCallbackByName(const char *name)
     }
 
     while(el) {
-        if(strcmp(el->name, name) == 0) {
+	if(strcmp(el->name, name) == 0) {
 	    if(prev == NULL) {
 		Rf_ToplevelTaskHandlers = el->next;
 	    } else {
@@ -1350,7 +1414,7 @@ R_removeTaskCallback(SEXP which)
 }
 
 SEXP
-R_getTaskCallbackNames()
+R_getTaskCallbackNames(void)
 {
     SEXP ans;
     R_ToplevelCallbackEl *el;
@@ -1388,7 +1452,7 @@ R_getTaskCallbackNames()
 static Rboolean Rf_RunningToplevelHandlers = FALSE;
 
 void
-Rf_callToplevelHandlers(SEXP expr, SEXP value, Rboolean succeeded, 
+Rf_callToplevelHandlers(SEXP expr, SEXP value, Rboolean succeeded,
 			Rboolean visible)
 {
     R_ToplevelCallbackEl *h, *prev = NULL;
@@ -1402,11 +1466,11 @@ Rf_callToplevelHandlers(SEXP expr, SEXP value, Rboolean succeeded,
     while(h) {
 	again = (h->cb)(expr, value, succeeded, visible, h->data);
 	if(R_CollectWarnings) {
-	    REprintf(_("warning messages from top-level task callback '%s'\n"), 
+	    REprintf(_("warning messages from top-level task callback '%s'\n"),
 		     h->name);
 	    PrintWarnings();
 	}
-        if(again) {
+	if(again) {
 	    prev = h;
 	    h = h->next;
 	} else {
@@ -1417,9 +1481,9 @@ Rf_callToplevelHandlers(SEXP expr, SEXP value, Rboolean succeeded,
 	    h = h->next;
 	    if(tmp == Rf_ToplevelTaskHandlers)
 		Rf_ToplevelTaskHandlers = h;
-            if(tmp->finalizer)
+	    if(tmp->finalizer)
 		tmp->finalizer(tmp->data);
-            free(tmp);
+	    free(tmp);
 	}
     }
 
@@ -1434,16 +1498,14 @@ R_taskCallbackRoutine(SEXP expr, SEXP value, Rboolean succeeded,
     SEXP f = (SEXP) userData;
     SEXP e, tmp, val, cur;
     int errorOccurred;
-    Rboolean again;
-    Rboolean useData;
-    useData = LOGICAL(VECTOR_ELT(f, 2))[0];
+    Rboolean again, useData = LOGICAL(VECTOR_ELT(f, 2))[0];
 
     PROTECT(e = allocVector(LANGSXP, 5 + useData));
     SETCAR(e, VECTOR_ELT(f, 0));
     cur = CDR(e);
     SETCAR(cur, tmp = allocVector(LANGSXP, 2));
-        SETCAR(tmp, install("quote"));
-        SETCAR(CDR(tmp), expr);
+	SETCAR(tmp, R_QuoteSymbol);
+	SETCAR(CDR(tmp), expr);
     cur = CDR(cur);
     SETCAR(cur, value);
     cur = CDR(cur);
@@ -1459,13 +1521,13 @@ R_taskCallbackRoutine(SEXP expr, SEXP value, Rboolean succeeded,
     if(!errorOccurred) {
 	PROTECT(val);
 	if(TYPEOF(val) != LGLSXP) {
-              /* It would be nice to identify the function. */
+	      /* It would be nice to identify the function. */
 	    warning(_("top-level task callback did not return a logical value"));
 	}
 	again = asLogical(val);
 	UNPROTECT(1);
     } else {
-        /* warning("error occurred in top-level task callback\n"); */
+	/* warning("error occurred in top-level task callback\n"); */
 	again = FALSE;
     }
     return(again);
@@ -1490,15 +1552,15 @@ R_addTaskCallback(SEXP f, SEXP data, SEXP useData, SEXP name)
 
     PROTECT(index = allocVector(INTSXP, 1));
     el = Rf_addTaskCallback(R_taskCallbackRoutine,  internalData,
-			    (void (*)(void*)) R_ReleaseObject, tmpName, 
+			    (void (*)(void*)) R_ReleaseObject, tmpName,
 			    INTEGER(index));
 
     if(length(name) == 0) {
 	PROTECT(name = mkString(el->name));
-        setAttrib(index, R_NamesSymbol, name);
+	setAttrib(index, R_NamesSymbol, name);
 	UNPROTECT(1);
     } else {
-        setAttrib(index, R_NamesSymbol, name);
+	setAttrib(index, R_NamesSymbol, name);
     }
 
     UNPROTECT(1);
@@ -1506,3 +1568,14 @@ R_addTaskCallback(SEXP f, SEXP data, SEXP useData, SEXP name)
 }
 
 #undef __MAIN__
+
+#ifndef Win32
+/* this is here solely to pull in xxxpr.o */
+#include <R_ext/RS.h>
+void F77_SYMBOL(intpr) (const char *, int *, int *, int *);
+void attribute_hidden dummy12345(void)
+{
+    int i = 0;
+    F77_CALL(intpr)("dummy", &i, &i, &i);
+}
+#endif
