@@ -18,10 +18,6 @@
  *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is handled as a whole.
-   Collation is locale-specific if strcoll exists and works.
- */
-
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -43,7 +39,7 @@ SEXP attribute_hidden do_relop(SEXP call, SEXP op, SEXP args, SEXP env)
 
     if (DispatchGroup("Ops", call, op, args, env, &ans))
 	return ans;
-
+    checkArity(op, args);
     return do_relop_dflt(call, op, CAR(args), CADR(args));
 }
 
@@ -74,7 +70,7 @@ SEXP attribute_hidden do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
 	return ans;
     }
 
-    /* That symbols and calls were allowwed was undocumented prior to
+    /* That symbols and calls were allowed was undocumented prior to
        R 2.5.0.  We deparse them as deparse() would, minus attributes */
     if ((iS = isSymbol(x)) || TYPEOF(x) == LANGSXP) {
 	SEXP tmp = allocVector(STRSXP, 1);
