@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2004  Robert Gentleman, Ross Ihaka and the
- *			      R Development Core Team
+ *  Copyright (C) 1997--2009  The R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 /* <UTF8> byte-level access is only to compare with chars <= 0x7F */
@@ -30,6 +29,8 @@
 #include <Rmath.h>
 #include <Fileio.h>
 #include <Rversion.h>
+#include <R_ext/RS.h>           /* for CallocCharBuf, Free */
+#include <errno.h>
 
 /* From time to time changes in R, such as the addition of a new SXP,
  * may require changes in the save file format.  Here are some
@@ -58,7 +59,7 @@
 /* ----- V e r s i o n -- T w o -- S a v e / R e s t o r e ----- */
 
 /* Adapted from Chris Young and Ross Ihaka's Version One by Luke
-   Tierney.  Copyright Assigned to the R Project. 
+   Tierney.  Copyright Assigned to the R Project.
 
    The approach used here uses a single pass over the node tree to be
    serialized.  Sharing of reference objects is preserved, but sharing
@@ -100,13 +101,13 @@
    serialization format--just rewrite ReadItem to pass the place to
    store the CDR it reads. (It's a bit of a pain to do, that is why it
    is being deferred until it is clearly needed.)
-   
+
    CHARSXPs are now handled in a way that preserves both embedded null
    characters and NA_STRING values.
 
    The XDR save format now only uses the in-memory xdr facility for
    converting integers and doubles to a portable format.
-   
+
    The output format packs the type flag and other flags into a single
    integer.  This produces more compact output for code; it has little
    effect on data.
@@ -166,11 +167,11 @@ static SEXP ReadBC(SEXP ref_table, R_inpstream_t stream);
 
 /* The default version used when a stream Init function is called with
    version = 0 */
-static int R_DefaultSerializeVersion = 2;
 
+static const int R_DefaultSerializeVersion = 2;
 
 /*
- * Utility Functions 
+ * Utility Functions
  *
  * An assert function which doesn't crash the program.
  * Something like this might be useful in an R header file
@@ -259,7 +260,7 @@ static void OutComplex(R_outpstream_t stream, Rcomplex c)
     OutReal(stream, c.i);
 }
 
-static void OutByte(R_outpstream_t stream, int i)
+static void OutByte(R_outpstream_t stream, Rbyte i)
 {
     char buf[128];
     switch (stream->type) {
@@ -276,7 +277,7 @@ static void OutByte(R_outpstream_t stream, int i)
     }
 }
 
-static void OutString(R_outpstream_t stream, char *s, int length)
+static void OutString(R_outpstream_t stream, const char *s, int length)
 {
     if (stream->type == R_pstream_ascii_format) {
 	int i;
@@ -294,15 +295,15 @@ static void OutString(R_outpstream_t stream, char *s, int length)
 	    case '\?': sprintf(buf, "\\?");  break;
 	    case '\'': sprintf(buf, "\\'");  break;
 	    case '\"': sprintf(buf, "\\\""); break;
-	    default  : 
+	    default  :
 		/* cannot print char in octal mode -> cast to unsigned
 		   char first */
-		/* actually, since s is signed char and '\?' == 127 
+		/* actually, since s is signed char and '\?' == 127
 		   is handled above, s[i] > 126 can't happen, but
 		   I'm superstitious...  -pd */
 		if (s[i] <= 32 || s[i] > 126)
 		    sprintf(buf, "\\%03o", (unsigned char) s[i]);
-		else 
+		else
 		    sprintf(buf, "%c", s[i]);
 	    }
 	    stream->OutBytes(stream, buf, strlen(buf));
@@ -310,7 +311,7 @@ static void OutString(R_outpstream_t stream, char *s, int length)
 	stream->OutChar(stream, '\n');
     }
     else
-	stream->OutBytes(stream, s, length);
+	stream->OutBytes(stream, (void *)s, length); /* FIXME: is this case right? */
 }
 
 
@@ -335,7 +336,7 @@ static void InWord(R_inpstream_t stream, char * buf, int size)
 	error(_("read error"));
     buf[i] = 0;
 }
-    
+
 static int InInteger(R_inpstream_t stream)
 {
     char word[128];
@@ -400,24 +401,6 @@ static Rcomplex InComplex(R_inpstream_t stream)
     return c;
 }
 
-static int InByte(R_inpstream_t stream)
-{
-    char word[128];
-    Rbyte rb;
-
-    switch (stream->type) {
-    case R_pstream_ascii_format:
-	InWord(stream, word, sizeof(word));
-	return (Rbyte) word[0];
-    case R_pstream_binary_format:
-    case R_pstream_xdr_format:
-	stream->InBytes(stream, &rb, 1);
-	return rb;
-    default:
-	return 0;
-    }
-}
-
 /* These utilities for reading characters with an unget option are
    defined so the code in InString can match the code in
    saveload.c:InStringAscii--that way it is easier to match changes in
@@ -572,7 +555,7 @@ static void InFormat(R_inpstream_t stream)
 
 #define HASHSIZE 1099
 
-#define PTRHASH(obj) (((unsigned long) (obj)) >> 2)
+#define PTRHASH(obj) (((R_size_t) (obj)) >> 2)
 
 #define HASH_TABLE_COUNT(ht) TRUELENGTH(CDR(ht))
 #define SET_HASH_TABLE_COUNT(ht, val) SET_TRUELENGTH(CDR(ht), val)
@@ -638,12 +621,14 @@ static int HashGet(SEXP item, SEXP ht)
 #define BCREPDEF          244
 #define BCREPREF          243
 #endif
+#define EMPTYENV_SXP	  242
+#define BASEENV_SXP	  241
 
 /*
  * Type/Flag Packing and Unpacking
  *
  * To reduce space consumption for serializing code (lots of list
- * structure) the type a9at most 8 bits), several single bit flags,
+ * structure) the type (at most 8 bits), several single bit flags,
  * and the sxpinfo gp field (LEVELS, 16 bits) are packed into a single
  * integer.  The integer is signed, so this shouldn't be pushed too
  * far.  It assumes at least 28 bits, but that should be no problem.
@@ -652,13 +637,22 @@ static int HashGet(SEXP item, SEXP ht)
 #define IS_OBJECT_BIT_MASK (1 << 8)
 #define HAS_ATTR_BIT_MASK (1 << 9)
 #define HAS_TAG_BIT_MASK (1 << 10)
-#define ENCODE_LEVELS(v) (v << 12)
-#define DECODE_LEVELS(v) (v >> 12)
-#define DECODE_TYPE(v) (v & 255)
+#define ENCODE_LEVELS(v) ((v) << 12)
+#define DECODE_LEVELS(v) ((v) >> 12)
+#define DECODE_TYPE(v) ((v) & 255)
 
 static int PackFlags(int type, int levs, int isobj, int hasattr, int hastag)
 {
-    int val = type | ENCODE_LEVELS(levs);
+    /* We don't write out bit 5 as from R 2.8.0.
+       It is used to indicate if an object is in CHARSXP cache
+       - not that it matters to this version of R, but it saves
+       checking all previous versions.
+
+       Also make sure the HASHASH bit is not written out.
+    */
+    int val;
+    if (type == CHARSXP) levs &= (~(CACHED_MASK | HASHASH_MASK));
+    val = type | ENCODE_LEVELS(levs);
     if (isobj) val |= IS_OBJECT_BIT_MASK;
     if (hasattr) val |= HAS_ATTR_BIT_MASK;
     if (hastag) val |= HAS_TAG_BIT_MASK;
@@ -724,6 +718,8 @@ static SEXP GetPersistentName(R_outpstream_t stream, SEXP s)
 	case EXTPTRSXP: break;
 	case ENVSXP:
 	    if (s == R_GlobalEnv ||
+		s == R_BaseEnv ||
+		s == R_EmptyEnv ||
 		R_IsNamespaceEnv(s) ||
 		R_IsPackageEnv(s))
 		return R_NilValue;
@@ -752,6 +748,8 @@ static SEXP PersistentRestore(R_inpstream_t stream, SEXP s)
 static int SaveSpecialHook(SEXP item)
 {
     if (item == R_NilValue)      return NILVALUE_SXP;
+    if (item == R_EmptyEnv)	 return EMPTYENV_SXP;
+    if (item == R_BaseEnv)	 return BASEENV_SXP;
     if (item == R_GlobalEnv)     return GLOBALENV_SXP;
     if (item == R_UnboundValue)  return UNBOUNDVALUE_SXP;
     if (item == R_MissingArg)    return MISSINGARG_SXP;
@@ -777,13 +775,13 @@ static void OutStringVec(R_outpstream_t stream, SEXP s, SEXP ref_table)
     OutInteger(stream, len);
     for (i = 0; i < len; i++)
 	WriteItem(STRING_ELT(s, i), ref_table, stream);
-}	
+}
 
 /* e.g., OutVec(fp, obj, INTEGER, OutInteger) */
-#define OutVec(fp, obj, accessor, outfunc)	 			\
-	do { 								\
+#define OutVec(fp, obj, accessor, outfunc)				\
+	do {								\
 		int cnt;						\
-		for (cnt = 0; cnt < LENGTH(obj); ++cnt) 		\
+		for (cnt = 0; cnt < LENGTH(obj); ++cnt)		\
 			outfunc(fp, accessor(obj, cnt));		\
 	} while (0)
 
@@ -799,6 +797,7 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
     SEXP t;
 
  tailcall:
+    R_CheckStack();
     if ((t = GetPersistentName(stream, s)) != R_NilValue) {
 	R_assert(TYPEOF(t) == STRSXP && LENGTH(t) > 0);
 	PROTECT(t);
@@ -838,12 +837,12 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
 	    OutInteger(stream, R_EnvironmentIsLocked(s) ? 1 : 0);
 	    WriteItem(ENCLOS(s), ref_table, stream);
 	    WriteItem(FRAME(s), ref_table, stream);
-	    WriteItem(TAG(s), ref_table, stream);
+	    WriteItem(HASHTAB(s), ref_table, stream);
 	    WriteItem(ATTRIB(s), ref_table, stream);
 	}
     }
     else {
-	int flags, hastag;
+	int flags, hastag, hasattr;
 	switch(TYPEOF(s)) {
 	case LISTSXP:
 	case LANGSXP:
@@ -852,8 +851,16 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
 	case DOTSXP: hastag = TAG(s) != R_NilValue; break;
 	default: hastag = FALSE;
 	}
+#ifdef USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS
+	/* With the CHARSXP cache chains maintained through the ATTRIB
+	   field the content of that field must not be serialized, so
+	   we treat it as not there. */
+	hasattr = (TYPEOF(s) != CHARSXP && ATTRIB(s) != R_NilValue);
+#else
+	hasattr = ATTRIB(s) != R_NilValue;
+#endif
 	flags = PackFlags(TYPEOF(s), LEVELS(s), OBJECT(s),
-                             ATTRIB(s) != R_NilValue, hastag);
+			  hasattr, hastag);
 	OutInteger(stream, flags);
 	switch (TYPEOF(s)) {
 	case LISTSXP:
@@ -864,7 +871,7 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
 	    /* Dotted pair objects */
 	    /* These write their ATTRIB fields first to allow us to avoid
 	       recursion on the CDR */
-	    if (ATTRIB(s) != R_NilValue)
+	    if (hasattr)
 		WriteItem(ATTRIB(s), ref_table, stream);
 	    if (TAG(s) != R_NilValue)
 		WriteItem(TAG(s), ref_table, stream);
@@ -910,6 +917,10 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
 	    OutVec(stream, s, COMPLEX_ELT, OutComplex);
 	    break;
 	case STRSXP:
+	    OutInteger(stream, LENGTH(s));
+	    for (i = 0; i < LENGTH(s); i++)
+		WriteItem(STRING_ELT(s, i), ref_table, stream);
+	    break;
 	case VECSXP:
 	case EXPRSXP:
 	    OutInteger(stream, LENGTH(s));
@@ -927,16 +938,18 @@ static void WriteItem (SEXP s, SEXP ref_table, R_outpstream_t stream)
 	    OutInteger(stream, LENGTH(s));
 	    OutVec(stream, s, RAW_ELT, OutByte);
 	    break;
+	case S4SXP:
+	  break; /* only attributes (i.e., slots) count */
 	default:
 	    error(_("WriteItem: unknown type %i"), TYPEOF(s));
 	}
-	if (ATTRIB(s) != R_NilValue)
+	if (hasattr)
 	    WriteItem(ATTRIB(s), ref_table, stream);
     }
 }
 
 #ifdef BYTECODE
-static SEXP MakeCircleHashTable()
+static SEXP MakeCircleHashTable(void)
 {
     return CONS(R_NilValue, allocVector(VECSXP, HASHSIZE));
 }
@@ -971,7 +984,7 @@ static void ScanForCircles1(SEXP s, SEXP ct)
     switch (TYPEOF(s)) {
     case LANGSXP:
     case LISTSXP:
-        if (! AddCircleHash(s, ct)) {
+	if (! AddCircleHash(s, ct)) {
 	    ScanForCircles1(CAR(s), ct);
 	    ScanForCircles1(CDR(s), ct);
 	}
@@ -1017,7 +1030,7 @@ static void WriteBCLang(SEXP s, SEXP ref_table, SEXP reps,
 	    /* we have a cell referenced more than once */
 	    if (TAG(r) == R_NilValue) {
 		/* this is the first reference, so update and register
-                   the counter */
+		   the counter */
 		int i = INTEGER(CAR(reps))[0]++;
 		SET_TAG(r, allocVector(INTSXP, 1));
 		INTEGER(TAG(r))[0] = i;
@@ -1079,7 +1092,7 @@ static void WriteBC(SEXP s, SEXP ref_table, R_outpstream_t stream)
     PROTECT(reps = CONS(R_NilValue, reps));
     OutInteger(stream, length(reps));
     SETCAR(reps, allocVector(INTSXP, 1));
-    INTEGER(CAR(reps))[0] = 0; 
+    INTEGER(CAR(reps))[0] = 0;
     WriteBC1(s, ref_table, reps, stream);
     UNPROTECT(1);
 }
@@ -1096,7 +1109,7 @@ void R_Serialize(SEXP s, R_outpstream_t stream)
     case 2:
 	OutInteger(stream, version);
 	OutInteger(stream, R_VERSION);
-	OutInteger(stream, R_Version(1,4,0));
+	OutInteger(stream, R_Version(2,3,0));
 	break;
     default: error(_("version %d not supported"), version);
     }
@@ -1150,7 +1163,7 @@ static void AddReadRef(SEXP table, SEXP value)
     SET_TRUELENGTH(data, count);
     SET_VECTOR_ELT(data, count - 1, value);
 }
-    
+
 static SEXP InStringVec(R_inpstream_t stream, SEXP ref_table)
 {
     SEXP s;
@@ -1164,7 +1177,7 @@ static SEXP InStringVec(R_inpstream_t stream, SEXP ref_table)
     UNPROTECT(1);
     return s;
 }
-    
+
 #define InVec(fp, obj, accessor, infunc, length)			\
 	do {								\
 		int cnt;						\
@@ -1185,6 +1198,7 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
     SEXPTYPE type;
     SEXP s;
     int flags, levs, objf, hasattr, hastag, length, count;
+    char *cbuf;
 
     R_assert(TYPEOF(ref_table) == LISTSXP && TYPEOF(CAR(ref_table)) == VECSXP);
 
@@ -1193,6 +1207,8 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 
     switch(type) {
     case NILVALUE_SXP:      return R_NilValue;
+    case EMPTYENV_SXP:	    return R_EmptyEnv;
+    case BASEENV_SXP:	    return R_BaseEnv;
     case GLOBALENV_SXP:     return R_GlobalEnv;
     case UNBOUNDVALUE_SXP:  return R_UnboundValue;
     case MISSINGARG_SXP:    return R_MissingArg;
@@ -1236,7 +1252,7 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    /* Now fill it in  */
 	    SET_ENCLOS(s, ReadItem(ref_table, stream));
 	    SET_FRAME(s, ReadItem(ref_table, stream));
-	    SET_TAG(s, ReadItem(ref_table, stream));
+	    SET_HASHTAB(s, ReadItem(ref_table, stream));
 	    SET_ATTRIB(s, ReadItem(ref_table, stream));
 	    if (ATTRIB(s) != R_NilValue &&
 		getAttrib(s, R_ClassSymbol) != R_NilValue)
@@ -1245,6 +1261,8 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 		SET_OBJECT(s, 1);
 	    R_RestoreHashCount(s);
 	    if (locked) R_LockEnvironment(s, FALSE);
+	    /* Convert a NULL enclosure to baseenv() */
+	    if (ENCLOS(s) == R_NilValue) SET_ENCLOS(s, R_BaseEnv);
 	    UNPROTECT(1);
 	    return s;
 	}
@@ -1254,13 +1272,13 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
     case PROMSXP:
     case DOTSXP:
 	/* This handling of dotted pair objects still uses recursion
-           on the CDR and so will overflow the PROTECT stack for long
-           lists.  The save format does permit using an iterative
-           approach; it just has to pass around the place to write the
-           CDR into when it is allocated.  It's more trouble than it
-           is worth to write the code to handle this now, but if it
-           becomes necessary we can do it without needing to change
-           the save format. */
+	   on the CDR and so will overflow the PROTECT stack for long
+	   lists.  The save format does permit using an iterative
+	   approach; it just has to pass around the place to write the
+	   CDR into when it is allocated.  It's more trouble than it
+	   is worth to write the code to handle this now, but if it
+	   becomes necessary we can do it without needing to change
+	   the save format. */
 	PROTECT(s = allocSExp(type));
 	SETLEVELS(s, levs);
 	SET_OBJECT(s, objf);
@@ -1268,6 +1286,9 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	SET_TAG(s, hastag ? ReadItem(ref_table, stream) : R_NilValue);
 	SETCAR(s, ReadItem(ref_table, stream));
 	SETCDR(s, ReadItem(ref_table, stream));
+	/* For reading closures and promises stored in earlier versions, convert NULL env to baseenv() */
+	if      (type == CLOSXP && CLOENV(s) == R_NilValue) SET_CLOENV(s, R_BaseEnv);
+	else if (type == PROMSXP && PRENV(s) == R_NilValue) SET_PRENV(s, R_BaseEnv);
 	UNPROTECT(1); /* s */
 	return s;
     default:
@@ -1289,23 +1310,40 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    break;
 	case SPECIALSXP:
 	case BUILTINSXP:
+	    /* These are all short strings */
 	    length = InInteger(stream);
-	    PROTECT(s = allocVector(CHARSXP, length)); /* as buffer */
-	    InString(stream,CHAR(s), length);
-	    s = mkPRIMSXP(StrToInternal(CHAR(s)), type == BUILTINSXP);
-	    UNPROTECT(1);  /* pop the old s off the protect stack */
-	    PROTECT(s);    /* and push the new s on it */
+	    cbuf = alloca(length+1);
+	    InString(stream, cbuf, length);
+	    cbuf[length] = '\0';
+	    PROTECT(s = mkPRIMSXP(StrToInternal(cbuf), type == BUILTINSXP));
 	    break;
 	case CHARSXP:
 	    length = InInteger(stream);
 	    if (length == -1)
 		PROTECT(s = NA_STRING);
-	    else {
-		PROTECT(s = allocVector(CHARSXP, length));
-		InString(stream, CHAR(s), length);
+	    else if (length < 1000) {
+		int enc = CE_NATIVE;
+		cbuf = alloca(length+1);
+		InString(stream, cbuf, length);
+		cbuf[length] = '\0';
+		if (levs & UTF8_MASK) enc = CE_UTF8;
+		else if (levs & LATIN1_MASK) enc = CE_LATIN1;
+		PROTECT(s = mkCharLenCE(cbuf, length, enc));
+	    } else {
+		int enc = CE_NATIVE;
+		cbuf = CallocCharBuf(length);
+		InString(stream, cbuf, length);
+		if (levs & UTF8_MASK) enc = CE_UTF8;
+		else if (levs & LATIN1_MASK) enc = CE_LATIN1;
+		PROTECT(s = mkCharLenCE(cbuf, length, enc));
+		Free(cbuf);
 	    }
 	    break;
 	case LGLSXP:
+	    length = InInteger(stream);
+	    PROTECT(s = allocVector(type, length));
+	    InVec(stream, s, SET_LOGICAL_ELT, InInteger, length);
+	    break;
 	case INTSXP:
 	    length = InInteger(stream);
 	    PROTECT(s = allocVector(type, length));
@@ -1322,6 +1360,11 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	    InVec(stream, s, SET_COMPLEX_ELT, InComplex, length);
 	    break;
 	case STRSXP:
+	    length = InInteger(stream);
+	    PROTECT(s = allocVector(type, length));
+	    for (count = 0; count < length; ++count)
+		SET_STRING_ELT(s, count, ReadItem(ref_table, stream));
+	    break;
 	case VECSXP:
 	case EXPRSXP:
 	    length = InInteger(stream);
@@ -1343,15 +1386,33 @@ static SEXP ReadItem (SEXP ref_table, R_inpstream_t stream)
 	case RAWSXP:
 	    length = InInteger(stream);
 	    PROTECT(s = allocVector(type, length));
-	    InVec(stream, s, SET_RAW_ELT, InByte, length);
+	    stream->InBytes(stream, RAW(s), length);
+	    break;
+	case S4SXP:
+	    PROTECT(s = allocS4Object());
 	    break;
 	default:
 	    s = R_NilValue; /* keep compiler happy */
-	    error(_("ReadItem: unknown type %i"), type);
+	    error(_("ReadItem: unknown type %i, perhaps written by later version of R"), type);
 	}
-	SETLEVELS(s, levs);
+	if (type != CHARSXP)
+	    SETLEVELS(s, levs);
 	SET_OBJECT(s, objf);
+#ifdef USE_ATTRIB_FIELD_FOR_CHARSXP_CACHE_CHAINS
+	if (TYPEOF(s) == CHARSXP) {
+	    /* With the CHARSXP cache maintained through the ATTRIB
+	       field that field has already been filled in by the
+	       mkChar/mkCharCE call above, so we need to leave it
+	       alone.  If there is an attribute (as there might be if
+	       the serialized data was created by an older version) we
+	       read and ignore the value. */
+	    if (hasattr) ReadItem(ref_table, stream);
+	}
+	else
+	    SET_ATTRIB(s, hasattr ? ReadItem(ref_table, stream) : R_NilValue);
+#else
 	SET_ATTRIB(s, hasattr ? ReadItem(ref_table, stream) : R_NilValue);
+#endif
 	UNPROTECT(1); /* s */
 	return s;
     }
@@ -1418,7 +1479,7 @@ static SEXP ReadBCConsts(SEXP ref_table, SEXP reps, R_inpstream_t stream)
     UNPROTECT(1);
     return ans;
 }
-    
+
 static SEXP ReadBC1(SEXP ref_table, SEXP reps, R_inpstream_t stream)
 {
     SEXP s;
@@ -1486,11 +1547,12 @@ SEXP R_Unserialize(R_inpstream_t stream)
 }
 
 
-/* 
+/*
  * Generic Persistent Stream Initializers
  */
 
-void R_InitInPStream(R_inpstream_t stream, R_pstream_data_t data,
+void
+R_InitInPStream(R_inpstream_t stream, R_pstream_data_t data,
 		     R_pstream_format_t type,
 		     int (*inchar)(R_inpstream_t),
 		     void (*inbytes)(R_inpstream_t, void *, int),
@@ -1504,7 +1566,8 @@ void R_InitInPStream(R_inpstream_t stream, R_pstream_data_t data,
     stream->InPersistHookData = pdata;
 }
 
-void R_InitOutPStream(R_outpstream_t stream, R_pstream_data_t data,
+void
+R_InitOutPStream(R_outpstream_t stream, R_pstream_data_t data,
 		      R_pstream_format_t type, int version,
 		      void (*outchar)(R_outpstream_t, int),
 		      void (*outbytes)(R_outpstream_t, void *, int),
@@ -1520,7 +1583,7 @@ void R_InitOutPStream(R_outpstream_t stream, R_pstream_data_t data,
 }
 
 
-/* 
+/*
  * Persistent File Streams
  */
 
@@ -1540,16 +1603,19 @@ static int InCharFile(R_inpstream_t stream)
 static void OutBytesFile(R_outpstream_t stream, void *buf, int length)
 {
     FILE *fp = stream->data;
-    fwrite(buf, 1, length, fp); /**** error message */
+    size_t out = fwrite(buf, 1, length, fp);
+    if (out != length) error(_("write failed"));
 }
 
 static void InBytesFile(R_inpstream_t stream, void *buf, int length)
 {
     FILE *fp = stream->data;
-    fread(buf, 1, length, fp); /**** error message */
+    size_t in = fread(buf, 1, length, fp);
+    if (in != length) error(_("read failed"));
 }
 
-void R_InitFileOutPStream(R_outpstream_t stream, FILE *fp,
+void
+R_InitFileOutPStream(R_outpstream_t stream, FILE *fp,
 			  R_pstream_format_t type, int version,
 			  SEXP (*phook)(SEXP, SEXP), SEXP pdata)
 {
@@ -1557,14 +1623,15 @@ void R_InitFileOutPStream(R_outpstream_t stream, FILE *fp,
 		     OutCharFile, OutBytesFile, phook, pdata);
 }
 
-void R_InitFileInPStream(R_inpstream_t stream, FILE *fp,
+void
+R_InitFileInPStream(R_inpstream_t stream, FILE *fp,
 			 R_pstream_format_t type,
 			 SEXP (*phook)(SEXP, SEXP), SEXP pdata)
 {
     R_InitInPStream(stream, (R_pstream_data_t) fp, type,
 		    InCharFile, InBytesFile, phook, pdata);
 }
- 
+
 
 /*
  * Persistent Connection Streams
@@ -1599,8 +1666,23 @@ static void InBytesConn(R_inpstream_t stream, void *buf, int length)
 	    p[i] = Rconn_fgetc(con);
     }
     else {
-	if (length != con->read(buf, 1, length, con))
-	    error(_("error reading from connection"));
+	if (stream->type == R_pstream_ascii_format) {
+	    char linebuf[4];
+	    unsigned char *p = buf;
+	    int i, ncread;
+	    unsigned int res;
+	    for (i = 0; i < length; i++) {
+		ncread = Rconn_getline(con, linebuf, 3);
+		if (ncread != 2)
+		    error(_("error reading from ascii connection"));
+		if (!sscanf(linebuf, "%02x", &res))
+		    error(_("unexpected format in ascii connection"));
+		*p++ = (unsigned char)res;
+	    }
+	} else {
+	    if (length != con->read(buf, 1, length, con))
+		error(_("error reading from connection"));
+	}
     }
 }
 
@@ -1684,7 +1766,8 @@ static SEXP CallHook(SEXP x, SEXP fun)
     return val;
 }
 
-SEXP do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden
+do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     /* serializeToConn(object, conn, ascii, version, hook) */
 
@@ -1702,7 +1785,7 @@ SEXP do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
     con = getConnection(asInteger(CADR(args)));
 
     if (TYPEOF(CADDR(args)) != LGLSXP)
-	errorcall(call, _("'ascii' must be logical"));
+	error(_("'ascii' must be logical"));
     ascii = INTEGER(CADDR(args))[0];
     if (ascii) type = R_pstream_ascii_format;
     else type = R_pstream_xdr_format;
@@ -1724,7 +1807,7 @@ SEXP do_serializeToConn(SEXP call, SEXP op, SEXP args, SEXP env)
     return R_NilValue;
 }
 
-SEXP do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden do_unserializeFromConn(SEXP call, SEXP op, SEXP args, SEXP env)
 {
     /* unserializeFromConn(conn, hook) */
 
@@ -1799,7 +1882,8 @@ static void InitBConOutPStream(R_outpstream_t stream, bconbuf_t bb,
 		     OutCharBB, OutBytesBB, phook, pdata);
 }
 
-SEXP R_serializeb(SEXP object, SEXP icon, SEXP fun)
+/* only for use by serialize(), with binary write to a socket connection */
+SEXP attribute_hidden R_serializeb(SEXP object, SEXP icon, SEXP fun)
 {
     struct R_outpstream_st out;
     SEXP (*hook)(SEXP, SEXP);
@@ -1820,17 +1904,24 @@ SEXP R_serializeb(SEXP object, SEXP icon, SEXP fun)
  */
 
 typedef struct membuf_st {
-    int size;
-    int count;
+    R_size_t size;
+    R_size_t count;
     unsigned char *buf;
 } *membuf_t;
 
-static void resize_buffer(membuf_t mb, int needed)
+static void resize_buffer(membuf_t mb, R_size_t needed)
 {
-    int newsize = 2 * needed;
+    /* This used to allocate double 'needed', but that was problematic for
+       large buffers */
+    R_size_t newsize = needed;
+    /* we need to store the result in a RAWSXP */
+    if(needed > INT_MAX)
+	error(_("serialization is too large to store in a raw vector"));
+    if(needed < INT_MAX - MAXELTSIZE) needed += MAXELTSIZE;
     mb->buf = realloc(mb->buf, newsize);
     if (mb->buf == NULL)
 	error(_("cannot allocate buffer"));
+    mb->size = newsize;
 }
 
 static void OutCharMem(R_outpstream_t stream, int c)
@@ -1844,13 +1935,16 @@ static void OutCharMem(R_outpstream_t stream, int c)
 static void OutBytesMem(R_outpstream_t stream, void *buf, int length)
 {
     membuf_t mb = stream->data;
-    if (mb->count + length > mb->size)
-	resize_buffer(mb, mb->count + length);
+    R_size_t needed = mb->count + (R_size_t) length;
+    /* There is a potential overflow here on 32-bit systems */
+    if((double) mb->count + length > (double) INT_MAX)
+	error(_("serialization is too large to store in a raw vector"));
+    if (needed > mb->size) resize_buffer(mb, needed);
     memcpy(mb->buf + mb->count, buf, length);
-    mb->count += length;
+    mb->count = needed;
 }
 
-static int InCHarMem(R_inpstream_t stream)
+static int InCharMem(R_inpstream_t stream)
 {
     membuf_t mb = stream->data;
     if (mb->count >= mb->size)
@@ -1861,7 +1955,7 @@ static int InCHarMem(R_inpstream_t stream)
 static void InBytesMem(R_inpstream_t stream, void *buf, int length)
 {
     membuf_t mb = stream->data;
-    if (mb->count + length > mb->size)
+    if (mb->count + (R_size_t) length > mb->size)
 	error(_("read error"));
     memcpy(buf, mb->buf + mb->count, length);
     mb->count += length;
@@ -1875,7 +1969,7 @@ static void InitMemInPStream(R_inpstream_t stream, membuf_t mb,
     mb->size = length;
     mb->buf = buf;
     R_InitInPStream(stream, (R_pstream_data_t) mb, R_pstream_any_format,
-		    InCHarMem, InBytesMem, phook, pdata);
+		    InCharMem, InBytesMem, phook, pdata);
 }
 
 static void InitMemOutPStream(R_outpstream_t stream, membuf_t mb,
@@ -1898,20 +1992,23 @@ static void free_mem_buffer(void *data)
 	free(buf);
     }
 }
-    
+
 static SEXP CloseMemOutPStream(R_outpstream_t stream)
 {
     SEXP val;
     membuf_t mb = stream->data;
-    PROTECT(val = allocVector(CHARSXP, mb->count));
-    memcpy(CHAR(val), mb->buf, mb->count);
-    val = ScalarString(val);
+    /* duplicate check, for future proofing */
+    if(mb->count > INT_MAX)
+	error(_("serialization is too large to store in a raw vector"));
+    PROTECT(val = allocVector(RAWSXP, mb->count));
+    memcpy(RAW(val), mb->buf, mb->count);
     free_mem_buffer(mb);
     UNPROTECT(1);
     return val;
 }
 
-SEXP R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP fun)
+SEXP attribute_hidden
+R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP fun)
 {
     struct R_outpstream_st out;
     R_pstream_format_t type;
@@ -1928,7 +2025,7 @@ SEXP R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP fun)
 	SEXP val;
 
 	/* set up a context which will free the buffer if there is an error */
-	begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_NilValue, R_NilValue,
+	begincontext(&cntxt, CTXT_CCODE, R_NilValue, R_BaseEnv, R_BaseEnv,
 		     R_NilValue, R_NilValue);
 	cntxt.cend = &free_mem_buffer;
 	cntxt.cenddata = &mbs;
@@ -1952,7 +2049,8 @@ SEXP R_serialize(SEXP object, SEXP icon, SEXP ascii, SEXP fun)
     }
 }
 
-SEXP R_unserialize(SEXP icon, SEXP fun)
+
+SEXP attribute_hidden R_unserialize(SEXP icon, SEXP fun)
 {
     struct R_inpstream_st in;
     SEXP (*hook)(SEXP, SEXP);
@@ -1960,13 +2058,16 @@ SEXP R_unserialize(SEXP icon, SEXP fun)
     hook = fun != R_NilValue ? CallHook : NULL;
 
     if (TYPEOF(icon) == STRSXP && LENGTH(icon) > 0) {
+	/* was the format in R < 2.4.0, removed in R 2.8.0 */
+	error("character vectors are no longer accepted by unserialize()");
+	return R_NilValue; /* -Wall */
+    } else if (TYPEOF(icon) == RAWSXP) {
 	struct membuf_st mbs;
-	void *data = CHAR(STRING_ELT(icon, 0));
-	int length = LENGTH(STRING_ELT(icon, 0));
+	void *data = RAW(icon);
+	int length = LENGTH(icon);
 	InitMemInPStream(&in, &mbs, data,  length, hook, fun);
 	return R_Unserialize(&in);
-    }
-    else {
+    } else {
 	Rconnection con = getConnection(asInteger(icon));
 	R_InitConnInPStream(&in, con, R_pstream_any_format, hook, fun);
 	return R_Unserialize(&in);
@@ -1978,17 +2079,14 @@ SEXP R_unserialize(SEXP icon, SEXP fun)
  * Support Code for Lazy Loading of Packages
  */
 
-/* This include is to bring in declarations of R_compress1 and
-   R_decompress1 */
-#include "basedecl.h"
 
 #define IS_PROPER_STRING(s) (TYPEOF(s) == STRSXP && LENGTH(s) > 0)
 
-/* Appends a scalar string to the end of a file using binary mode.
+/* Appends a raw vector to the end of a file using binary mode.
    Returns an integer vector of the initial offset of the string in
-   the file and the length of the string. */
+   the file and the length of the vector. */
 
-static SEXP appendStringToFile(SEXP file, SEXP string)
+static SEXP appendRawToFile(SEXP file, SEXP bytes)
 {
     FILE *fp;
     size_t len, out;
@@ -1997,21 +2095,25 @@ static SEXP appendStringToFile(SEXP file, SEXP string)
 
     if (! IS_PROPER_STRING(file))
 	error(_("not a proper file name"));
-    if (! IS_PROPER_STRING(string))
-	error(_("not a proper string"));
+    if (TYPEOF(bytes) != RAWSXP)
+	error(_("not a proper raw vector"));
 #ifdef HAVE_WORKING_FTELL
     /* Windows' ftell returns position 0 with "ab" */
-    if ((fp = fopen(CHAR(STRING_ELT(file, 0)), "ab")) == NULL)
-	error(_("file open failed"));
+    if ((fp = R_fopen(CHAR(STRING_ELT(file, 0)), "ab")) == NULL) {
+	error( _("cannot open file '%s': %s"), CHAR(STRING_ELT(file, 0)),
+	       strerror(errno));
+    }
 #else
-    if ((fp = fopen(CHAR(STRING_ELT(file, 0)), "r+b")) == NULL)
-	 error(_("file open failed"));
+    if ((fp = R_fopen(CHAR(STRING_ELT(file, 0)), "r+b")) == NULL) {
+	error( _("cannot open file '%s': %s"), CHAR(STRING_ELT(file, 0)),
+	       strerror(errno));
+    }
     fseek(fp, 0, SEEK_END);
 #endif
 
-    len = LENGTH(STRING_ELT(string, 0));
+    len = LENGTH(bytes);
     pos = ftell(fp);
-    out = fwrite(CHAR(STRING_ELT(string, 0)), 1, len, fp);
+    out = fwrite(RAW(bytes), 1, len, fp);
     fclose(fp);
 
     if (out != len) error(_("write failed"));
@@ -2023,17 +2125,17 @@ static SEXP appendStringToFile(SEXP file, SEXP string)
     return val;
 }
 
-/* Experimental interface to cache the pkg.rdb files */
+/* Interface to cache the pkg.rdb files */
 
 #define NC 100
 static int used = 0;
 static char names[NC][PATH_MAX];
 static char *ptr[NC];
 
-SEXP R_lazyLoadDBflush(SEXP file)
+SEXP attribute_hidden R_lazyLoadDBflush(SEXP file)
 {
     int i;
-    char *cfile = CHAR(STRING_ELT(file, 0));
+    const char *cfile = CHAR(STRING_ELT(file, 0));
 
     /* fprintf(stderr, "flushing file %s", cfile); */
     for (i = 0; i < used; i++)
@@ -2049,14 +2151,16 @@ SEXP R_lazyLoadDBflush(SEXP file)
 
 
 /* Reads, in binary mode, the bytes in the range specified by a
-   position/length vector and returns them as a scalar string. */
+   position/length vector and returns them as raw vector. */
 
-static SEXP readStringFromFile(SEXP file, SEXP key)
+/* There are some large lazy-data examples, e.g. 80Mb for SNPMaP.cdm */
+#define LEN_LIMIT 10*1048576
+static SEXP readRawFromFile(SEXP file, SEXP key)
 {
     FILE *fp;
     int offset, len, in, i, icache = -1, filelen;
     SEXP val;
-    char *cfile = CHAR(STRING_ELT(file, 0));
+    const char *cfile = CHAR(STRING_ELT(file, 0));
 
     if (! IS_PROPER_STRING(file))
 	error(_("not a proper file name"));
@@ -2066,13 +2170,12 @@ static SEXP readStringFromFile(SEXP file, SEXP key)
     offset = INTEGER(key)[0];
     len = INTEGER(key)[1];
 
-    val = allocVector(CHARSXP, len);
-    val = ScalarString(val);
+    val = allocVector(RAWSXP, len);
     /* Do we have this database cached? */
     for (i = 0; i < used; i++)
 	if(strcmp(cfile, names[i]) == 0) {icache = i; break;}
     if (icache >= 0) {
-	memcpy(CHAR(STRING_ELT(val, 0)), ptr[icache]+offset, len);
+	memcpy(RAW(val), ptr[icache]+offset, len);
 	return val;
     }
 
@@ -2081,38 +2184,49 @@ static SEXP readStringFromFile(SEXP file, SEXP key)
 	if(strcmp("", names[i]) == 0) {icache = i; break;}
     if(icache < 0 && used < NC) icache = used++;
 
-    if(icache >= 0) { 
-	strcpy(names[icache], cfile);
-	if ((fp = fopen(cfile, "rb")) == NULL)
-	    error(_("open failed on %s"), cfile);
+    if(icache >= 0) {
+	if ((fp = R_fopen(cfile, "rb")) == NULL)
+	    error(_("cannot open file '%s': %s"), cfile, strerror(errno));
 	if (fseek(fp, 0, SEEK_END) != 0) {
 	    fclose(fp);
 	    error(_("seek failed on %s"), cfile);
 	}
 	filelen = ftell(fp);
-	/* fprintf(stderr, "adding file %s at pos %d in cache, length %d\n", 
-	   cfile, icache, filelen); */
-	ptr[icache] = malloc(filelen);
-	if (fseek(fp, 0, SEEK_SET) != 0) {
+	if (filelen < LEN_LIMIT) {
+	    /* fprintf(stderr, "adding file '%s' at pos %d in cache, length %d\n",
+	       cfile, icache, filelen); */
+	    strcpy(names[icache], cfile);
+	    ptr[icache] = malloc(filelen);
+	    if (fseek(fp, 0, SEEK_SET) != 0) {
+		fclose(fp);
+		error(_("seek failed on %s"), cfile);
+	    }
+	    in = fread(ptr[icache], 1, filelen, fp);
 	    fclose(fp);
-	    error(_("seek failed on %s"), cfile);
-	}
-	in = fread(ptr[icache], 1, filelen, fp);
-	fclose(fp);
-	if (filelen != in) error(_("read failed on %s"), cfile);
-	memcpy(CHAR(STRING_ELT(val, 0)), ptr[icache]+offset, len);	
-    } else {
-	if ((fp = fopen(cfile, "rb")) == NULL)
-	    error(_("open failed on %s"), cfile);
-	if (fseek(fp, offset, SEEK_SET) != 0) {
+	    if (filelen != in) error(_("read failed on %s"), cfile);
+	    memcpy(RAW(val), ptr[icache]+offset, len);
+	    return val;
+	} else {
+	    if (fseek(fp, offset, SEEK_SET) != 0) {
+		fclose(fp);
+		error(_("seek failed on %s"), cfile);
+	    }
+	    in = fread(RAW(val), 1, len, fp);
 	    fclose(fp);
-	    error(_("seek failed on %s"), cfile);
+	    if (len != in) error(_("read failed on %s"), cfile);
+	    return val;
 	}
-	in = fread(CHAR(STRING_ELT(val, 0)), 1, len, fp);
-	fclose(fp);
-	if (len != in) error(_("read failed on %s"), cfile);
     }
-    
+
+    if ((fp = R_fopen(cfile, "rb")) == NULL)
+	error(_("cannot open file '%s': %s"), cfile, strerror(errno));
+    if (fseek(fp, offset, SEEK_SET) != 0) {
+	fclose(fp);
+	error(_("seek failed on %s"), cfile);
+    }
+    in = fread(RAW(val), 1, len, fp);
+    fclose(fp);
+    if (len != in) error(_("read failed on %s"), cfile);
     return val;
 }
 
@@ -2120,60 +2234,80 @@ static SEXP readStringFromFile(SEXP file, SEXP key)
    as a list.  If the force argument is true, promises are forced;
    otherwise they are not. */
 
-SEXP R_getVarsFromFrame(SEXP vars, SEXP env, SEXP forcesxp)
+SEXP attribute_hidden R_getVarsFromFrame(SEXP vars, SEXP env, SEXP forcesxp)
 {
     SEXP val, tmp, sym;
     Rboolean force;
     int i, len;
 
-    if (TYPEOF(env) != NILSXP && TYPEOF(env) != ENVSXP)
-        error(_("bad environment"));
+    if (TYPEOF(env) == NILSXP) {
+	error(_("use of NULL environment is defunct"));
+	env = R_BaseEnv;
+    } else
+    if (TYPEOF(env) != ENVSXP)
+	error(_("bad environment"));
     if (TYPEOF(vars) != STRSXP)
-        error(_("bad variable names"));
+	error(_("bad variable names"));
     force = asLogical(forcesxp);
 
     len = LENGTH(vars);
     PROTECT(val = allocVector(VECSXP, len));
     for (i = 0; i < len; i++) {
 	sym = install(CHAR(STRING_ELT(vars, i)));
-        if (TYPEOF(env) == NILSXP)
-	    tmp = findVar(sym, env);
-	else
-	    tmp = findVarInFrame(env, sym);
-	if (tmp == R_UnboundValue)
+
+	tmp = findVarInFrame(env, sym);
+	if (tmp == R_UnboundValue) {
+/*		PrintValue(env);
+		PrintValue(R_GetTraceback(0)); */  /* DJM debugging */
 	    error(_("object '%s' not found"), CHAR(STRING_ELT(vars, i)));
-        if (force && TYPEOF(tmp) == PROMSXP) {
-            PROTECT(tmp);
-            tmp = eval(tmp, R_GlobalEnv);
-            SET_NAMED(tmp, 2);
-            UNPROTECT(1);
-        }
-        else if (TYPEOF(tmp) != NILSXP && NAMED(tmp) < 1)
-            SET_NAMED(tmp, 1);
+	    }
+	if (force && TYPEOF(tmp) == PROMSXP) {
+	    PROTECT(tmp);
+	    tmp = eval(tmp, R_GlobalEnv);
+	    SET_NAMED(tmp, 2);
+	    UNPROTECT(1);
+	}
+	else if (TYPEOF(tmp) != NILSXP && NAMED(tmp) < 1)
+	    SET_NAMED(tmp, 1);
 	SET_VECTOR_ELT(val, i, tmp);
     }
-    setAttrib(val, R_NamesSymbol, vars);    
+    setAttrib(val, R_NamesSymbol, vars);
     UNPROTECT(1);
 
     return val;
 }
 
+SEXP R_compress1(SEXP in);
+SEXP R_decompress1(SEXP in);
+SEXP R_compress2(SEXP in);
+SEXP R_decompress2(SEXP in);
+SEXP R_compress3(SEXP in);
+SEXP R_decompress3(SEXP in);
+
 /* Serializes and, optionally, compresses a value and appends the
    result to a file.  Returns the key position/length key for
    retrieving the value */
 
-SEXP R_lazyLoadDBinsertValue(SEXP value, SEXP file, SEXP ascii,
-			     SEXP compsxp, SEXP hook)
+SEXP attribute_hidden
+R_lazyLoadDBinsertValue(SEXP value, SEXP file, SEXP ascii,
+			SEXP compsxp, SEXP hook)
 {
     PROTECT_INDEX vpi;
-    Rboolean compress = asLogical(compsxp);
+    Rboolean compress = asInteger(compsxp);
     SEXP key;
 
     value = R_serialize(value, R_NilValue, ascii, hook);
     PROTECT_WITH_INDEX(value, &vpi);
-    if (compress)
+#ifdef HAVE_LZMA
+    if (compress == 3)
+	REPROTECT(value = R_compress3(value), vpi);
+    else
+#endif
+    if (compress == 2)
+	REPROTECT(value = R_compress2(value), vpi);
+    else if (compress)
 	REPROTECT(value = R_compress1(value), vpi);
-    key = appendStringToFile(file, value);
+    key = appendRawToFile(file, value);
     UNPROTECT(1);
     return key;
 }
@@ -2182,20 +2316,36 @@ SEXP R_lazyLoadDBinsertValue(SEXP value, SEXP file, SEXP ascii,
    from a file, optionally decompresses, and unserializes the bytes.
    If the result is a promise, then the promise is forced. */
 
-SEXP R_lazyLoadDBfetch(SEXP key, SEXP file, SEXP compsxp, SEXP hook)
+SEXP attribute_hidden
+do_lazyLoadDBfetch(SEXP call, SEXP op, SEXP args, SEXP env)
 {
+    SEXP key, file, compsxp, hook;
     PROTECT_INDEX vpi;
-    Rboolean compressed = asLogical(compsxp);
+    Rboolean compressed;
     SEXP val;
 
-    PROTECT_WITH_INDEX(val = readStringFromFile(file, key), &vpi);
-    if (compressed)
+    checkArity(op, args);
+    key = CAR(args); args = CDR(args);
+    file = CAR(args); args = CDR(args);
+    compsxp = CAR(args); args = CDR(args);
+    hook = CAR(args);
+    compressed = asInteger(compsxp);
+
+    PROTECT_WITH_INDEX(val = readRawFromFile(file, key), &vpi);
+#ifdef HAVE_LZMA
+    if (compressed == 3)
+	REPROTECT(val = R_decompress3(val), vpi);
+    else
+#endif
+   if (compressed == 2)
+	REPROTECT(val = R_decompress2(val), vpi);
+    else if (compressed)
 	REPROTECT(val = R_decompress1(val), vpi);
     val = R_unserialize(val, hook);
     if (TYPEOF(val) == PROMSXP) {
-        REPROTECT(val, vpi);
-        val = eval(val, R_GlobalEnv);
-        SET_NAMED(val, 2);
+	REPROTECT(val, vpi);
+	val = eval(val, R_GlobalEnv);
+	SET_NAMED(val, 2);
     }
     UNPROTECT(1);
     return val;
