@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2003  Robert Gentleman, Ross Ihaka and the
+ *  Copyright (C) 1997--2007  Robert Gentleman, Ross Ihaka and the
  *                            R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,39 +15,62 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
+ *
+ * EXPORTS:
+ *
+ *  OneIndex()        -- used for "[[<-" in ./subassign.c
+ *  get1index()       -- used for "[["   in ./subassign.c & subset.c
+ *  vectorIndex()     -- used for "[[" with a vector arg
+
+ *  mat2indsub()      -- for "mat[i]"     "    "            "
+
+ *  makeSubscript()   -- for "[" and "[<-" in ./subset.c and ./subassign.c,
+ *			 and "[[<-" with a scalar in ./subassign.c
+ *  vectorSubscript() -- for makeSubscript()   {currently unused externally}
+ *  arraySubscript()  -- for "[i,j,..." and "[<-..." in ./subset.c, ./subassign.c
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include "Defn.h"
+#include <Defn.h>
 
-static int integerOneIndex(int i, int len) {
+/* We might get a call with R_NilValue from subassignment code */
+#define ECALL(call, yy) if(call == R_NilValue) error(yy); else errorcall(call, yy);
+
+static int integerOneIndex(int i, int len, SEXP call)
+{
     int indx = -1;
 
     if (i > 0)
 	indx = i - 1;
-    else if (i == 0 || len < 2)
-	error("attempt to select less than one element");
-    else if (len == 2 && i > -3)
+    else if (i == 0 || len < 2) {
+	ECALL(call, _("attempt to select less than one element"));
+    } else if (len == 2 && i > -3)
 	indx = 2 + i;
-    else
-	error("attempt to select more than one element");
+    else {
+	ECALL(call, _("attempt to select more than one element"));
+    }
     return(indx);
 }
 
-int OneIndex(SEXP x, SEXP s, int len, int partial, SEXP *newname, int pos)
+/* Utility used (only in) do_subassign2_dflt(), i.e. "[[<-" in ./subassign.c : */
+int attribute_hidden
+OneIndex(SEXP x, SEXP s, int len, int partial, SEXP *newname, int pos, SEXP call)
 {
     SEXP names;
     int i, indx, nx;
 
-    if (pos < 0 && length(s) > 1)
-	error("attempt to select more than one element");
-    if (pos < 0 && length(s) < 1)
-	error("attempt to select less than one element");
+    if (pos < 0 && length(s) > 1) {
+	ECALL(call, _("attempt to select more than one element"));
+    }
+    if (pos < 0 && length(s) < 1) {
+	ECALL(call, _("attempt to select less than one element"));
+    }
+
     if(pos < 0) pos = 0;
 
     indx = -1;
@@ -55,28 +78,31 @@ int OneIndex(SEXP x, SEXP s, int len, int partial, SEXP *newname, int pos)
     switch(TYPEOF(s)) {
     case LGLSXP:
     case INTSXP:
-	indx = integerOneIndex(INTEGER(s)[pos], len);
+	indx = integerOneIndex(INTEGER(s)[pos], len, call);
 	break;
     case REALSXP:
-	indx = integerOneIndex(REAL(s)[pos], len);
+	indx = integerOneIndex(REAL(s)[pos], len, call);
 	break;
     case STRSXP:
 	nx = length(x);
 	names = getAttrib(x, R_NamesSymbol);
 	if (names != R_NilValue) {
 	    /* Try for exact match */
-	    for (i = 0; i < nx; i++)
-		if (streql(CHAR(STRING_ELT(names, i)),
-			   CHAR(STRING_ELT(s, pos)))) {
+	    for (i = 0; i < nx; i++) {
+		const char *tmp = translateChar(STRING_ELT(names, i));
+		if (!tmp[0]) continue;
+		if (streql(tmp, translateChar(STRING_ELT(s, pos)))) {
 		    indx = i;
 		    break;
 		}
+	    }
 	    /* Try for partial match */
 	    if (partial && indx < 0) {
-		len = strlen(CHAR(STRING_ELT(s, pos)));
+		len = strlen(translateChar(STRING_ELT(s, pos)));
 		for(i = 0; i < nx; i++) {
-		    if(!strncmp(CHAR(STRING_ELT(names, i)),
-				CHAR(STRING_ELT(s, pos)), len)) {
+		    const char *tmp = translateChar(STRING_ELT(names, i));
+		    if (!tmp[0]) continue;
+		    if(!strncmp(tmp, translateChar(STRING_ELT(s, pos)), len)) {
 			if(indx == -1 )
 			    indx = i;
 			else
@@ -94,8 +120,8 @@ int OneIndex(SEXP x, SEXP s, int len, int partial, SEXP *newname, int pos)
 	names = getAttrib(x, R_NamesSymbol);
 	if (names != R_NilValue) {
 	    for (i = 0; i < nx; i++)
-		if (streql(CHAR(STRING_ELT(names, i)),
-			   CHAR(PRINTNAME(s)))) {
+		if (streql(translateChar(STRING_ELT(names, i)),
+			   translateChar(PRINTNAME(s)))) {
 		    indx = i;
 		    break;
 		}
@@ -105,28 +131,42 @@ int OneIndex(SEXP x, SEXP s, int len, int partial, SEXP *newname, int pos)
 	*newname = STRING_ELT(s, pos);
 	break;
     default:
-	error("invalid subscript type");
+	if (call == R_NilValue)
+	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
+	else
+	    errorcall(call, _("invalid subscript type '%s'"),
+		      type2char(TYPEOF(s)));
     }
     return indx;
 }
 
-int get1index(SEXP s, SEXP names, int len, Rboolean pok, int pos)
+int attribute_hidden
+get1index(SEXP s, SEXP names, int len, int pok, int pos, SEXP call)
 {
 /* Get a single index for the [[ operator.
    Check that only one index is being selected.
    pok : is "partial ok" ?
+	 if pok is -1, warn if partial matching occurs
 */
-    int indx, i;
+    int indx, i, warn_pok = 0;
     double dblind;
+    const char *ss, *cur_name;
+
+    if (pok == -1) {
+	pok = 1;
+	warn_pok = 1;
+    }
 
     if (pos < 0 && length(s) != 1) {
-	if (length(s) > 1)
-	    error("attempt to select more than one element");
-	else
-	    error("attempt to select less than one element");
+	if (length(s) > 1) {
+	    ECALL(call, _("attempt to select more than one element"));
+	} else {
+	    ECALL(call, _("attempt to select less than one element"));
+	}
     } else
-	if(pos >= length(s))
-	    error("internal error in use of recursive indexing");
+	if(pos >= length(s)) {
+	    ECALL(call, _("internal error in use of recursive indexing"));
+	}
     if(pos < 0) pos = 0;
     indx = -1;
     switch (TYPEOF(s)) {
@@ -134,75 +174,145 @@ int get1index(SEXP s, SEXP names, int len, Rboolean pok, int pos)
     case INTSXP:
 	i = INTEGER(s)[pos];
 	if(i != NA_INTEGER)
-	    indx = integerOneIndex(i, len);
+	    indx = integerOneIndex(i, len, call);
 	break;
     case REALSXP:
 	dblind = REAL(s)[pos];
 	if(!ISNAN(dblind))
-	    indx = integerOneIndex((int)dblind, len);
+	    indx = integerOneIndex((int)dblind, len, call);
 	break;
     case STRSXP:
+	/* NA matches nothing */
+	if(STRING_ELT(s, pos) == NA_STRING) break;
+	/* "" matches nothing: see names.Rd */
+	if(!CHAR(STRING_ELT(s, pos))[0]) break;
+
 	/* Try for exact match */
+	ss = translateChar(STRING_ELT(s, pos));
 	for (i = 0; i < length(names); i++)
-	    if (streql(CHAR(STRING_ELT(names, i)),
-		       CHAR(STRING_ELT(s, pos)))) {
-		indx = i;
-		break;
+	    if (STRING_ELT(names, i) != NA_STRING) {
+		if (streql(translateChar(STRING_ELT(names, i)), ss)) {
+		    indx = i;
+		    break;
+		}
 	    }
 	/* Try for partial match */
 	if (pok && indx < 0) {
-	    len = strlen(CHAR(STRING_ELT(s, pos)));
+	    len = strlen(ss);
 	    for(i = 0; i < length(names); i++) {
-		if(!strncmp(CHAR(STRING_ELT(names, i)),
-			    CHAR(STRING_ELT(s, pos)), len)) {
-		    if(indx == -1)/* first one */
-			indx = i;
-		    else
-			indx = -2;/* more than one partial match */
+		if (STRING_ELT(names, i) != NA_STRING) {
+		    cur_name = translateChar(STRING_ELT(names, i));
+		    if(!strncmp(cur_name, ss, len)) {
+			if(indx == -1) {/* first one */
+			    indx = i;
+			    if (warn_pok) {
+				if (call == R_NilValue)
+				    warning(_("partial match of '%s' to '%s'"),
+					    ss, cur_name);
+				else
+				    warningcall(call,
+						_("partial match of '%s' to '%s'"),
+						ss, cur_name);
+			    }
+			}
+			else {
+			    indx = -2;/* more than one partial match */
+			    if (warn_pok) /* already given context */
+				warningcall(R_NilValue,
+					    _("further partial match of '%s' to '%s'"),
+					    ss, cur_name);
+			    break;
+			}
+		    }
 		}
 	    }
 	}
 	break;
     case SYMSXP:
 	for (i = 0; i < length(names); i++)
-	    if (streql(CHAR(STRING_ELT(names, i)), CHAR(PRINTNAME(s)))) {
+	    if (STRING_ELT(names, i) != NA_STRING &&
+		streql(translateChar(STRING_ELT(names, i)),
+		       CHAR(PRINTNAME(s)))) {
 		indx = i;
 		break;
 	    }
     default:
-	error("invalid subscript type");
+	if (call == R_NilValue)
+	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
+	else
+	    errorcall(call, _("invalid subscript type '%s'"),
+		      type2char(TYPEOF(s)));
     }
     return indx;
 }
 
+SEXP attribute_hidden
+vectorIndex(SEXP x, SEXP thesub, int start, int stop, int pok, SEXP call) 
+{
+    int i, offset;
+
+    for(i = start; i < stop; i++) {
+	if(!isVectorList(x) && !isPairList(x))
+	    errorcall(call, _("recursive indexing failed at level %d\n"), i+1);
+	offset = get1index(thesub, getAttrib(x, R_NamesSymbol),
+		           length(x), pok, i, call);
+	if(offset < 0 || offset >= length(x))
+	    errorcall(call, _("no such index at level %d\n"), i+1);
+	if(isPairList(x)) {
+	    x = CAR(nthcdr(x, offset));
+	} else {
+	    x = VECTOR_ELT(x, offset);
+    	}
+    }
+    return x;
+}
+
 /* Special Matrix Subscripting: Handles the case x[i] where */
 /* x is an n-way array and i is a matrix with n columns. */
 /* This code returns a vector containing the integer subscripts */
 /* to be extracted when x is regarded as unravelled. */
+/* Negative indices are not allowed. */
+/* A zero anywhere in a row will cause a zero in the same */
+/* position in the result. */
 
-SEXP mat2indsub(SEXP dims, SEXP s)
+SEXP attribute_hidden mat2indsub(SEXP dims, SEXP s, SEXP call)
 {
-    int tdim, j, i, nrs = nrows(s);
+    int tdim, j, i, k, nrs = nrows(s);
     SEXP rvec;
 
+    if (ncols(s) != LENGTH(dims)) {
+	ECALL(call, _("incorrect number of columns in matrix subscript"));
+    }
+
     PROTECT(rvec = allocVector(INTSXP, nrs));
     s = coerceVector(s, INTSXP);
     setIVector(INTEGER(rvec), nrs, 0);
 
-    /* compute 0-based subscripts */
     for (i = 0; i < nrs; i++) {
 	tdim = 1;
+	/* compute 0-based subscripts for a row (0 in the input gets -1
+	   in the output here) */
 	for (j = 0; j < LENGTH(dims); j++) {
-	    if(INTEGER(s)[i + j * nrs] == NA_INTEGER) {
+	    k = INTEGER(s)[i + j * nrs];
+	    if(k == NA_INTEGER) {
 		INTEGER(rvec)[i] = NA_INTEGER;
 		break;
 	    }
-	    if (INTEGER(s)[i + j * nrs] > INTEGER(dims)[j])
-		error("subscript out of bounds");
-	    INTEGER(rvec)[i] += (INTEGER(s)[i+j*nrs] - 1) * tdim;
+	    if(k < 0) {
+		ECALL(call, _("negative values are not allowed in a matrix subscript"));
+	    }
+	    if(k == 0) {
+		INTEGER(rvec)[i] = -1;
+		break;
+	    }
+	    if (k > INTEGER(dims)[j]) {
+		ECALL(call, _("subscript out of bounds"));
+	    }
+	    INTEGER(rvec)[i] += (k - 1) * tdim;
 	    tdim *= INTEGER(dims)[j];
 	}
-	/* transform to 1 based subscripting */
+	/* transform to 1 based subscripting (0 in the input gets 0
+	   in the output here) */
 	if(INTEGER(rvec)[i] != NA_INTEGER)
 	    INTEGER(rvec)[i]++;
     }
@@ -220,13 +330,14 @@ static SEXP nullSubscript(int n)
     return indx;
 }
 
-static SEXP logicalSubscript(SEXP s, int ns, int nx, int *stretch)
+static SEXP logicalSubscript(SEXP s, int ns, int nx, int *stretch, SEXP call)
 {
     int canstretch, count, i, nmax;
     SEXP indx;
     canstretch = *stretch;
-    if (!canstretch && ns > nx)
-	error("(subscript) logical subscript too long");
+    if (!canstretch && ns > nx) {
+	ECALL(call, _("(subscript) logical subscript too long"));
+    }
     nmax = (ns > nx) ? ns : nx;
     *stretch = (ns > nx) ? ns : 0;
     if (ns == 0)
@@ -247,20 +358,20 @@ static SEXP logicalSubscript(SEXP s, int ns, int nx, int *stretch)
     return indx;
 }
 
-static SEXP negativeSubscript(SEXP s, int ns, int nx)
+static SEXP negativeSubscript(SEXP s, int ns, int nx, SEXP call)
 {
     SEXP indx;
     int stretch = 0;
     int i, ix;
-    PROTECT(indx = allocVector(INTSXP, nx));
+    PROTECT(indx = allocVector(LGLSXP, nx));
     for (i = 0; i < nx; i++)
-	INTEGER(indx)[i] = 1;
+	LOGICAL(indx)[i] = 1;
     for (i = 0; i < ns; i++) {
 	ix = INTEGER(s)[i];
 	if (ix != 0 && ix != NA_INTEGER && -ix <= nx)
-	    INTEGER(indx)[-ix - 1] = 0;
+	    LOGICAL(indx)[-ix - 1] = 0;
     }
-    s = logicalSubscript(indx, nx, nx, &stretch);
+    s = logicalSubscript(indx, nx, nx, &stretch, call);
     UNPROTECT(1);
     return s;
 }
@@ -284,7 +395,7 @@ static SEXP positiveSubscript(SEXP s, int ns, int nx)
 	return s;
 }
 
-static SEXP integerSubscript(SEXP s, int ns, int nx, int *stretch)
+static SEXP integerSubscript(SEXP s, int ns, int nx, int *stretch, SEXP call)
 {
     int i, ii, min, max, canstretch;
     Rboolean isna = FALSE;
@@ -301,15 +412,17 @@ static SEXP integerSubscript(SEXP s, int ns, int nx, int *stretch)
 		max = ii;
 	} else isna = TRUE;
     }
-    if (min < -nx)
-	error("subscript out of bounds");
     if (max > nx) {
 	if(canstretch) *stretch = max;
-	else error("subscript out of bounds");
+	else {
+	    ECALL(call, _("subscript out of bounds"));
+	}
     }
     if (min < 0) {
-	if (max == 0 && !isna) return negativeSubscript(s, ns, nx);
-	else error("only 0's may mix with negative subscripts");
+	if (max == 0 && !isna) return negativeSubscript(s, ns, nx, call);
+	else {
+	    ECALL(call, _("only 0's may be mixed with negative subscripts"));
+	}
     }
     else return positiveSubscript(s, ns, nx);
     return R_NilValue;
@@ -333,18 +446,15 @@ typedef SEXP (*StringEltGetter)(SEXP x, int i);
  * large, then it will be too slow unless ns is very small.
  */
 
-#define USE_HASHING 1
-static SEXP stringSubscript(SEXP s, int ns, int nx, SEXP names,
-			    StringEltGetter strg, int *stretch, Rboolean in)
+static SEXP
+stringSubscript(SEXP s, int ns, int nx, SEXP names,
+		StringEltGetter strg, int *stretch, Rboolean in, SEXP call)
 {
     SEXP indx, indexnames;
     int i, j, nnames, sub, extra;
     int canstretch = *stretch;
-#ifdef USE_HASHING
-    Rboolean usehashing = in && (ns * nx > 1000);
-#else
-    Rboolean usehashing = FALSE;
-#endif
+    /* product may overflow, so check factors as well. */
+    Rboolean usehashing = in && ( ((ns > 1000 && nx) || (nx > 1000 && ns)) || (ns * nx > 15*nx + ns) );
 
     PROTECT(s);
     PROTECT(names);
@@ -359,23 +469,30 @@ static SEXP stringSubscript(SEXP s, int ns, int nx, SEXP names,
      * nonmatch will have given an error.)
      */
 
-#ifdef USE_HASHING
     if(usehashing) {
 	/* must be internal, so names contains a character vector */
+	/* NB: this does not behave in the same way with respect to ""
+	   and NA names: they will match */
 	PROTECT(indx = match(names, s, 0));
+	/* second pass to correct this */
+	for (i = 0; i < ns; i++)
+	    if(STRING_ELT(s, i) == NA_STRING || !CHAR(STRING_ELT(s, i))[0])
+		INTEGER(indx)[i] = 0;
+	/* FIXME: this should not be allowed, CHARSXPs only */
 	for (i = 0; i < ns; i++) SET_STRING_ELT(indexnames, i, R_NilValue);
     } else {
-#endif
 	PROTECT(indx = allocVector(INTSXP, ns));
 	for (i = 0; i < ns; i++) {
 	    sub = 0;
 	    if (names != R_NilValue) {
 		for (j = 0; j < nnames; j++) {
 		    SEXP names_j = strg(names, j);
-		    if (!in && TYPEOF(names_j) != CHARSXP)
-			error("character vector element does not have type CHARSXP");
+		    if (!in && TYPEOF(names_j) != CHARSXP) {
+			ECALL(call, _("character vector element does not have type CHARSXP"));
+		    }
 		    if (NonNullStringMatch(STRING_ELT(s, i), names_j)) {
 			sub = j + 1;
+			/* FIXME: this should not be allowed, CHARSXPs only */
 			SET_STRING_ELT(indexnames, i, R_NilValue);
 			break;
 		    }
@@ -383,9 +500,8 @@ static SEXP stringSubscript(SEXP s, int ns, int nx, SEXP names,
 	    }
 	    INTEGER(indx)[i] = sub;
 	}
-#ifdef USE_HASHING
     }
-#endif
+
 
     for (i = 0; i < ns; i++) {
 	sub = INTEGER(indx)[i];
@@ -398,18 +514,19 @@ static SEXP stringSubscript(SEXP s, int ns, int nx, SEXP names,
 		}
 	}
 	if (sub == 0) {
-	    if (!canstretch)
-		error("subscript out of bounds");
+	    if (!canstretch) {
+		ECALL(call, _("subscript out of bounds"));
+	    }
 	    extra += 1;
 	    sub = extra;
 	    SET_STRING_ELT(indexnames, i, STRING_ELT(s, i));
 	}
 	INTEGER(indx)[i] = sub;
     }
-    /* Ghastly hack!  We attach the new names to the attribute */
-    /* slot on the returned subscript vector. */
+    /* We return the new names as the names attribute of the returned
+       subscript vector. */
     if (extra != nnames)
-	SET_ATTRIB(indx, indexnames);
+	setAttrib(indx, R_NamesSymbol, indexnames);
     if (canstretch)
 	*stretch = extra;
     UNPROTECT(4);
@@ -426,9 +543,9 @@ static SEXP stringSubscript(SEXP s, int ns, int nx, SEXP names,
 
 typedef SEXP AttrGetter(SEXP x, SEXP data);
 
-static SEXP 
+static SEXP
 int_arraySubscript(int dim, SEXP s, SEXP dims, AttrGetter dng,
-		   StringEltGetter strg, SEXP x, Rboolean in)
+		   StringEltGetter strg, SEXP x, Rboolean in, SEXP call)
 {
     int nd, ns, stretch = 0;
     SEXP dnames, tmp;
@@ -439,33 +556,41 @@ int_arraySubscript(int dim, SEXP s, SEXP dims, AttrGetter dng,
     case NILSXP:
 	return allocVector(INTSXP, 0);
     case LGLSXP:
-	return logicalSubscript(s, ns, nd, &stretch);
+	return logicalSubscript(s, ns, nd, &stretch, call);
     case INTSXP:
-	return integerSubscript(s, ns, nd, &stretch);
+	return integerSubscript(s, ns, nd, &stretch, call);
     case REALSXP:
-    	PROTECT(tmp = coerceVector(s, INTSXP));
-	tmp = integerSubscript(tmp, ns, nd, &stretch);
-    	UNPROTECT(1);
+	PROTECT(tmp = coerceVector(s, INTSXP));
+	tmp = integerSubscript(tmp, ns, nd, &stretch, call);
+	UNPROTECT(1);
 	return tmp;
     case STRSXP:
 	dnames = dng(x, R_DimNamesSymbol);
-	if (dnames == R_NilValue)
-	    error("no dimnames attribute for array");
+	if (dnames == R_NilValue) {
+	    ECALL(call, _("no 'dimnames' attribute for array"));
+	}
 	dnames = VECTOR_ELT(dnames, dim);
-	return stringSubscript(s, ns, nd, dnames, strg, &stretch, in);
+	return stringSubscript(s, ns, nd, dnames, strg, &stretch, in, call);
     case SYMSXP:
 	if (s == R_MissingArg)
 	    return nullSubscript(nd);
     default:
-	error("invalid subscript");
+	if (call == R_NilValue)
+	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
+	else
+	    errorcall(call, _("invalid subscript type '%s'"),
+		      type2char(TYPEOF(s)));
     }
     return R_NilValue;
 }
 
-SEXP arraySubscript(int dim, SEXP s, SEXP dims, AttrGetter dng,
-		    StringEltGetter strg, SEXP x)
+/* This is used by packages arules and cba. Seems dangerous as the
+   typedef is not exported */
+SEXP
+arraySubscript(int dim, SEXP s, SEXP dims, AttrGetter dng,
+	       StringEltGetter strg, SEXP x)
 {
-    return int_arraySubscript(dim, s, dims, dng, strg, x, TRUE);
+    return int_arraySubscript(dim, s, dims, dng, strg, x, TRUE, R_NilValue);
 }
 
 /* Subscript creation.  The first thing we do is check to see */
@@ -476,7 +601,7 @@ SEXP arraySubscript(int dim, SEXP s, SEXP dims, AttrGetter dng,
    otherwise, stretch returns the new required length for x
 */
 
-SEXP makeSubscript(SEXP x, SEXP s, int *stretch)
+SEXP attribute_hidden makeSubscript(SEXP x, SEXP s, int *stretch, SEXP call)
 {
     int nx;
     SEXP ans;
@@ -485,9 +610,12 @@ SEXP makeSubscript(SEXP x, SEXP s, int *stretch)
     if (isVector(x) || isList(x) || isLanguage(x)) {
 	nx = length(x);
 
-	ans = vectorSubscript(nx, s, stretch, getAttrib, (STRING_ELT), x);
+	ans = vectorSubscript(nx, s, stretch, getAttrib, (STRING_ELT),
+			      x, call);
+    }
+    else {
+	ECALL(call, _("subscripting on non-vector"));
     }
-    else error("subscripting on non-vector");
     return ans;
 
 }
@@ -498,12 +626,12 @@ SEXP makeSubscript(SEXP x, SEXP s, int *stretch)
    subsetting,
 */
 
-static SEXP 
+static SEXP
 int_vectorSubscript(int nx, SEXP s, int *stretch, AttrGetter dng,
-		    StringEltGetter strg, SEXP x, Rboolean in)
+		    StringEltGetter strg, SEXP x, Rboolean in, SEXP call)
 {
     int ns;
-    SEXP ans=R_NilValue, tmp;
+    SEXP ans = R_NilValue, tmp;
 
     ns = length(s);
     /* special case for simple indices -- does not duplicate */
@@ -514,8 +642,9 @@ int_vectorSubscript(int nx, SEXP s, int *stretch, AttrGetter dng,
 	    return s;
 	}
     }
-    PROTECT(s=duplicate(s));
+    PROTECT(s = duplicate(s));
     SET_ATTRIB(s, R_NilValue);
+    SET_OBJECT(s, 0);
     switch (TYPEOF(s)) {
     case NILSXP:
 	*stretch = 0;
@@ -523,21 +652,21 @@ int_vectorSubscript(int nx, SEXP s, int *stretch, AttrGetter dng,
 	break;
     case LGLSXP:
 	/* *stretch = 0; */
-	ans = logicalSubscript(s, ns, nx, stretch);
+	ans = logicalSubscript(s, ns, nx, stretch, call);
 	break;
     case INTSXP:
-	    ans = integerSubscript(s, ns, nx, stretch);
-	    break;
+	ans = integerSubscript(s, ns, nx, stretch, call);
+	break;
     case REALSXP:
 	PROTECT(tmp = coerceVector(s, INTSXP));
-	ans = integerSubscript(tmp, ns, nx, stretch);
+	ans = integerSubscript(tmp, ns, nx, stretch, call);
 	UNPROTECT(1);
 	break;
     case STRSXP:
     {
 	SEXP names = dng(x, R_NamesSymbol);
 	/* *stretch = 0; */
-	ans = stringSubscript(s, ns, nx, names, strg, stretch, in);
+	ans = stringSubscript(s, ns, nx, names, strg, stretch, in, call);
     }
     break;
     case SYMSXP:
@@ -547,16 +676,20 @@ int_vectorSubscript(int nx, SEXP s, int *stretch, AttrGetter dng,
 	    break;
 	}
     default:
-	error("invalid subscript type");
+	if (call == R_NilValue)
+	    error(_("invalid subscript type '%s'"), type2char(TYPEOF(s)));
+	else
+	    errorcall(call, _("invalid subscript type '%s'"),
+		      type2char(TYPEOF(s)));
     }
     UNPROTECT(1);
     return ans;
 }
 
 
-SEXP vectorSubscript(int nx, SEXP s, int *stretch, AttrGetter dng,
-		     StringEltGetter strg, SEXP x)
+SEXP attribute_hidden
+vectorSubscript(int nx, SEXP s, int *stretch, AttrGetter dng,
+		StringEltGetter strg, SEXP x, SEXP call)
 {
-    return int_vectorSubscript(nx, s, stretch, dng, strg, x, TRUE);
+    return int_vectorSubscript(nx, s, stretch, dng, strg, x, TRUE, call);
 }
-
