@@ -40,6 +40,10 @@
 
 #include "Defn.h"
 
+/* JMC convinced MM that this was not a good idea: */
+#undef _S4_subsettable
+
+
 /* ExtractSubset does the transfer of elements from "x" to "result" */
 /* according to the integer subscripts given in "indx". */
 
@@ -119,7 +123,7 @@ static SEXP ExtractSubset(SEXP x, SEXP result, SEXP indx, SEXP call)
 		RAW(result)[i] = (Rbyte) 0;
 	    break;
 	default:
-	    errorcall(call, R_MSG_ob_nonsub);
+	    errorcall(call, R_MSG_ob_nonsub, type2char(mode));
 	}
     }
     return result;
@@ -165,27 +169,36 @@ static SEXP VectorSubset(SEXP x, SEXP s, SEXP call)
 	SET_NAMED(result, 2);
 
     PROTECT(result = ExtractSubset(x, result, indx, call));
-    if (result != R_NilValue &&
-	(
+    if (result != R_NilValue) {
+	if (
 	    ((attrib = getAttrib(x, R_NamesSymbol)) != R_NilValue) ||
 	    ( /* here we might have an array.  Use row names if 1D */
 		isArray(x) && LENGTH(getAttrib(x, R_DimNamesSymbol)) == 1 &&
 		(attrib = getAttrib(x, R_DimNamesSymbol)) != R_NilValue &&
 		(attrib = GetRowNames(attrib)) != R_NilValue
 		)
-	    )) {
-	nattrib = allocVector(TYPEOF(attrib), n);
-	PROTECT(nattrib); /* seems unneeded */
-	nattrib = ExtractSubset(attrib, nattrib, indx, call);
-	setAttrib(result, R_NamesSymbol, nattrib);
-	UNPROTECT(1);
-    }
-    if (result != R_NilValue && (attrib = getAttrib(x, R_SrcrefSymbol)) != R_NilValue) {
-	nattrib = allocVector(VECSXP, n);
-	PROTECT(nattrib); /* seems unneeded */
-	nattrib = ExtractSubset(attrib, nattrib, indx, call);
-	setAttrib(result, R_SrcrefSymbol, nattrib);
-	UNPROTECT(1);
+	    ) {
+	    nattrib = allocVector(TYPEOF(attrib), n);
+	    PROTECT(nattrib); /* seems unneeded */
+	    nattrib = ExtractSubset(attrib, nattrib, indx, call);
+	    setAttrib(result, R_NamesSymbol, nattrib);
+	    UNPROTECT(1);
+	}
+	if ((attrib = getAttrib(x, R_SrcrefSymbol)) != R_NilValue &&
+	    TYPEOF(attrib) == VECSXP) {
+	    nattrib = allocVector(VECSXP, n);
+	    PROTECT(nattrib); /* seems unneeded */
+	    nattrib = ExtractSubset(attrib, nattrib, indx, call);
+	    setAttrib(result, R_SrcrefSymbol, nattrib);
+	    UNPROTECT(1);
+	}
+	/* FIXME:  this is wrong, because the slots are gone, so result is an invalid object of the S4 class! JMC 3/3/09 */
+#ifdef _S4_subsettable
+	if(IS_S4_OBJECT(x)) { /* e.g. contains = "list" */
+	    setAttrib(result, R_ClassSymbol, getAttrib(x, R_ClassSymbol));
+	    SET_S4_OBJECT(result);
+	}
+#endif
     }
     UNPROTECT(3);
     return result;
@@ -659,7 +672,7 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	for(px = x, i = 0 ; px != R_NilValue ; px = CDR(px))
 	    SET_VECTOR_ELT(ax, i++, CAR(px));
     }
-    else errorcall(call, R_MSG_ob_nonsub);
+    else errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
 
     /* This is the actual subsetting code. */
     /* The separation of arrays and matrices is purely an optimization. */
@@ -667,7 +680,8 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     if(nsubs < 2) {
 	SEXP dim = getAttrib(x, R_DimSymbol);
 	int ndim = length(dim);
-	ans = VectorSubset(ax, (nsubs == 1 ? CAR(subs) : R_MissingArg), call);
+	PROTECT(ans = VectorSubset(ax, (nsubs == 1 ? CAR(subs) : R_MissingArg),
+				   call));
 	/* one-dimensional arrays went through here, and they should
 	   have their dimensions dropped only if the result has
 	   length one and drop == TRUE
@@ -677,7 +691,6 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    int len = length(ans);
 
 	    if(!drop || len > 1) {
-		PROTECT(ans);
 		PROTECT(attr = allocVector(INTSXP, 1));
 		INTEGER(attr)[0] = length(ans);
 		setAttrib(ans, R_DimSymbol, attr);
@@ -691,7 +704,6 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    setAttrib(ans, R_NamesSymbol, R_NilValue);
 		    UNPROTECT(1);
 		}
-		UNPROTECT(1);
 	    }
 	}
     } else {
@@ -701,8 +713,8 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 	    ans = MatrixSubset(ax, subs, call, drop);
 	else
 	    ans = ArraySubset(ax, subs, call, drop);
+	PROTECT(ans);
     }
-    PROTECT(ans);
 
     /* Note: we do not coerce back to pair-based lists. */
     /* They are "defunct" in this version of R. */
@@ -722,9 +734,12 @@ SEXP attribute_hidden do_subset_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     else {
 	PROTECT(ans);
     }
-    if (ATTRIB(ans) != R_NilValue) {
+    if (ATTRIB(ans) != R_NilValue) { /* remove probably erroneous attr's */
 	setAttrib(ans, R_TspSymbol, R_NilValue);
-	setAttrib(ans, R_ClassSymbol, R_NilValue);
+#ifdef _S4_subsettable
+	if(!IS_S4_OBJECT(x))
+#endif
+	    setAttrib(ans, R_ClassSymbol, R_NilValue);
     }
     UNPROTECT(4);
     return ans;
@@ -758,6 +773,7 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     SEXP ans, dims, dimnames, indx, subs, x;
     int i, ndims, nsubs, offset = 0;
     int drop = 1, pok, exact = -1;
+    int named_x;
 
     PROTECT(args);
     ExtractDropArg(args, &drop);
@@ -791,6 +807,13 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
     if(nsubs > 1 && nsubs != ndims)
 	errorcall(call, _("incorrect number of subscripts"));
 
+    /* code to allow classes to extend environment */
+    if(TYPEOF(x) == S4SXP) {
+        x = R_getS4DataSlot(x, ANYSXP);
+	if(x == R_NilValue)
+	  errorcall(call, _("this S4 class is not subsettable"));
+    }
+
     /* split out ENVSXP for now */
     if( TYPEOF(x) == ENVSXP ) {
       if( nsubs != 1 || !isString(CAR(subs)) || length(CAR(subs)) != 1 )
@@ -812,26 +835,19 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     /* back to the regular program */
     if (!(isVector(x) || isList(x) || isLanguage(x)))
-	errorcall(call, R_MSG_ob_nonsub);
+	errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
+
+    named_x = NAMED(x);  /* x may change below; save this now.  See PR#13411 */
 
     if(nsubs == 1) { /* vector indexing */
 	SEXP thesub = CAR(subs);
-	int i = -1, len = length(thesub);
+	int len = length(thesub);
 
-	/* new case in 1.7.0, one vector index for a list */
-	if(isVectorList(x) && length(CAR(subs)) > 1) {
-	    for(i = 0; i < len - 1; i++) {
-		if(!isVectorList(x))
-		    errorcall(call, _("recursive indexing failed at level %d\n"), i+1);
-		offset = get1index(CAR(subs), getAttrib(x, R_NamesSymbol),
-				   length(x), pok, i, call);
-		if(offset < 0 || offset >= length(x))
-		    errorcall(call, _("no such index at level %d\n"), i+1);
-		x = VECTOR_ELT(x, offset);
-	    }
-	}
-	offset = get1index(CAR(subs), getAttrib(x, R_NamesSymbol),
-			   length(x), pok, i, call);
+	if (len > 1)
+	    x = vectorIndex(x, thesub, 0, len-1, pok, call);
+	    
+	offset = get1index(thesub, getAttrib(x, R_NamesSymbol),
+			   length(x), pok, len > 1 ? len-1 : -1, call);
 	if (offset < 0 || offset >= length(x)) {
 	    /* a bold attempt to get the same behaviour for $ and [[ */
 	    if (offset < 0 && (isNewList(x) ||
@@ -873,13 +889,13 @@ SEXP attribute_hidden do_subset2_dflt(SEXP call, SEXP op, SEXP args, SEXP rho)
 
     if(isPairList(x)) {
 	ans = CAR(nthcdr(x, offset));
-	if (NAMED(x) > NAMED(ans))
-	    SET_NAMED(ans, NAMED(x));
+	if (named_x > NAMED(ans))
+	    SET_NAMED(ans, named_x);
     } else if(isVectorList(x)) {
 	/* did unconditional duplication before 2.4.0 */
 	ans = VECTOR_ELT(x, offset);
-	if (NAMED(x) > NAMED(ans))
-	    SET_NAMED(ans, NAMED(x));
+	if (named_x > NAMED(ans))
+	    SET_NAMED(ans, named_x);
     } else {
 	ans = allocVector(TYPEOF(x), 1);
 	switch (TYPEOF(x)) {
@@ -998,6 +1014,12 @@ SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
 
     /* Optimisation to prevent repeated recalculation */
     slen = strlen(translateChar(input));
+     /* The mechanism to allow  a class extending "environment" */
+    if( IS_S4_OBJECT(x) && TYPEOF(x) == S4SXP ){
+        x = R_getS4DataSlot(x, ANYSXP);
+	if(x == R_NilValue)
+	    errorcall(call, "$ operator not defined for this S4 class");
+    }
 
     /* If this is not a list object we return NULL. */
     /* Or should this be allocVector(VECSXP, 0)? */
@@ -1110,9 +1132,8 @@ SEXP attribute_hidden R_subset3_dflt(SEXP x, SEXP input, SEXP call)
     else if( isVectorAtomic(x) ){
 	errorcall(call, "$ operator is invalid for atomic vectors");
     }
-    else if( IS_S4_OBJECT(x) ){
-	errorcall(call, "$ operator not defined for this S4 class");
-    }
+    else /* e.g. a function */
+	errorcall(call, R_MSG_ob_nonsub, type2char(TYPEOF(x)));
     UNPROTECT(2);
     return R_NilValue;
 }
