@@ -1,8 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997-2007   Robert Gentleman, Ross Ihaka and the
- *			      R Development Core Team
+ *  Copyright (C) 1997-2008   R Development Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,8 +14,8 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,7 +23,8 @@
 #endif
 
 #include <Defn.h>
-#include <Rmath.h>
+#define R_MSG_type	_("invalid 'type' (%s) of argument")
+#define imax2(x, y) ((x < y) ? y : x)
 
 #define R_INT_MIN	(1+INT_MIN)
 	/* since INT_MIN is the NA_INTEGER value ! */
@@ -41,7 +41,7 @@
 #endif
 
 
-static Rboolean isum(int *x, int n, int *value, Rboolean narm)
+static Rboolean isum(int *x, int n, int *value, Rboolean narm, SEXP call)
 {
     double s = 0.0;
     int i;
@@ -58,7 +58,7 @@ static Rboolean isum(int *x, int n, int *value, Rboolean narm)
 	}
     }
     if(s > INT_MAX || s < R_INT_MIN){
-	warning(_("Integer overflow; use sum(as.numeric(.))"));
+	warningcall(call, _("Integer overflow - use sum(as.numeric(.))"));
 	*value = NA_INTEGER;
     }
     else *value = s;
@@ -149,16 +149,6 @@ static Rboolean rmin(double *x, int n, double *value, Rboolean narm)
     return(updated);
 }
 
-#if defined(Win32) && defined(SUPPORT_UTF8)
-# define STRCOLL Rstrcoll
-#else
-# ifdef HAVE_STRCOLL
-#  define STRCOLL strcoll
-# else
-#  define STRCOLL strcmp
-# endif
-#endif
-
 static Rboolean smin(SEXP x, SEXP *value, Rboolean narm)
 {
     int i;
@@ -167,9 +157,8 @@ static Rboolean smin(SEXP x, SEXP *value, Rboolean narm)
 
     for (i = 0; i < length(x); i++) {
 	if (STRING_ELT(x, i) != NA_STRING) {
-	    if (!updated || 
-		STRCOLL(translateChar(s), 
-			translateChar(STRING_ELT(x, i))) > 0) {
+	    if (!updated ||
+		(s != STRING_ELT(x, i) && Scollate(s, STRING_ELT(x, i)) > 0)) {
 		s = STRING_ELT(x, i);
 		if(!updated) updated = TRUE;
 	    }
@@ -227,6 +216,7 @@ static Rboolean rmax(double *x, int n, double *value, Rboolean narm)
 
     return(updated);
 }
+
 static Rboolean smax(SEXP x, SEXP *value, Rboolean narm)
 {
     int i;
@@ -235,9 +225,8 @@ static Rboolean smax(SEXP x, SEXP *value, Rboolean narm)
 
     for (i = 0; i < length(x); i++) {
 	if (STRING_ELT(x, i) != NA_STRING) {
-	    if (!updated || 
-		STRCOLL(translateChar(s),
-			translateChar(STRING_ELT(x, i))) < 0) {
+	    if (!updated ||
+		(s != STRING_ELT(x, i) && Scollate(s, STRING_ELT(x, i)) < 0)) {
 		s = STRING_ELT(x, i);
 		if(!updated) updated = TRUE;
 	    }
@@ -319,6 +308,38 @@ static Rboolean cprod(Rcomplex *x, int n, Rcomplex *value, Rboolean narm)
 }
 
 
+SEXP fixup_NaRm(SEXP args)
+{
+    SEXP a, r, t, na_value, prev = R_NilValue;
+
+    /* Need to make sure na.rm is last and exists */
+    na_value = ScalarLogical(FALSE);
+    for(a = args ; a != R_NilValue; a = CDR(a)) {
+	if(TAG(a) == R_NaRmSymbol) {
+	    if(CDR(a) == R_NilValue) return args;
+	    na_value = CAR(a);
+	    if(prev == R_NilValue) args = CDR(a);
+	    else SETCDR(prev, CDR(a));
+	}
+	prev = a;
+    }
+
+    PROTECT(na_value);
+    t = CONS(na_value, R_NilValue);
+    UNPROTECT(1);
+    PROTECT(t);
+    SET_TAG(t, R_NaRmSymbol);
+    if (args == R_NilValue)
+	args = t;
+    else {
+	r = args;
+	while (CDR(r) != R_NilValue) r = CDR(r);
+	SETCDR(r, t);
+    }
+    UNPROTECT(1);
+    return args;
+}
+
 /* do_summary provides a variety of data summaries
 	op : 0 = sum, 1 = mean, 2 = min, 3 = max, 4 = prod
  */
@@ -329,7 +350,7 @@ static Rboolean cprod(Rcomplex *x, int n, Rcomplex *value, Rboolean narm)
 
 SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, a, stmp = NA_STRING /* -Wall */, scum = NA_STRING;
+    SEXP ans, a, stmp = NA_STRING /* -Wall */, scum = NA_STRING, call2;
     double tmp = 0.0, s;
     Rcomplex z, ztmp, zcum={0.0, 0.0} /* -Wall */;
     int itmp = 0, icum=0, int_a, real_a, empty, warn = 0 /* dummy */;
@@ -387,15 +408,22 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 	    COMPLEX(ans)[0].i = si;
 	    break;
 	default:
-	    errorcall(call, R_MSG_type, type2str(TYPEOF(x)));
+	    error(R_MSG_type, type2char(TYPEOF(x)));
 	}
 	UNPROTECT(1);
 	return ans;
     }
 
+    /* match to foo(..., na.rm=FALSE) */
+    PROTECT(args = fixup_NaRm(args));
+    PROTECT(call2 = duplicate(call));
+    SETCDR(call2, args);
 
-    if(DispatchGroup("Summary", call, op, args, env, &ans))
-	return ans;
+    if (DispatchGroup("Summary", call2, op, args, env, &ans)) {
+	UNPROTECT(2);
+	return(ans);
+    }
+    UNPROTECT(1);
 
 #ifdef DEBUG_Summary
     REprintf("C do_summary(op%s, *): did NOT dispatch\n", PRIMNAME(op));
@@ -458,7 +486,7 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 	a = CAR(args);
 	int_a = 0;/* int_a = 1	<-->	a is INTEGER */
 	real_a = 0;
-	
+
 	if(length(a) > 0) {
 	    updated = 0;/*- GLOBAL -*/
 
@@ -468,7 +496,7 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 
 		switch(TYPEOF(a)) {
 		case LGLSXP:
-		case INTSXP: 
+		case INTSXP:
 		    int_a = 1;
 		    if (iop == 2) updated = imin(INTEGER(a), length(a), &itmp, narm);
 		    else	  updated = imax(INTEGER(a), length(a), &itmp, narm);
@@ -518,10 +546,8 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 				stmp = StringFromInteger(itmp, &warn);
 			    if(real_a)
 				stmp = StringFromReal(tmp, &warn);
-			    if(((iop == 2 && 
-				 STRCOLL(translateChar(stmp), translateChar(scum)) < 0)) ||
-			       (iop == 3 && 
-				STRCOLL(translateChar(stmp), translateChar(scum)) > 0) )
+			    if(((iop == 2 && stmp != scum && Scollate(stmp, scum) < 0)) ||
+			       (iop == 3 && stmp != scum && Scollate(stmp, scum) > 0) )
 				scum = stmp;
 			}
 		    }
@@ -539,15 +565,15 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
 		switch(TYPEOF(a)) {
 		case LGLSXP:
 		case INTSXP:
-		    updated = isum(TYPEOF(a) == LGLSXP ? 
-                                   LOGICAL(a) :INTEGER(a), length(a),
-                                   &itmp, narm);
+		    updated = isum(TYPEOF(a) == LGLSXP ?
+				   LOGICAL(a) :INTEGER(a), length(a),
+				   &itmp, narm, call);
 		    if(updated) {
 			if(itmp == NA_INTEGER) goto na_answer;
 			if(ans_type == INTSXP) {
 			    s = (double) icum + (double) itmp;
 			    if(s > INT_MAX || s < R_INT_MIN){
-				warning(_("Integer overflow; use sum(as.numeric(.))"));
+				warningcall(call,_("Integer overflow - use sum(as.numeric(.))"));
 				goto na_answer;
 			    }
 			    else icum += itmp;
@@ -655,12 +681,12 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
     /*-------------------------------------------------------*/
     if(empty && (iop == 2 || iop == 3)) {
 	if(ans_type == STRSXP) {
-	    warning(_("no non-missing arguments, returning NA"));
+	    warningcall(call, _("no non-missing arguments, returning NA"));
 	} else {
 	    if(iop == 2)
-		warning(_("no non-missing arguments to min; returning Inf"));
+		warningcall(call, _("no non-missing arguments to min; returning Inf"));
 	    else
-		warning(_("no non-missing arguments to max; returning -Inf"));
+		warningcall(call, _("no non-missing arguments to max; returning -Inf"));
 	    ans_type = REALSXP;
 	}
     }
@@ -672,9 +698,10 @@ SEXP attribute_hidden do_summary(SEXP call, SEXP op, SEXP args, SEXP env)
     case CPLXSXP:  COMPLEX(ans)[0].r = zcum.r; COMPLEX(ans)[0].i = zcum.i;break;
     case STRSXP:   SET_STRING_ELT(ans, 0, scum); break;
     }
+    UNPROTECT(1);  /* args */
     return ans;
 
-na_answer: /* only INTSXP case curently used */
+na_answer: /* only INTSXP case currently used */
     ans = allocVector(ans_type, 1);
     switch(ans_type) {
     case INTSXP:	INTEGER(ans)[0] = NA_INTEGER; break;
@@ -682,6 +709,7 @@ na_answer: /* only INTSXP case curently used */
     case CPLXSXP:	COMPLEX(ans)[0].r = COMPLEX(ans)[0].i = NA_REAL; break;
     case STRSXP:        SET_STRING_ELT(ans, 0, NA_STRING); break;
     }
+    UNPROTECT(1);  /* args */
     return ans;
 
 invalid_type:
@@ -689,18 +717,27 @@ invalid_type:
     return R_NilValue;
 }/* do_summary */
 
+
 SEXP attribute_hidden do_range(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans, a, b, prargs;
+    SEXP ans, a, b, prargs, call2;
 
-    if (DispatchGroup("Summary", call, op, args, env, &ans))
+    PROTECT(args = fixup_NaRm(args));
+    PROTECT(call2 = duplicate(call));
+    SETCDR(call2, args);
+
+    if (DispatchGroup("Summary", call2, op, args, env, &ans)) {
+	UNPROTECT(2);
 	return(ans);
+    }
+    UNPROTECT(1);
+
     PROTECT(op = findFun(install("range.default"), env));
     PROTECT(prargs = promiseArgs(args, R_GlobalEnv));
     for (a = args, b = prargs; a != R_NilValue; a = CDR(a), b = CDR(b))
 	SET_PRVALUE(CAR(b), CAR(a));
     ans = applyClosure(call, op, prargs, env, R_BaseEnv);
-    UNPROTECT(2);
+    UNPROTECT(3);
     return(ans);
 }
 
@@ -714,7 +751,7 @@ SEXP attribute_hidden do_first_min(SEXP call, SEXP op, SEXP args, SEXP rho)
     checkArity(op, args);
     PROTECT(sx = coerceVector(CAR(args), REALSXP));
     if (!isNumeric(sx))
-	errorcall(call, _("non-numeric argument"));
+	error(_("non-numeric argument"));
     r = REAL(sx);
     n = LENGTH(sx);
     indx = NA_INTEGER;
@@ -739,9 +776,8 @@ SEXP attribute_hidden do_first_min(SEXP call, SEXP op, SEXP args, SEXP rho)
 	INTEGER(ans)[0] = indx + 1;
 	if (getAttrib(sx, R_NamesSymbol) != R_NilValue) { /* preserve names */
 	    SEXP ansnam;
-	    PROTECT(ansnam = allocVector(STRSXP, 1));
-	    SET_STRING_ELT(ansnam, 0,
-			   STRING_ELT(getAttrib(sx, R_NamesSymbol), indx));
+	    PROTECT(ansnam =
+		    ScalarString(STRING_ELT(getAttrib(sx, R_NamesSymbol), indx)));
 	    setAttrib(ans, R_NamesSymbol, ansnam);
 	    UNPROTECT(1);
 	}
@@ -761,7 +797,7 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
     len = -1;
 
     for (s = args; s != R_NilValue; s = CDR(s)) {
-	if (isList(CAR(s))/* || isFrame(CAR(s)) */) {
+	if (isList(CAR(s))) {
 	    for (t = CAR(s); t != R_NilValue; t = CDR(t))
 		if (isMatrix(CAR(t))) {
 		    u = getAttrib(CAR(t), R_DimSymbol);
@@ -777,32 +813,42 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
 			goto bad;
 		}
 		else
-		    errorcall(call, R_MSG_type, type2char(TYPEOF(CAR(t))));
+		    error(R_MSG_type, type2char(TYPEOF(CAR(t))));
 	}
 	/* FIXME : Need to be careful with the use of isVector() */
-	/* since this includes the new list structure and expressions. */
+	/* since this includes lists and expressions. */
 	else if (isNewList(CAR(s))) {
 	    int it, nt;
 	    t = CAR(s);
 	    nt = length(t);
-	    for (it = 0 ; it < nt ; it++) {
-		if (isMatrix(VECTOR_ELT(t, it))) {
-		    u = getAttrib(VECTOR_ELT(t, it), R_DimSymbol);
+	    /* 0-column data frames are a special case */
+	    if(nt) {
+		for (it = 0 ; it < nt ; it++) {
+		    if (isMatrix(VECTOR_ELT(t, it))) {
+			u = getAttrib(VECTOR_ELT(t, it), R_DimSymbol);
+			if (len < 0)
+			    len = INTEGER(u)[0];
+			else if (len != INTEGER(u)[0])
+			    goto bad;
+		    }
+		    else if (isVector(VECTOR_ELT(t, it))) {
+			if (len < 0)
+			    len = LENGTH(VECTOR_ELT(t, it));
+			else if (len != LENGTH(VECTOR_ELT(t, it)))
+			    goto bad;
+		    }
+		    else
+			error(R_MSG_type, "unknown");
+		}
+	    } else {
+		u = getAttrib(t, R_RowNamesSymbol);
+		if (!isNull(u)) {
 		    if (len < 0)
-			len = INTEGER(u)[0];
+			len = LENGTH(u);
 		    else if (len != INTEGER(u)[0])
 			goto bad;
 		}
-		else if (isVector(VECTOR_ELT(t, it))) {
-		    if (len < 0)
-			len = LENGTH(VECTOR_ELT(t, it));
-		    else if (len != LENGTH(VECTOR_ELT(t, it)))
-			goto bad;
-		}
-		else
-		    errorcall(call, R_MSG_type, "unknown");
 	    }
-
 	}
 	else if (isMatrix(CAR(s))) {
 	    u = getAttrib(CAR(s), R_DimSymbol);
@@ -818,15 +864,17 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
 		goto bad;
 	}
 	else
-	    errorcall(call, R_MSG_type, type2char(TYPEOF(CAR(s))));
+	    error(R_MSG_type, type2char(TYPEOF(CAR(s))));
     }
+
+    if (len < 0)
+	error(_("no input has determined the number of cases"));
     PROTECT(rval = allocVector(LGLSXP, len));
-    for (i = 0; i < len; i++)
-	INTEGER(rval)[i] = 1;
+    for (i = 0; i < len; i++) INTEGER(rval)[i] = 1;
     /* FIXME : there is a lot of shared code here for vectors. */
     /* It should be abstracted out and optimized. */
     for (s = args; s != R_NilValue; s = CDR(s)) {
-	if (isList(CAR(s)) /* || isFrame(CAR(s))*/) {
+	if (isList(CAR(s))) {
 	    /* Now we only need to worry about vectors */
 	    /* since we use mod to handle arrays. */
 	    /* FIXME : using mod like this causes */
@@ -854,7 +902,7 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
 			break;
 		    default:
 			UNPROTECT(1);
-			errorcall(call, R_MSG_type, type2char(TYPEOF(u)));
+			error(R_MSG_type, type2char(TYPEOF(u)));
 		    }
 		}
 	    }
@@ -886,7 +934,7 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
 			break;
 		    default:
 			UNPROTECT(1);
-			errorcall(call, R_MSG_type, type2char(TYPEOF(u)));
+			error(R_MSG_type, type2char(TYPEOF(u)));
 		    }
 		}
 	    }
@@ -914,7 +962,7 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
 		    break;
 		default:
 		    UNPROTECT(1);
-		    errorcall(call, R_MSG_type, type2char(TYPEOF(u)));
+		    error(R_MSG_type, type2char(TYPEOF(u)));
 		}
 	    }
 	}
@@ -923,26 +971,26 @@ SEXP attribute_hidden do_compcases(SEXP call, SEXP op, SEXP args, SEXP rho)
     return rval;
 
  bad:
-    errorcall(call, _("not all arguments have the same length"));
+    error(_("not all arguments have the same length"));
     return R_NilValue; /* -Wall */
 }
 
 /* op = 0 is pmin.int, op = 1 is pmax.int
-   It seems that NULL and logicals are supposed to be handled as 
-   if they have been coerced to integer. 
+   It seems that NULL and logicals are supposed to be handled as
+   if they have been coerced to integer.
  */
 SEXP attribute_hidden do_pmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 {
     SEXP a, x, ans;
     int i, n, len, narm;
     SEXPTYPE type, anstype;
-    
+
     narm = asLogical(CAR(args));
     if(narm == NA_LOGICAL)
-	errorcall(call, _("invalid '%s' value"), "na.rm");
+	error(_("invalid '%s' value"), "na.rm");
     args = CDR(args);
     x = CAR(args);
-    if(args == R_NilValue) errorcall(call, _("no arguments"));
+    if(args == R_NilValue) error(_("no arguments"));
 
     anstype = TYPEOF(x);
     switch(anstype) {
@@ -953,7 +1001,7 @@ SEXP attribute_hidden do_pmin(SEXP call, SEXP op, SEXP args, SEXP rho)
     case STRSXP:
 	break;
     default:
-	errorcall(call, _("invalid input type"));	
+	error(_("invalid input type"));
     }
     a = CDR(args);
     if(a == R_NilValue) return x; /* one input */
@@ -970,12 +1018,12 @@ SEXP attribute_hidden do_pmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 	case STRSXP:
 	    break;
 	default:
-	    errorcall(call, _("invalid input type"));
+	    error(_("invalid input type"));
 	}
 	if(type > anstype) anstype = type;
 	n = length(x);
 	if ((len > 0) ^ (n > 0))
-	    errorcall(call, _("cannot mix 0-length vectors with others"));
+	    error(_("cannot mix 0-length vectors with others"));
 	len = imax2(len, n);
     }
     if(anstype < INTSXP) anstype = INTSXP;
@@ -1000,13 +1048,13 @@ SEXP attribute_hidden do_pmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 		tmp = r[i % n];
 		if(PRIMVAL(op) == 1) {
 		    if( (narm && ra[i] == NA_INTEGER) ||
-			(ra[i] != NA_INTEGER && tmp != NA_INTEGER 
+			(ra[i] != NA_INTEGER && tmp != NA_INTEGER
 			 && tmp > ra[i]) ||
 			(!narm && tmp == NA_INTEGER) )
 			ra[i] = tmp;
 		} else {
 		    if( (narm && ra[i] == NA_INTEGER) ||
-			(ra[i] != NA_INTEGER && tmp != NA_INTEGER 
+			(ra[i] != NA_INTEGER && tmp != NA_INTEGER
 			 && tmp < ra[i]) ||
 			(!narm && tmp == NA_INTEGER) )
 			ra[i] = tmp;
@@ -1061,14 +1109,12 @@ SEXP attribute_hidden do_pmin(SEXP call, SEXP op, SEXP args, SEXP rho)
 		t2 = STRING_ELT(ans, i);
 		if(PRIMVAL(op) == 1) {
 		    if( (narm && t2 == NA_STRING) ||
-			(t2 != NA_STRING && tmp != NA_STRING 
-			 && STRCOLL(translateChar(tmp), translateChar(t2)) > 0) ||
+			(t2 != NA_STRING && tmp != NA_STRING && tmp != t2 && Scollate(tmp, t2) > 0) ||
 			(!narm && tmp == NA_STRING) )
 			SET_STRING_ELT(ans, i, tmp);
 		} else {
 		    if( (narm && t2 == NA_STRING) ||
-			(t2 != NA_STRING && tmp != NA_STRING 
-			 && STRCOLL(translateChar(tmp), translateChar(t2)) < 0) ||
+			(t2 != NA_STRING && tmp != NA_STRING && tmp != t2 && Scollate(tmp, t2) < 0) ||
 			(!narm && tmp == NA_STRING) )
 			SET_STRING_ELT(ans, i, tmp);
 		}
